/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/guest-ppc/insn/common.h"

static const uint8_t input[] = {
    0xFC,0x22,0x18,0x2A,  // fadd f1,f2,f3
};

static const unsigned int guest_opt = 0;
static const unsigned int common_opt = 0;

static const bool expected_success = true;

static const char expected[] =
    "[info] Scanning terminated at requested limit 0x3\n"
    "    0: LOAD_ARG   r1, 0\n"
    "    1: LOAD_ARG   r2, 1\n"
    "    2: GET_ALIAS  r3, a3\n"
    "    3: GET_ALIAS  r4, a4\n"
    "    4: FADD       r5, r3, r4\n"
    "    5: LOAD_IMM   r6, 0x20000000000000\n"
    "    6: BITCAST    r7, r5\n"
    "    7: SLLI       r8, r7, 1\n"
    "    8: SEQ        r9, r8, r6\n"
    "    9: GOTO_IF_Z  r9, L1\n"
    "   10: FGETSTATE  r10\n"
    "   11: FSETROUND  r11, r10, TRUNC\n"
    "   12: FSETSTATE  r11\n"
    "   13: FADD       r12, r3, r4\n"
    "   14: FGETSTATE  r13\n"
    "   15: FCOPYROUND r14, r13, r10\n"
    "   16: FSETSTATE  r14\n"
    "   17: LABEL      L1\n"
    "   18: GET_ALIAS  r15, a5\n"
    "   19: FGETSTATE  r16\n"
    "   20: FCLEAREXC  r17, r16\n"
    "   21: FSETSTATE  r17\n"
    "   22: FTESTEXC   r18, r16, INVALID\n"
    "   23: GOTO_IF_Z  r18, L3\n"
    "   24: BITCAST    r19, r3\n"
    "   25: SLLI       r20, r19, 13\n"
    "   26: BFEXT      r21, r19, 51, 12\n"
    "   27: SEQI       r22, r21, 4094\n"
    "   28: GOTO_IF_Z  r22, L5\n"
    "   29: GOTO_IF_NZ r20, L4\n"
    "   30: LABEL      L5\n"
    "   31: BITCAST    r23, r4\n"
    "   32: SLLI       r24, r23, 13\n"
    "   33: BFEXT      r25, r23, 51, 12\n"
    "   34: SEQI       r26, r25, 4094\n"
    "   35: GOTO_IF_Z  r26, L6\n"
    "   36: GOTO_IF_NZ r24, L4\n"
    "   37: LABEL      L6\n"
    "   38: NOT        r27, r15\n"
    "   39: ORI        r28, r15, 8388608\n"
    "   40: ANDI       r29, r27, 8388608\n"
    "   41: SET_ALIAS  a5, r28\n"
    "   42: GOTO_IF_Z  r29, L7\n"
    "   43: ORI        r30, r28, -2147483648\n"
    "   44: SET_ALIAS  a5, r30\n"
    "   45: LABEL      L7\n"
    "   46: ANDI       r31, r15, 128\n"
    "   47: GOTO_IF_Z  r31, L8\n"
    "   48: GET_ALIAS  r32, a5\n"
    "   49: BFEXT      r33, r32, 12, 7\n"
    "   50: ANDI       r34, r33, 31\n"
    "   51: GET_ALIAS  r35, a5\n"
    "   52: BFINS      r36, r35, r34, 12, 7\n"
    "   53: SET_ALIAS  a5, r36\n"
    "   54: GOTO       L2\n"
    "   55: LABEL      L8\n"
    "   56: LOAD_IMM   r37, nan(0x8000000000000)\n"
    "   57: SET_ALIAS  a2, r37\n"
    "   58: LOAD_IMM   r38, 17\n"
    "   59: GET_ALIAS  r39, a5\n"
    "   60: BFINS      r40, r39, r38, 12, 7\n"
    "   61: SET_ALIAS  a5, r40\n"
    "   62: GOTO       L2\n"
    "   63: LABEL      L4\n"
    "   64: NOT        r41, r15\n"
    "   65: ORI        r42, r15, 16777216\n"
    "   66: ANDI       r43, r41, 16777216\n"
    "   67: SET_ALIAS  a5, r42\n"
    "   68: GOTO_IF_Z  r43, L9\n"
    "   69: ORI        r44, r42, -2147483648\n"
    "   70: SET_ALIAS  a5, r44\n"
    "   71: LABEL      L9\n"
    "   72: ANDI       r45, r15, 128\n"
    "   73: GOTO_IF_Z  r45, L3\n"
    "   74: GET_ALIAS  r46, a5\n"
    "   75: BFEXT      r47, r46, 12, 7\n"
    "   76: ANDI       r48, r47, 31\n"
    "   77: GET_ALIAS  r49, a5\n"
    "   78: BFINS      r50, r49, r48, 12, 7\n"
    "   79: SET_ALIAS  a5, r50\n"
    "   80: GOTO       L2\n"
    "   81: LABEL      L3\n"
    "   82: SET_ALIAS  a2, r5\n"
    "   83: BITCAST    r51, r5\n"
    "   84: SGTUI      r52, r51, 0\n"
    "   85: SLTSI      r53, r51, 0\n"
    "   86: BFEXT      r57, r51, 52, 11\n"
    "   87: SEQI       r54, r57, 0\n"
    "   88: SEQI       r55, r57, 2047\n"
    "   89: SLLI       r58, r51, 12\n"
    "   90: SEQI       r56, r58, 0\n"
    "   91: AND        r59, r54, r56\n"
    "   92: XORI       r60, r56, 1\n"
    "   93: AND        r61, r55, r60\n"
    "   94: AND        r62, r54, r52\n"
    "   95: OR         r63, r62, r61\n"
    "   96: OR         r64, r59, r61\n"
    "   97: XORI       r65, r64, 1\n"
    "   98: XORI       r66, r53, 1\n"
    "   99: AND        r67, r53, r65\n"
    "  100: AND        r68, r66, r65\n"
    "  101: SLLI       r69, r63, 4\n"
    "  102: SLLI       r70, r67, 3\n"
    "  103: SLLI       r71, r68, 2\n"
    "  104: SLLI       r72, r59, 1\n"
    "  105: OR         r73, r69, r70\n"
    "  106: OR         r74, r71, r72\n"
    "  107: OR         r75, r73, r55\n"
    "  108: OR         r76, r75, r74\n"
    "  109: FTESTEXC   r77, r16, INEXACT\n"
    "  110: SLLI       r78, r77, 5\n"
    "  111: OR         r79, r76, r78\n"
    "  112: GET_ALIAS  r80, a5\n"
    "  113: BFINS      r81, r80, r79, 12, 7\n"
    "  114: SET_ALIAS  a5, r81\n"
    "  115: GOTO_IF_Z  r77, L10\n"
    "  116: GET_ALIAS  r82, a5\n"
    "  117: NOT        r83, r82\n"
    "  118: ORI        r84, r82, 33554432\n"
    "  119: ANDI       r85, r83, 33554432\n"
    "  120: SET_ALIAS  a5, r84\n"
    "  121: GOTO_IF_Z  r85, L11\n"
    "  122: ORI        r86, r84, -2147483648\n"
    "  123: SET_ALIAS  a5, r86\n"
    "  124: LABEL      L11\n"
    "  125: LABEL      L10\n"
    "  126: FTESTEXC   r87, r16, OVERFLOW\n"
    "  127: GOTO_IF_Z  r87, L12\n"
    "  128: GET_ALIAS  r88, a5\n"
    "  129: NOT        r89, r88\n"
    "  130: ORI        r90, r88, 268435456\n"
    "  131: ANDI       r91, r89, 268435456\n"
    "  132: SET_ALIAS  a5, r90\n"
    "  133: GOTO_IF_Z  r91, L13\n"
    "  134: ORI        r92, r90, -2147483648\n"
    "  135: SET_ALIAS  a5, r92\n"
    "  136: LABEL      L13\n"
    "  137: LABEL      L12\n"
    "  138: FTESTEXC   r93, r16, UNDERFLOW\n"
    "  139: GOTO_IF_Z  r93, L2\n"
    "  140: GET_ALIAS  r94, a5\n"
    "  141: NOT        r95, r94\n"
    "  142: ORI        r96, r94, 134217728\n"
    "  143: ANDI       r97, r95, 134217728\n"
    "  144: SET_ALIAS  a5, r96\n"
    "  145: GOTO_IF_Z  r97, L14\n"
    "  146: ORI        r98, r96, -2147483648\n"
    "  147: SET_ALIAS  a5, r98\n"
    "  148: LABEL      L14\n"
    "  149: LABEL      L2\n"
    "  150: LOAD_IMM   r99, 4\n"
    "  151: SET_ALIAS  a1, r99\n"
    "  152: RETURN     r1\n"
    "\n"
    "Alias 1: int32 @ 964(r1)\n"
    "Alias 2: float64 @ 400(r1)\n"
    "Alias 3: float64 @ 416(r1)\n"
    "Alias 4: float64 @ 432(r1)\n"
    "Alias 5: int32 @ 944(r1)\n"
    "\n"
    "Block 0: <none> --> [0,9] --> 1,2\n"
    "Block 1: 0 --> [10,16] --> 2\n"
    "Block 2: 1,0 --> [17,23] --> 3,16\n"
    "Block 3: 2 --> [24,28] --> 4,5\n"
    "Block 4: 3 --> [29,29] --> 5,12\n"
    "Block 5: 4,3 --> [30,35] --> 6,7\n"
    "Block 6: 5 --> [36,36] --> 7,12\n"
    "Block 7: 6,5 --> [37,42] --> 8,9\n"
    "Block 8: 7 --> [43,44] --> 9\n"
    "Block 9: 8,7 --> [45,47] --> 10,11\n"
    "Block 10: 9 --> [48,54] --> 28\n"
    "Block 11: 9 --> [55,62] --> 28\n"
    "Block 12: 4,6 --> [63,68] --> 13,14\n"
    "Block 13: 12 --> [69,70] --> 14\n"
    "Block 14: 13,12 --> [71,73] --> 15,16\n"
    "Block 15: 14 --> [74,80] --> 28\n"
    "Block 16: 2,14 --> [81,115] --> 17,20\n"
    "Block 17: 16 --> [116,121] --> 18,19\n"
    "Block 18: 17 --> [122,123] --> 19\n"
    "Block 19: 18,17 --> [124,124] --> 20\n"
    "Block 20: 19,16 --> [125,127] --> 21,24\n"
    "Block 21: 20 --> [128,133] --> 22,23\n"
    "Block 22: 21 --> [134,135] --> 23\n"
    "Block 23: 22,21 --> [136,136] --> 24\n"
    "Block 24: 23,20 --> [137,139] --> 25,28\n"
    "Block 25: 24 --> [140,145] --> 26,27\n"
    "Block 26: 25 --> [146,147] --> 27\n"
    "Block 27: 26,25 --> [148,148] --> 28\n"
    "Block 28: 27,10,11,15,24 --> [149,152] --> <none>\n"
    ;

#include "tests/rtl-disasm-test.i"
