/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/guest-ppc/insn/common.h"

static const uint8_t input[] = {
    0xFC,0x22,0x18,0x2A,  // fadd f1,f2,f3
};

static const unsigned int guest_opt = BINREC_OPT_G_PPC_IGNORE_FPSCR_VXFOO;
static const unsigned int common_opt = 0;

static const bool expected_success = true;

static const char expected[] =
    "[info] Scanning terminated at requested limit 0x3\n"
    "    0: LOAD_ARG   r1, 0\n"
    "    1: LOAD_ARG   r2, 1\n"
    "    2: GET_ALIAS  r3, a3\n"
    "    3: GET_ALIAS  r4, a4\n"
    "    4: FADD       r5, r3, r4\n"
    "    5: LOAD_IMM   r6, 0x20000000000000\n"
    "    6: BITCAST    r7, r5\n"
    "    7: SLLI       r8, r7, 1\n"
    "    8: SEQ        r9, r8, r6\n"
    "    9: GOTO_IF_Z  r9, L1\n"
    "   10: FGETSTATE  r10\n"
    "   11: FSETROUND  r11, r10, TRUNC\n"
    "   12: FSETSTATE  r11\n"
    "   13: FADD       r12, r3, r4\n"
    "   14: FGETSTATE  r13\n"
    "   15: FCOPYROUND r14, r13, r10\n"
    "   16: FSETSTATE  r14\n"
    "   17: LABEL      L1\n"
    "   18: GET_ALIAS  r15, a5\n"
    "   19: FGETSTATE  r16\n"
    "   20: FCLEAREXC  r17, r16\n"
    "   21: FSETSTATE  r17\n"
    "   22: FTESTEXC   r18, r16, INVALID\n"
    "   23: GOTO_IF_Z  r18, L3\n"
    "   24: NOT        r19, r15\n"
    "   25: ORI        r20, r15, 16777216\n"
    "   26: ANDI       r21, r19, 16777216\n"
    "   27: SET_ALIAS  a5, r20\n"
    "   28: GOTO_IF_Z  r21, L4\n"
    "   29: ORI        r22, r20, -2147483648\n"
    "   30: SET_ALIAS  a5, r22\n"
    "   31: LABEL      L4\n"
    "   32: ANDI       r23, r15, 128\n"
    "   33: GOTO_IF_Z  r23, L3\n"
    "   34: GET_ALIAS  r24, a5\n"
    "   35: BFEXT      r25, r24, 12, 7\n"
    "   36: ANDI       r26, r25, 31\n"
    "   37: GET_ALIAS  r27, a5\n"
    "   38: BFINS      r28, r27, r26, 12, 7\n"
    "   39: SET_ALIAS  a5, r28\n"
    "   40: GOTO       L2\n"
    "   41: LABEL      L3\n"
    "   42: SET_ALIAS  a2, r5\n"
    "   43: BITCAST    r29, r5\n"
    "   44: SGTUI      r30, r29, 0\n"
    "   45: SLTSI      r31, r29, 0\n"
    "   46: BFEXT      r35, r29, 52, 11\n"
    "   47: SEQI       r32, r35, 0\n"
    "   48: SEQI       r33, r35, 2047\n"
    "   49: SLLI       r36, r29, 12\n"
    "   50: SEQI       r34, r36, 0\n"
    "   51: AND        r37, r32, r34\n"
    "   52: XORI       r38, r34, 1\n"
    "   53: AND        r39, r33, r38\n"
    "   54: AND        r40, r32, r30\n"
    "   55: OR         r41, r40, r39\n"
    "   56: OR         r42, r37, r39\n"
    "   57: XORI       r43, r42, 1\n"
    "   58: XORI       r44, r31, 1\n"
    "   59: AND        r45, r31, r43\n"
    "   60: AND        r46, r44, r43\n"
    "   61: SLLI       r47, r41, 4\n"
    "   62: SLLI       r48, r45, 3\n"
    "   63: SLLI       r49, r46, 2\n"
    "   64: SLLI       r50, r37, 1\n"
    "   65: OR         r51, r47, r48\n"
    "   66: OR         r52, r49, r50\n"
    "   67: OR         r53, r51, r33\n"
    "   68: OR         r54, r53, r52\n"
    "   69: FTESTEXC   r55, r16, INEXACT\n"
    "   70: SLLI       r56, r55, 5\n"
    "   71: OR         r57, r54, r56\n"
    "   72: GET_ALIAS  r58, a5\n"
    "   73: BFINS      r59, r58, r57, 12, 7\n"
    "   74: SET_ALIAS  a5, r59\n"
    "   75: GOTO_IF_Z  r55, L5\n"
    "   76: GET_ALIAS  r60, a5\n"
    "   77: NOT        r61, r60\n"
    "   78: ORI        r62, r60, 33554432\n"
    "   79: ANDI       r63, r61, 33554432\n"
    "   80: SET_ALIAS  a5, r62\n"
    "   81: GOTO_IF_Z  r63, L6\n"
    "   82: ORI        r64, r62, -2147483648\n"
    "   83: SET_ALIAS  a5, r64\n"
    "   84: LABEL      L6\n"
    "   85: LABEL      L5\n"
    "   86: FTESTEXC   r65, r16, OVERFLOW\n"
    "   87: GOTO_IF_Z  r65, L7\n"
    "   88: GET_ALIAS  r66, a5\n"
    "   89: NOT        r67, r66\n"
    "   90: ORI        r68, r66, 268435456\n"
    "   91: ANDI       r69, r67, 268435456\n"
    "   92: SET_ALIAS  a5, r68\n"
    "   93: GOTO_IF_Z  r69, L8\n"
    "   94: ORI        r70, r68, -2147483648\n"
    "   95: SET_ALIAS  a5, r70\n"
    "   96: LABEL      L8\n"
    "   97: LABEL      L7\n"
    "   98: FTESTEXC   r71, r16, UNDERFLOW\n"
    "   99: GOTO_IF_Z  r71, L2\n"
    "  100: GET_ALIAS  r72, a5\n"
    "  101: NOT        r73, r72\n"
    "  102: ORI        r74, r72, 134217728\n"
    "  103: ANDI       r75, r73, 134217728\n"
    "  104: SET_ALIAS  a5, r74\n"
    "  105: GOTO_IF_Z  r75, L9\n"
    "  106: ORI        r76, r74, -2147483648\n"
    "  107: SET_ALIAS  a5, r76\n"
    "  108: LABEL      L9\n"
    "  109: LABEL      L2\n"
    "  110: LOAD_IMM   r77, 4\n"
    "  111: SET_ALIAS  a1, r77\n"
    "  112: RETURN     r1\n"
    "\n"
    "Alias 1: int32 @ 964(r1)\n"
    "Alias 2: float64 @ 400(r1)\n"
    "Alias 3: float64 @ 416(r1)\n"
    "Alias 4: float64 @ 432(r1)\n"
    "Alias 5: int32 @ 944(r1)\n"
    "\n"
    "Block 0: <none> --> [0,9] --> 1,2\n"
    "Block 1: 0 --> [10,16] --> 2\n"
    "Block 2: 1,0 --> [17,23] --> 3,7\n"
    "Block 3: 2 --> [24,28] --> 4,5\n"
    "Block 4: 3 --> [29,30] --> 5\n"
    "Block 5: 4,3 --> [31,33] --> 6,7\n"
    "Block 6: 5 --> [34,40] --> 19\n"
    "Block 7: 2,5 --> [41,75] --> 8,11\n"
    "Block 8: 7 --> [76,81] --> 9,10\n"
    "Block 9: 8 --> [82,83] --> 10\n"
    "Block 10: 9,8 --> [84,84] --> 11\n"
    "Block 11: 10,7 --> [85,87] --> 12,15\n"
    "Block 12: 11 --> [88,93] --> 13,14\n"
    "Block 13: 12 --> [94,95] --> 14\n"
    "Block 14: 13,12 --> [96,96] --> 15\n"
    "Block 15: 14,11 --> [97,99] --> 16,19\n"
    "Block 16: 15 --> [100,105] --> 17,18\n"
    "Block 17: 16 --> [106,107] --> 18\n"
    "Block 18: 17,16 --> [108,108] --> 19\n"
    "Block 19: 18,6,15 --> [109,112] --> <none>\n"
    ;

#include "tests/rtl-disasm-test.i"
