/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/guest-ppc/insn/common.h"

static const uint8_t input[] = {
    0xFC,0x22,0x18,0x2A,  // fadd f1,f2,f3
};

static const unsigned int guest_opt = 0;
static const unsigned int common_opt = BINREC_OPT_NATIVE_IEEE_UNDERFLOW;

static const bool expected_success = true;

static const char expected[] =
    "[info] Scanning terminated at requested limit 0x3\n"
    "    0: LOAD_ARG   r1, 0\n"
    "    1: LOAD_ARG   r2, 1\n"
    "    2: GET_ALIAS  r3, a3\n"
    "    3: GET_ALIAS  r4, a4\n"
    "    4: FADD       r5, r3, r4\n"
    "    5: GET_ALIAS  r6, a5\n"
    "    6: FGETSTATE  r7\n"
    "    7: FCLEAREXC  r8, r7\n"
    "    8: FSETSTATE  r8\n"
    "    9: FTESTEXC   r9, r7, INVALID\n"
    "   10: GOTO_IF_Z  r9, L2\n"
    "   11: BITCAST    r10, r3\n"
    "   12: SLLI       r11, r10, 13\n"
    "   13: BFEXT      r12, r10, 51, 12\n"
    "   14: SEQI       r13, r12, 4094\n"
    "   15: GOTO_IF_Z  r13, L4\n"
    "   16: GOTO_IF_NZ r11, L3\n"
    "   17: LABEL      L4\n"
    "   18: BITCAST    r14, r4\n"
    "   19: SLLI       r15, r14, 13\n"
    "   20: BFEXT      r16, r14, 51, 12\n"
    "   21: SEQI       r17, r16, 4094\n"
    "   22: GOTO_IF_Z  r17, L5\n"
    "   23: GOTO_IF_NZ r15, L3\n"
    "   24: LABEL      L5\n"
    "   25: NOT        r18, r6\n"
    "   26: ORI        r19, r6, 8388608\n"
    "   27: ANDI       r20, r18, 8388608\n"
    "   28: SET_ALIAS  a5, r19\n"
    "   29: GOTO_IF_Z  r20, L6\n"
    "   30: ORI        r21, r19, -2147483648\n"
    "   31: SET_ALIAS  a5, r21\n"
    "   32: LABEL      L6\n"
    "   33: ANDI       r22, r6, 128\n"
    "   34: GOTO_IF_Z  r22, L7\n"
    "   35: GET_ALIAS  r23, a5\n"
    "   36: BFEXT      r24, r23, 12, 7\n"
    "   37: ANDI       r25, r24, 31\n"
    "   38: GET_ALIAS  r26, a5\n"
    "   39: BFINS      r27, r26, r25, 12, 7\n"
    "   40: SET_ALIAS  a5, r27\n"
    "   41: GOTO       L1\n"
    "   42: LABEL      L7\n"
    "   43: LOAD_IMM   r28, nan(0x8000000000000)\n"
    "   44: SET_ALIAS  a2, r28\n"
    "   45: LOAD_IMM   r29, 17\n"
    "   46: GET_ALIAS  r30, a5\n"
    "   47: BFINS      r31, r30, r29, 12, 7\n"
    "   48: SET_ALIAS  a5, r31\n"
    "   49: GOTO       L1\n"
    "   50: LABEL      L3\n"
    "   51: NOT        r32, r6\n"
    "   52: ORI        r33, r6, 16777216\n"
    "   53: ANDI       r34, r32, 16777216\n"
    "   54: SET_ALIAS  a5, r33\n"
    "   55: GOTO_IF_Z  r34, L8\n"
    "   56: ORI        r35, r33, -2147483648\n"
    "   57: SET_ALIAS  a5, r35\n"
    "   58: LABEL      L8\n"
    "   59: ANDI       r36, r6, 128\n"
    "   60: GOTO_IF_Z  r36, L2\n"
    "   61: GET_ALIAS  r37, a5\n"
    "   62: BFEXT      r38, r37, 12, 7\n"
    "   63: ANDI       r39, r38, 31\n"
    "   64: GET_ALIAS  r40, a5\n"
    "   65: BFINS      r41, r40, r39, 12, 7\n"
    "   66: SET_ALIAS  a5, r41\n"
    "   67: GOTO       L1\n"
    "   68: LABEL      L2\n"
    "   69: SET_ALIAS  a2, r5\n"
    "   70: BITCAST    r42, r5\n"
    "   71: SGTUI      r43, r42, 0\n"
    "   72: SLTSI      r44, r42, 0\n"
    "   73: BFEXT      r48, r42, 52, 11\n"
    "   74: SEQI       r45, r48, 0\n"
    "   75: SEQI       r46, r48, 2047\n"
    "   76: SLLI       r49, r42, 12\n"
    "   77: SEQI       r47, r49, 0\n"
    "   78: AND        r50, r45, r47\n"
    "   79: XORI       r51, r47, 1\n"
    "   80: AND        r52, r46, r51\n"
    "   81: AND        r53, r45, r43\n"
    "   82: OR         r54, r53, r52\n"
    "   83: OR         r55, r50, r52\n"
    "   84: XORI       r56, r55, 1\n"
    "   85: XORI       r57, r44, 1\n"
    "   86: AND        r58, r44, r56\n"
    "   87: AND        r59, r57, r56\n"
    "   88: SLLI       r60, r54, 4\n"
    "   89: SLLI       r61, r58, 3\n"
    "   90: SLLI       r62, r59, 2\n"
    "   91: SLLI       r63, r50, 1\n"
    "   92: OR         r64, r60, r61\n"
    "   93: OR         r65, r62, r63\n"
    "   94: OR         r66, r64, r46\n"
    "   95: OR         r67, r66, r65\n"
    "   96: FTESTEXC   r68, r7, INEXACT\n"
    "   97: SLLI       r69, r68, 5\n"
    "   98: OR         r70, r67, r69\n"
    "   99: GET_ALIAS  r71, a5\n"
    "  100: BFINS      r72, r71, r70, 12, 7\n"
    "  101: SET_ALIAS  a5, r72\n"
    "  102: GOTO_IF_Z  r68, L9\n"
    "  103: GET_ALIAS  r73, a5\n"
    "  104: NOT        r74, r73\n"
    "  105: ORI        r75, r73, 33554432\n"
    "  106: ANDI       r76, r74, 33554432\n"
    "  107: SET_ALIAS  a5, r75\n"
    "  108: GOTO_IF_Z  r76, L10\n"
    "  109: ORI        r77, r75, -2147483648\n"
    "  110: SET_ALIAS  a5, r77\n"
    "  111: LABEL      L10\n"
    "  112: LABEL      L9\n"
    "  113: FTESTEXC   r78, r7, OVERFLOW\n"
    "  114: GOTO_IF_Z  r78, L11\n"
    "  115: GET_ALIAS  r79, a5\n"
    "  116: NOT        r80, r79\n"
    "  117: ORI        r81, r79, 268435456\n"
    "  118: ANDI       r82, r80, 268435456\n"
    "  119: SET_ALIAS  a5, r81\n"
    "  120: GOTO_IF_Z  r82, L12\n"
    "  121: ORI        r83, r81, -2147483648\n"
    "  122: SET_ALIAS  a5, r83\n"
    "  123: LABEL      L12\n"
    "  124: LABEL      L11\n"
    "  125: FTESTEXC   r84, r7, UNDERFLOW\n"
    "  126: GOTO_IF_Z  r84, L1\n"
    "  127: GET_ALIAS  r85, a5\n"
    "  128: NOT        r86, r85\n"
    "  129: ORI        r87, r85, 134217728\n"
    "  130: ANDI       r88, r86, 134217728\n"
    "  131: SET_ALIAS  a5, r87\n"
    "  132: GOTO_IF_Z  r88, L13\n"
    "  133: ORI        r89, r87, -2147483648\n"
    "  134: SET_ALIAS  a5, r89\n"
    "  135: LABEL      L13\n"
    "  136: LABEL      L1\n"
    "  137: LOAD_IMM   r90, 4\n"
    "  138: SET_ALIAS  a1, r90\n"
    "  139: RETURN     r1\n"
    "\n"
    "Alias 1: int32 @ 964(r1)\n"
    "Alias 2: float64 @ 400(r1)\n"
    "Alias 3: float64 @ 416(r1)\n"
    "Alias 4: float64 @ 432(r1)\n"
    "Alias 5: int32 @ 944(r1)\n"
    "\n"
    "Block 0: <none> --> [0,10] --> 1,14\n"
    "Block 1: 0 --> [11,15] --> 2,3\n"
    "Block 2: 1 --> [16,16] --> 3,10\n"
    "Block 3: 2,1 --> [17,22] --> 4,5\n"
    "Block 4: 3 --> [23,23] --> 5,10\n"
    "Block 5: 4,3 --> [24,29] --> 6,7\n"
    "Block 6: 5 --> [30,31] --> 7\n"
    "Block 7: 6,5 --> [32,34] --> 8,9\n"
    "Block 8: 7 --> [35,41] --> 26\n"
    "Block 9: 7 --> [42,49] --> 26\n"
    "Block 10: 2,4 --> [50,55] --> 11,12\n"
    "Block 11: 10 --> [56,57] --> 12\n"
    "Block 12: 11,10 --> [58,60] --> 13,14\n"
    "Block 13: 12 --> [61,67] --> 26\n"
    "Block 14: 0,12 --> [68,102] --> 15,18\n"
    "Block 15: 14 --> [103,108] --> 16,17\n"
    "Block 16: 15 --> [109,110] --> 17\n"
    "Block 17: 16,15 --> [111,111] --> 18\n"
    "Block 18: 17,14 --> [112,114] --> 19,22\n"
    "Block 19: 18 --> [115,120] --> 20,21\n"
    "Block 20: 19 --> [121,122] --> 21\n"
    "Block 21: 20,19 --> [123,123] --> 22\n"
    "Block 22: 21,18 --> [124,126] --> 23,26\n"
    "Block 23: 22 --> [127,132] --> 24,25\n"
    "Block 24: 23 --> [133,134] --> 25\n"
    "Block 25: 24,23 --> [135,135] --> 26\n"
    "Block 26: 25,8,9,13,22 --> [136,139] --> <none>\n"
    ;

#include "tests/rtl-disasm-test.i"
