/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/guest-ppc/insn/common.h"

static const uint8_t input[] = {
    0xFC,0x20,0x10,0x1C,  // fctiw f1,f2
};

static const unsigned int guest_opt = BINREC_OPT_G_PPC_IGNORE_FPSCR_VXFOO
                                    | BINREC_OPT_G_PPC_FAST_FCTIW;
static const unsigned int common_opt = 0;

static const bool expected_success = true;

static const char expected[] =
    "[info] Scanning terminated at requested limit 0x3\n"
    "    0: LOAD_ARG   r1, 0\n"
    "    1: LOAD_ARG   r2, 1\n"
    "    2: GET_ALIAS  r3, a3\n"
    "    3: FROUNDI    r4, r3\n"
    "    4: LOAD_IMM   r5, 2147483647.0\n"
    "    5: FCMP       r6, r3, r5, GE\n"
    "    6: LOAD_IMM   r7, 0x7FFFFFFF\n"
    "    7: SELECT     r8, r7, r4, r6\n"
    "    8: ZCAST      r9, r8\n"
    "    9: BITCAST    r10, r9\n"
    "   10: GET_ALIAS  r11, a4\n"
    "   11: FGETSTATE  r12\n"
    "   12: FCLEAREXC  r13, r12\n"
    "   13: FSETSTATE  r13\n"
    "   14: FTESTEXC   r14, r12, INVALID\n"
    "   15: GOTO_IF_Z  r14, L2\n"
    "   16: NOT        r15, r11\n"
    "   17: ORI        r16, r11, 16777472\n"
    "   18: ANDI       r17, r15, 16777472\n"
    "   19: SET_ALIAS  a4, r16\n"
    "   20: GOTO_IF_Z  r17, L3\n"
    "   21: ORI        r18, r16, -2147483648\n"
    "   22: SET_ALIAS  a4, r18\n"
    "   23: LABEL      L3\n"
    "   24: ANDI       r19, r11, 128\n"
    "   25: GOTO_IF_Z  r19, L2\n"
    "   26: GET_ALIAS  r20, a4\n"
    "   27: BFEXT      r21, r20, 12, 7\n"
    "   28: ANDI       r22, r21, 31\n"
    "   29: GET_ALIAS  r23, a4\n"
    "   30: BFINS      r24, r23, r22, 12, 7\n"
    "   31: SET_ALIAS  a4, r24\n"
    "   32: GOTO       L1\n"
    "   33: LABEL      L2\n"
    "   34: SET_ALIAS  a2, r10\n"
    "   35: BITCAST    r25, r10\n"
    "   36: SGTUI      r26, r25, 0\n"
    "   37: SLTSI      r27, r25, 0\n"
    "   38: BFEXT      r31, r25, 52, 11\n"
    "   39: SEQI       r28, r31, 0\n"
    "   40: SEQI       r29, r31, 2047\n"
    "   41: SLLI       r32, r25, 12\n"
    "   42: SEQI       r30, r32, 0\n"
    "   43: AND        r33, r28, r30\n"
    "   44: XORI       r34, r30, 1\n"
    "   45: AND        r35, r29, r34\n"
    "   46: AND        r36, r28, r26\n"
    "   47: OR         r37, r36, r35\n"
    "   48: OR         r38, r33, r35\n"
    "   49: XORI       r39, r38, 1\n"
    "   50: XORI       r40, r27, 1\n"
    "   51: AND        r41, r27, r39\n"
    "   52: AND        r42, r40, r39\n"
    "   53: SLLI       r43, r37, 4\n"
    "   54: SLLI       r44, r41, 3\n"
    "   55: SLLI       r45, r42, 2\n"
    "   56: SLLI       r46, r33, 1\n"
    "   57: OR         r47, r43, r44\n"
    "   58: OR         r48, r45, r46\n"
    "   59: OR         r49, r47, r29\n"
    "   60: OR         r50, r49, r48\n"
    "   61: FTESTEXC   r51, r12, INEXACT\n"
    "   62: SLLI       r52, r51, 5\n"
    "   63: OR         r53, r50, r52\n"
    "   64: GET_ALIAS  r54, a4\n"
    "   65: BFINS      r55, r54, r53, 12, 7\n"
    "   66: SET_ALIAS  a4, r55\n"
    "   67: GOTO_IF_Z  r51, L4\n"
    "   68: GET_ALIAS  r56, a4\n"
    "   69: NOT        r57, r56\n"
    "   70: ORI        r58, r56, 33554432\n"
    "   71: ANDI       r59, r57, 33554432\n"
    "   72: SET_ALIAS  a4, r58\n"
    "   73: GOTO_IF_Z  r59, L5\n"
    "   74: ORI        r60, r58, -2147483648\n"
    "   75: SET_ALIAS  a4, r60\n"
    "   76: LABEL      L5\n"
    "   77: LABEL      L4\n"
    "   78: LABEL      L1\n"
    "   79: LOAD_IMM   r61, 4\n"
    "   80: SET_ALIAS  a1, r61\n"
    "   81: RETURN     r1\n"
    "\n"
    "Alias 1: int32 @ 964(r1)\n"
    "Alias 2: float64 @ 400(r1)\n"
    "Alias 3: float64 @ 416(r1)\n"
    "Alias 4: int32 @ 944(r1)\n"
    "\n"
    "Block 0: <none> --> [0,15] --> 1,5\n"
    "Block 1: 0 --> [16,20] --> 2,3\n"
    "Block 2: 1 --> [21,22] --> 3\n"
    "Block 3: 2,1 --> [23,25] --> 4,5\n"
    "Block 4: 3 --> [26,32] --> 10\n"
    "Block 5: 0,3 --> [33,67] --> 6,9\n"
    "Block 6: 5 --> [68,73] --> 7,8\n"
    "Block 7: 6 --> [74,75] --> 8\n"
    "Block 8: 7,6 --> [76,76] --> 9\n"
    "Block 9: 8,5 --> [77,77] --> 10\n"
    "Block 10: 9,4 --> [78,81] --> <none>\n"
    ;

#include "tests/rtl-disasm-test.i"
