/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/guest-ppc/insn/common.h"

static const uint8_t input[] = {
    0xFC,0x22,0x18,0x24,  // fdiv f1,f2,f3
};

static const unsigned int guest_opt = 0;
static const unsigned int common_opt = BINREC_OPT_NATIVE_IEEE_NAN;

static const bool expected_success = true;

static const char expected[] =
    "[info] Scanning terminated at requested limit 0x3\n"
    "    0: LOAD_ARG   r1, 0\n"
    "    1: LOAD_ARG   r2, 1\n"
    "    2: GET_ALIAS  r3, a3\n"
    "    3: GET_ALIAS  r4, a4\n"
    "    4: FDIV       r5, r3, r4\n"
    "    5: LOAD_IMM   r6, 0x20000000000000\n"
    "    6: BITCAST    r7, r5\n"
    "    7: SLLI       r8, r7, 1\n"
    "    8: SEQ        r9, r8, r6\n"
    "    9: GOTO_IF_Z  r9, L1\n"
    "   10: FGETSTATE  r10\n"
    "   11: FSETROUND  r11, r10, TRUNC\n"
    "   12: FSETSTATE  r11\n"
    "   13: FDIV       r12, r3, r4\n"
    "   14: FGETSTATE  r13\n"
    "   15: FCOPYROUND r14, r13, r10\n"
    "   16: FSETSTATE  r14\n"
    "   17: LABEL      L1\n"
    "   18: GET_ALIAS  r15, a5\n"
    "   19: FGETSTATE  r16\n"
    "   20: FCLEAREXC  r17, r16\n"
    "   21: FSETSTATE  r17\n"
    "   22: FTESTEXC   r18, r16, INVALID\n"
    "   23: GOTO_IF_Z  r18, L3\n"
    "   24: BITCAST    r19, r3\n"
    "   25: SLLI       r20, r19, 13\n"
    "   26: BFEXT      r21, r19, 51, 12\n"
    "   27: SEQI       r22, r21, 4094\n"
    "   28: GOTO_IF_Z  r22, L5\n"
    "   29: GOTO_IF_NZ r20, L4\n"
    "   30: LABEL      L5\n"
    "   31: BITCAST    r23, r4\n"
    "   32: SLLI       r24, r23, 13\n"
    "   33: BFEXT      r25, r23, 51, 12\n"
    "   34: SEQI       r26, r25, 4094\n"
    "   35: GOTO_IF_Z  r26, L6\n"
    "   36: GOTO_IF_NZ r24, L4\n"
    "   37: LABEL      L6\n"
    "   38: BITCAST    r27, r3\n"
    "   39: BITCAST    r28, r4\n"
    "   40: OR         r29, r27, r28\n"
    "   41: SLLI       r30, r29, 1\n"
    "   42: GOTO_IF_NZ r30, L7\n"
    "   43: NOT        r31, r15\n"
    "   44: ORI        r32, r15, 2097152\n"
    "   45: ANDI       r33, r31, 2097152\n"
    "   46: SET_ALIAS  a5, r32\n"
    "   47: GOTO_IF_Z  r33, L8\n"
    "   48: ORI        r34, r32, -2147483648\n"
    "   49: SET_ALIAS  a5, r34\n"
    "   50: LABEL      L8\n"
    "   51: GOTO       L9\n"
    "   52: LABEL      L7\n"
    "   53: NOT        r35, r15\n"
    "   54: ORI        r36, r15, 4194304\n"
    "   55: ANDI       r37, r35, 4194304\n"
    "   56: SET_ALIAS  a5, r36\n"
    "   57: GOTO_IF_Z  r37, L10\n"
    "   58: ORI        r38, r36, -2147483648\n"
    "   59: SET_ALIAS  a5, r38\n"
    "   60: LABEL      L10\n"
    "   61: GOTO       L9\n"
    "   62: LABEL      L4\n"
    "   63: NOT        r39, r15\n"
    "   64: ORI        r40, r15, 16777216\n"
    "   65: ANDI       r41, r39, 16777216\n"
    "   66: SET_ALIAS  a5, r40\n"
    "   67: GOTO_IF_Z  r41, L11\n"
    "   68: ORI        r42, r40, -2147483648\n"
    "   69: SET_ALIAS  a5, r42\n"
    "   70: LABEL      L11\n"
    "   71: LABEL      L9\n"
    "   72: ANDI       r43, r15, 128\n"
    "   73: GOTO_IF_Z  r43, L3\n"
    "   74: LABEL      L12\n"
    "   75: GET_ALIAS  r44, a5\n"
    "   76: BFEXT      r45, r44, 12, 7\n"
    "   77: ANDI       r46, r45, 31\n"
    "   78: GET_ALIAS  r47, a5\n"
    "   79: BFINS      r48, r47, r46, 12, 7\n"
    "   80: SET_ALIAS  a5, r48\n"
    "   81: GOTO       L2\n"
    "   82: LABEL      L3\n"
    "   83: FTESTEXC   r49, r16, ZERO_DIVIDE\n"
    "   84: GOTO_IF_Z  r49, L13\n"
    "   85: NOT        r50, r15\n"
    "   86: ORI        r51, r15, 67108864\n"
    "   87: ANDI       r52, r50, 67108864\n"
    "   88: SET_ALIAS  a5, r51\n"
    "   89: GOTO_IF_Z  r52, L14\n"
    "   90: ORI        r53, r51, -2147483648\n"
    "   91: SET_ALIAS  a5, r53\n"
    "   92: LABEL      L14\n"
    "   93: ANDI       r54, r15, 16\n"
    "   94: GOTO_IF_NZ r54, L12\n"
    "   95: LABEL      L13\n"
    "   96: SET_ALIAS  a2, r5\n"
    "   97: BITCAST    r55, r5\n"
    "   98: SGTUI      r56, r55, 0\n"
    "   99: SLTSI      r57, r55, 0\n"
    "  100: BFEXT      r61, r55, 52, 11\n"
    "  101: SEQI       r58, r61, 0\n"
    "  102: SEQI       r59, r61, 2047\n"
    "  103: SLLI       r62, r55, 12\n"
    "  104: SEQI       r60, r62, 0\n"
    "  105: AND        r63, r58, r60\n"
    "  106: XORI       r64, r60, 1\n"
    "  107: AND        r65, r59, r64\n"
    "  108: AND        r66, r58, r56\n"
    "  109: OR         r67, r66, r65\n"
    "  110: OR         r68, r63, r65\n"
    "  111: XORI       r69, r68, 1\n"
    "  112: XORI       r70, r57, 1\n"
    "  113: AND        r71, r57, r69\n"
    "  114: AND        r72, r70, r69\n"
    "  115: SLLI       r73, r67, 4\n"
    "  116: SLLI       r74, r71, 3\n"
    "  117: SLLI       r75, r72, 2\n"
    "  118: SLLI       r76, r63, 1\n"
    "  119: OR         r77, r73, r74\n"
    "  120: OR         r78, r75, r76\n"
    "  121: OR         r79, r77, r59\n"
    "  122: OR         r80, r79, r78\n"
    "  123: FTESTEXC   r81, r16, INEXACT\n"
    "  124: SLLI       r82, r81, 5\n"
    "  125: OR         r83, r80, r82\n"
    "  126: GET_ALIAS  r84, a5\n"
    "  127: BFINS      r85, r84, r83, 12, 7\n"
    "  128: SET_ALIAS  a5, r85\n"
    "  129: GOTO_IF_Z  r81, L15\n"
    "  130: GET_ALIAS  r86, a5\n"
    "  131: NOT        r87, r86\n"
    "  132: ORI        r88, r86, 33554432\n"
    "  133: ANDI       r89, r87, 33554432\n"
    "  134: SET_ALIAS  a5, r88\n"
    "  135: GOTO_IF_Z  r89, L16\n"
    "  136: ORI        r90, r88, -2147483648\n"
    "  137: SET_ALIAS  a5, r90\n"
    "  138: LABEL      L16\n"
    "  139: LABEL      L15\n"
    "  140: FTESTEXC   r91, r16, OVERFLOW\n"
    "  141: GOTO_IF_Z  r91, L17\n"
    "  142: GET_ALIAS  r92, a5\n"
    "  143: NOT        r93, r92\n"
    "  144: ORI        r94, r92, 268435456\n"
    "  145: ANDI       r95, r93, 268435456\n"
    "  146: SET_ALIAS  a5, r94\n"
    "  147: GOTO_IF_Z  r95, L18\n"
    "  148: ORI        r96, r94, -2147483648\n"
    "  149: SET_ALIAS  a5, r96\n"
    "  150: LABEL      L18\n"
    "  151: LABEL      L17\n"
    "  152: FTESTEXC   r97, r16, UNDERFLOW\n"
    "  153: GOTO_IF_Z  r97, L2\n"
    "  154: GET_ALIAS  r98, a5\n"
    "  155: NOT        r99, r98\n"
    "  156: ORI        r100, r98, 134217728\n"
    "  157: ANDI       r101, r99, 134217728\n"
    "  158: SET_ALIAS  a5, r100\n"
    "  159: GOTO_IF_Z  r101, L19\n"
    "  160: ORI        r102, r100, -2147483648\n"
    "  161: SET_ALIAS  a5, r102\n"
    "  162: LABEL      L19\n"
    "  163: LABEL      L2\n"
    "  164: LOAD_IMM   r103, 4\n"
    "  165: SET_ALIAS  a1, r103\n"
    "  166: RETURN     r1\n"
    "\n"
    "Alias 1: int32 @ 964(r1)\n"
    "Alias 2: float64 @ 400(r1)\n"
    "Alias 3: float64 @ 416(r1)\n"
    "Alias 4: float64 @ 432(r1)\n"
    "Alias 5: int32 @ 944(r1)\n"
    "\n"
    "Block 0: <none> --> [0,9] --> 1,2\n"
    "Block 1: 0 --> [10,16] --> 2\n"
    "Block 2: 1,0 --> [17,23] --> 3,19\n"
    "Block 3: 2 --> [24,28] --> 4,5\n"
    "Block 4: 3 --> [29,29] --> 5,14\n"
    "Block 5: 4,3 --> [30,35] --> 6,7\n"
    "Block 6: 5 --> [36,36] --> 7,14\n"
    "Block 7: 6,5 --> [37,42] --> 8,11\n"
    "Block 8: 7 --> [43,47] --> 9,10\n"
    "Block 9: 8 --> [48,49] --> 10\n"
    "Block 10: 9,8 --> [50,51] --> 17\n"
    "Block 11: 7 --> [52,57] --> 12,13\n"
    "Block 12: 11 --> [58,59] --> 13\n"
    "Block 13: 12,11 --> [60,61] --> 17\n"
    "Block 14: 4,6 --> [62,67] --> 15,16\n"
    "Block 15: 14 --> [68,69] --> 16\n"
    "Block 16: 15,14 --> [70,70] --> 17\n"
    "Block 17: 16,10,13 --> [71,73] --> 18,19\n"
    "Block 18: 17,22 --> [74,81] --> 35\n"
    "Block 19: 2,17 --> [82,84] --> 20,23\n"
    "Block 20: 19 --> [85,89] --> 21,22\n"
    "Block 21: 20 --> [90,91] --> 22\n"
    "Block 22: 21,20 --> [92,94] --> 23,18\n"
    "Block 23: 22,19 --> [95,129] --> 24,27\n"
    "Block 24: 23 --> [130,135] --> 25,26\n"
    "Block 25: 24 --> [136,137] --> 26\n"
    "Block 26: 25,24 --> [138,138] --> 27\n"
    "Block 27: 26,23 --> [139,141] --> 28,31\n"
    "Block 28: 27 --> [142,147] --> 29,30\n"
    "Block 29: 28 --> [148,149] --> 30\n"
    "Block 30: 29,28 --> [150,150] --> 31\n"
    "Block 31: 30,27 --> [151,153] --> 32,35\n"
    "Block 32: 31 --> [154,159] --> 33,34\n"
    "Block 33: 32 --> [160,161] --> 34\n"
    "Block 34: 33,32 --> [162,162] --> 35\n"
    "Block 35: 34,18,31 --> [163,166] --> <none>\n"
    ;

#include "tests/rtl-disasm-test.i"
