/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/guest-ppc/insn/common.h"

static const uint8_t input[] = {
    0xEC,0x22,0x20,0xFA,  // fmadds f1,f2,f3,f4
};

static const unsigned int guest_opt = 0;
static const unsigned int common_opt = 0;

static const bool expected_success = true;

static const char expected[] =
    "[info] Scanning terminated at requested limit 0x3\n"
    "    0: LOAD_ARG   r1, 0\n"
    "    1: LOAD_ARG   r2, 1\n"
    "    2: GET_ALIAS  r3, a3\n"
    "    3: GET_ALIAS  r4, a4\n"
    "    4: GET_ALIAS  r5, a5\n"
    "    5: SET_ALIAS  a7, r3\n"
    "    6: SET_ALIAS  a8, r4\n"
    "    7: BITCAST    r6, r4\n"
    "    8: BITCAST    r7, r3\n"
    "    9: BFEXT      r8, r6, 0, 52\n"
    "   10: GOTO_IF_Z  r8, L1\n"
    "   11: BFEXT      r9, r7, 52, 11\n"
    "   12: BFEXT      r10, r6, 52, 11\n"
    "   13: SEQI       r11, r9, 2047\n"
    "   14: GOTO_IF_NZ r11, L1\n"
    "   15: SEQI       r12, r10, 2047\n"
    "   16: GOTO_IF_NZ r12, L1\n"
    "   17: GOTO_IF_NZ r10, L2\n"
    "   18: CLZ        r13, r8\n"
    "   19: ADDI       r14, r13, -11\n"
    "   20: SUB        r15, r9, r14\n"
    "   21: SGTSI      r16, r15, 0\n"
    "   22: GOTO_IF_Z  r16, L1\n"
    "   23: LOAD_IMM   r17, 0x8000000000000000\n"
    "   24: AND        r18, r6, r17\n"
    "   25: SLL        r19, r8, r14\n"
    "   26: BFINS      r20, r7, r15, 52, 11\n"
    "   27: OR         r21, r18, r19\n"
    "   28: BITCAST    r22, r20\n"
    "   29: BITCAST    r23, r21\n"
    "   30: SET_ALIAS  a7, r22\n"
    "   31: SET_ALIAS  a8, r23\n"
    "   32: LABEL      L2\n"
    "   33: GET_ALIAS  r24, a8\n"
    "   34: BITCAST    r25, r24\n"
    "   35: ANDI       r26, r25, -134217728\n"
    "   36: ANDI       r27, r25, 134217728\n"
    "   37: ADD        r28, r26, r27\n"
    "   38: BITCAST    r29, r28\n"
    "   39: SET_ALIAS  a8, r29\n"
    "   40: BFEXT      r30, r28, 52, 11\n"
    "   41: SEQI       r31, r30, 2047\n"
    "   42: GOTO_IF_Z  r31, L1\n"
    "   43: LOAD_IMM   r32, 0x10000000000000\n"
    "   44: SUB        r33, r28, r32\n"
    "   45: BITCAST    r34, r33\n"
    "   46: SET_ALIAS  a8, r34\n"
    "   47: BFEXT      r35, r7, 52, 11\n"
    "   48: GOTO_IF_Z  r35, L3\n"
    "   49: SGTUI      r36, r35, 2045\n"
    "   50: GOTO_IF_NZ r36, L1\n"
    "   51: ADD        r37, r7, r32\n"
    "   52: BITCAST    r38, r37\n"
    "   53: SET_ALIAS  a7, r38\n"
    "   54: GOTO       L1\n"
    "   55: LABEL      L3\n"
    "   56: SLLI       r39, r7, 1\n"
    "   57: BFINS      r40, r7, r39, 0, 63\n"
    "   58: BITCAST    r41, r40\n"
    "   59: SET_ALIAS  a7, r41\n"
    "   60: LABEL      L1\n"
    "   61: GET_ALIAS  r42, a7\n"
    "   62: GET_ALIAS  r43, a8\n"
    "   63: FMADD      r44, r42, r43, r5\n"
    "   64: LOAD_IMM   r45, 0.0\n"
    "   65: FCMP       r46, r42, r45, NUN\n"
    "   66: FCMP       r47, r43, r45, UN\n"
    "   67: FCMP       r48, r5, r45, UN\n"
    "   68: AND        r49, r46, r47\n"
    "   69: AND        r50, r49, r48\n"
    "   70: BITCAST    r51, r5\n"
    "   71: LOAD_IMM   r52, 0x8000000000000\n"
    "   72: OR         r53, r51, r52\n"
    "   73: BITCAST    r54, r53\n"
    "   74: SELECT     r55, r54, r44, r50\n"
    "   75: FCVT       r56, r55\n"
    "   76: SET_ALIAS  a9, r56\n"
    "   77: GET_ALIAS  r57, a6\n"
    "   78: BFEXT      r58, r57, 0, 2\n"
    "   79: GOTO_IF_NZ r58, L4\n"
    "   80: BITCAST    r59, r55\n"
    "   81: BFEXT      r60, r59, 52, 11\n"
    "   82: ZCAST      r61, r60\n"
    "   83: ADDI       r62, r61, -874\n"
    "   84: SLTUI      r63, r62, 277\n"
    "   85: GOTO_IF_Z  r63, L4\n"
    "   86: BFEXT      r64, r59, 0, 52\n"
    "   87: ADDI       r65, r61, -862\n"
    "   88: SLTUI      r66, r61, 897\n"
    "   89: LOAD_IMM   r67, 35\n"
    "   90: SELECT     r68, r65, r67, r66\n"
    "   91: SLL        r69, r64, r68\n"
    "   92: LOAD_IMM   r70, 0x8000000000000000\n"
    "   93: SEQ        r71, r69, r70\n"
    "   94: GOTO_IF_Z  r71, L4\n"
    "   95: FGETSTATE  r72\n"
    "   96: FCLEAREXC  r73, r72\n"
    "   97: FSETROUND  r74, r73, TRUNC\n"
    "   98: FSETSTATE  r74\n"
    "   99: FMADD      r75, r42, r43, r5\n"
    "  100: FGETSTATE  r76\n"
    "  101: FSETSTATE  r72\n"
    "  102: BITCAST    r77, r75\n"
    "  103: FTESTEXC   r78, r76, INEXACT\n"
    "  104: GOTO_IF_Z  r78, L4\n"
    "  105: BFEXT      r79, r77, 0, 52\n"
    "  106: SEQ        r80, r79, r64\n"
    "  107: GOTO_IF_NZ r80, L5\n"
    "  108: ADDI       r81, r59, -1\n"
    "  109: BITCAST    r82, r81\n"
    "  110: FCVT       r83, r82\n"
    "  111: SET_ALIAS  a9, r83\n"
    "  112: GOTO       L4\n"
    "  113: LABEL      L5\n"
    "  114: ADDI       r84, r59, 1\n"
    "  115: BITCAST    r85, r84\n"
    "  116: FCVT       r86, r85\n"
    "  117: SET_ALIAS  a9, r86\n"
    "  118: LABEL      L4\n"
    "  119: GET_ALIAS  r87, a9\n"
    "  120: LOAD_IMM   r88, 0x1000000\n"
    "  121: BITCAST    r89, r87\n"
    "  122: SLLI       r90, r89, 1\n"
    "  123: SEQ        r91, r90, r88\n"
    "  124: GOTO_IF_Z  r91, L6\n"
    "  125: FGETSTATE  r92\n"
    "  126: FSETROUND  r93, r92, TRUNC\n"
    "  127: FSETSTATE  r93\n"
    "  128: FMADD      r94, r42, r43, r5\n"
    "  129: FCVT       r95, r94\n"
    "  130: FGETSTATE  r96\n"
    "  131: FCOPYROUND r97, r96, r92\n"
    "  132: FSETSTATE  r97\n"
    "  133: LABEL      L6\n"
    "  134: FGETSTATE  r98\n"
    "  135: FCLEAREXC  r99, r98\n"
    "  136: FSETSTATE  r99\n"
    "  137: FTESTEXC   r100, r98, INVALID\n"
    "  138: GOTO_IF_Z  r100, L8\n"
    "  139: BITCAST    r101, r42\n"
    "  140: SLLI       r102, r101, 13\n"
    "  141: BFEXT      r103, r101, 51, 12\n"
    "  142: SEQI       r104, r103, 4094\n"
    "  143: GOTO_IF_Z  r104, L10\n"
    "  144: GOTO_IF_NZ r102, L9\n"
    "  145: LABEL      L10\n"
    "  146: BITCAST    r105, r43\n"
    "  147: SLLI       r106, r105, 13\n"
    "  148: BFEXT      r107, r105, 51, 12\n"
    "  149: SEQI       r108, r107, 4094\n"
    "  150: GOTO_IF_Z  r108, L11\n"
    "  151: GOTO_IF_NZ r106, L9\n"
    "  152: LABEL      L11\n"
    "  153: BITCAST    r109, r5\n"
    "  154: SLLI       r110, r109, 13\n"
    "  155: BFEXT      r111, r109, 51, 12\n"
    "  156: SEQI       r112, r111, 4094\n"
    "  157: GOTO_IF_Z  r112, L13\n"
    "  158: GOTO_IF_NZ r110, L12\n"
    "  159: LABEL      L13\n"
    "  160: BITCAST    r113, r42\n"
    "  161: BITCAST    r114, r43\n"
    "  162: SLLI       r115, r113, 1\n"
    "  163: SLLI       r116, r114, 1\n"
    "  164: LOAD_IMM   r117, 0xFFE0000000000000\n"
    "  165: SEQ        r118, r115, r117\n"
    "  166: GOTO_IF_Z  r118, L14\n"
    "  167: GOTO_IF_Z  r116, L15\n"
    "  168: GOTO       L16\n"
    "  169: LABEL      L14\n"
    "  170: GOTO_IF_NZ r115, L16\n"
    "  171: LABEL      L15\n"
    "  172: NOT        r119, r57\n"
    "  173: ORI        r120, r57, 1048576\n"
    "  174: ANDI       r121, r119, 1048576\n"
    "  175: SET_ALIAS  a6, r120\n"
    "  176: GOTO_IF_Z  r121, L17\n"
    "  177: ORI        r122, r120, -2147483648\n"
    "  178: SET_ALIAS  a6, r122\n"
    "  179: LABEL      L17\n"
    "  180: GOTO       L18\n"
    "  181: LABEL      L16\n"
    "  182: NOT        r123, r57\n"
    "  183: ORI        r124, r57, 8388608\n"
    "  184: ANDI       r125, r123, 8388608\n"
    "  185: SET_ALIAS  a6, r124\n"
    "  186: GOTO_IF_Z  r125, L19\n"
    "  187: ORI        r126, r124, -2147483648\n"
    "  188: SET_ALIAS  a6, r126\n"
    "  189: LABEL      L19\n"
    "  190: LABEL      L18\n"
    "  191: ANDI       r127, r57, 128\n"
    "  192: GOTO_IF_Z  r127, L20\n"
    "  193: GET_ALIAS  r128, a6\n"
    "  194: BFEXT      r129, r128, 12, 7\n"
    "  195: ANDI       r130, r129, 31\n"
    "  196: GET_ALIAS  r131, a6\n"
    "  197: BFINS      r132, r131, r130, 12, 7\n"
    "  198: SET_ALIAS  a6, r132\n"
    "  199: GOTO       L7\n"
    "  200: LABEL      L20\n"
    "  201: LOAD_IMM   r133, nan(0x400000)\n"
    "  202: FCVT       r134, r133\n"
    "  203: STORE      408(r1), r134\n"
    "  204: SET_ALIAS  a2, r134\n"
    "  205: LOAD_IMM   r135, 17\n"
    "  206: GET_ALIAS  r136, a6\n"
    "  207: BFINS      r137, r136, r135, 12, 7\n"
    "  208: SET_ALIAS  a6, r137\n"
    "  209: GOTO       L7\n"
    "  210: LABEL      L12\n"
    "  211: FMUL       r138, r42, r43\n"
    "  212: FGETSTATE  r139\n"
    "  213: FSETSTATE  r99\n"
    "  214: FTESTEXC   r140, r139, INVALID\n"
    "  215: GOTO_IF_Z  r140, L9\n"
    "  216: NOT        r141, r57\n"
    "  217: ORI        r142, r57, 17825792\n"
    "  218: ANDI       r143, r141, 17825792\n"
    "  219: SET_ALIAS  a6, r142\n"
    "  220: GOTO_IF_Z  r143, L21\n"
    "  221: ORI        r144, r142, -2147483648\n"
    "  222: SET_ALIAS  a6, r144\n"
    "  223: LABEL      L21\n"
    "  224: GOTO       L22\n"
    "  225: LABEL      L9\n"
    "  226: NOT        r145, r57\n"
    "  227: ORI        r146, r57, 16777216\n"
    "  228: ANDI       r147, r145, 16777216\n"
    "  229: SET_ALIAS  a6, r146\n"
    "  230: GOTO_IF_Z  r147, L23\n"
    "  231: ORI        r148, r146, -2147483648\n"
    "  232: SET_ALIAS  a6, r148\n"
    "  233: LABEL      L23\n"
    "  234: LABEL      L22\n"
    "  235: ANDI       r149, r57, 128\n"
    "  236: GOTO_IF_Z  r149, L8\n"
    "  237: GET_ALIAS  r150, a6\n"
    "  238: BFEXT      r151, r150, 12, 7\n"
    "  239: ANDI       r152, r151, 31\n"
    "  240: GET_ALIAS  r153, a6\n"
    "  241: BFINS      r154, r153, r152, 12, 7\n"
    "  242: SET_ALIAS  a6, r154\n"
    "  243: GOTO       L7\n"
    "  244: LABEL      L8\n"
    "  245: FCVT       r155, r87\n"
    "  246: STORE      408(r1), r155\n"
    "  247: SET_ALIAS  a2, r155\n"
    "  248: BITCAST    r156, r87\n"
    "  249: SGTUI      r157, r156, 0\n"
    "  250: SRLI       r158, r156, 31\n"
    "  251: BFEXT      r162, r156, 23, 8\n"
    "  252: SEQI       r159, r162, 0\n"
    "  253: SEQI       r160, r162, 255\n"
    "  254: SLLI       r163, r156, 9\n"
    "  255: SEQI       r161, r163, 0\n"
    "  256: AND        r164, r159, r161\n"
    "  257: XORI       r165, r161, 1\n"
    "  258: AND        r166, r160, r165\n"
    "  259: AND        r167, r159, r157\n"
    "  260: OR         r168, r167, r166\n"
    "  261: OR         r169, r164, r166\n"
    "  262: XORI       r170, r169, 1\n"
    "  263: XORI       r171, r158, 1\n"
    "  264: AND        r172, r158, r170\n"
    "  265: AND        r173, r171, r170\n"
    "  266: SLLI       r174, r168, 4\n"
    "  267: SLLI       r175, r172, 3\n"
    "  268: SLLI       r176, r173, 2\n"
    "  269: SLLI       r177, r164, 1\n"
    "  270: OR         r178, r174, r175\n"
    "  271: OR         r179, r176, r177\n"
    "  272: OR         r180, r178, r160\n"
    "  273: OR         r181, r180, r179\n"
    "  274: FTESTEXC   r182, r98, INEXACT\n"
    "  275: SLLI       r183, r182, 5\n"
    "  276: OR         r184, r181, r183\n"
    "  277: GET_ALIAS  r185, a6\n"
    "  278: BFINS      r186, r185, r184, 12, 7\n"
    "  279: SET_ALIAS  a6, r186\n"
    "  280: GOTO_IF_Z  r182, L24\n"
    "  281: GET_ALIAS  r187, a6\n"
    "  282: NOT        r188, r187\n"
    "  283: ORI        r189, r187, 33554432\n"
    "  284: ANDI       r190, r188, 33554432\n"
    "  285: SET_ALIAS  a6, r189\n"
    "  286: GOTO_IF_Z  r190, L25\n"
    "  287: ORI        r191, r189, -2147483648\n"
    "  288: SET_ALIAS  a6, r191\n"
    "  289: LABEL      L25\n"
    "  290: LABEL      L24\n"
    "  291: FTESTEXC   r192, r98, OVERFLOW\n"
    "  292: GOTO_IF_Z  r192, L26\n"
    "  293: GET_ALIAS  r193, a6\n"
    "  294: NOT        r194, r193\n"
    "  295: ORI        r195, r193, 268435456\n"
    "  296: ANDI       r196, r194, 268435456\n"
    "  297: SET_ALIAS  a6, r195\n"
    "  298: GOTO_IF_Z  r196, L27\n"
    "  299: ORI        r197, r195, -2147483648\n"
    "  300: SET_ALIAS  a6, r197\n"
    "  301: LABEL      L27\n"
    "  302: LABEL      L26\n"
    "  303: FTESTEXC   r198, r98, UNDERFLOW\n"
    "  304: GOTO_IF_Z  r198, L7\n"
    "  305: GET_ALIAS  r199, a6\n"
    "  306: NOT        r200, r199\n"
    "  307: ORI        r201, r199, 134217728\n"
    "  308: ANDI       r202, r200, 134217728\n"
    "  309: SET_ALIAS  a6, r201\n"
    "  310: GOTO_IF_Z  r202, L28\n"
    "  311: ORI        r203, r201, -2147483648\n"
    "  312: SET_ALIAS  a6, r203\n"
    "  313: LABEL      L28\n"
    "  314: LABEL      L7\n"
    "  315: LOAD_IMM   r204, 4\n"
    "  316: SET_ALIAS  a1, r204\n"
    "  317: RETURN     r1\n"
    "\n"
    "Alias 1: int32 @ 964(r1)\n"
    "Alias 2: float64 @ 400(r1)\n"
    "Alias 3: float64 @ 416(r1)\n"
    "Alias 4: float64 @ 432(r1)\n"
    "Alias 5: float64 @ 448(r1)\n"
    "Alias 6: int32 @ 944(r1)\n"
    "Alias 7: float64, no bound storage\n"
    "Alias 8: float64, no bound storage\n"
    "Alias 9: float32, no bound storage\n"
    "\n"
    "Block 0: <none> --> [0,10] --> 1,11\n"
    "Block 1: 0 --> [11,14] --> 2,11\n"
    "Block 2: 1 --> [15,16] --> 3,11\n"
    "Block 3: 2 --> [17,17] --> 4,6\n"
    "Block 4: 3 --> [18,22] --> 5,11\n"
    "Block 5: 4 --> [23,31] --> 6\n"
    "Block 6: 5,3 --> [32,42] --> 7,11\n"
    "Block 7: 6 --> [43,48] --> 8,10\n"
    "Block 8: 7 --> [49,50] --> 9,11\n"
    "Block 9: 8 --> [51,54] --> 11\n"
    "Block 10: 7 --> [55,59] --> 11\n"
    "Block 11: 10,9,0,1,2,4,6,8 --> [60,79] --> 12,18\n"
    "Block 12: 11 --> [80,85] --> 13,18\n"
    "Block 13: 12 --> [86,94] --> 14,18\n"
    "Block 14: 13 --> [95,104] --> 15,18\n"
    "Block 15: 14 --> [105,107] --> 16,17\n"
    "Block 16: 15 --> [108,112] --> 18\n"
    "Block 17: 15 --> [113,117] --> 18\n"
    "Block 18: 17,11,12,13,14,16 --> [118,124] --> 19,20\n"
    "Block 19: 18 --> [125,132] --> 20\n"
    "Block 20: 19,18 --> [133,138] --> 21,49\n"
    "Block 21: 20 --> [139,143] --> 22,23\n"
    "Block 22: 21 --> [144,144] --> 23,44\n"
    "Block 23: 22,21 --> [145,150] --> 24,25\n"
    "Block 24: 23 --> [151,151] --> 25,44\n"
    "Block 25: 24,23 --> [152,157] --> 26,27\n"
    "Block 26: 25 --> [158,158] --> 27,40\n"
    "Block 27: 26,25 --> [159,166] --> 28,30\n"
    "Block 28: 27 --> [167,167] --> 29,31\n"
    "Block 29: 28 --> [168,168] --> 34\n"
    "Block 30: 27 --> [169,170] --> 31,34\n"
    "Block 31: 30,28 --> [171,176] --> 32,33\n"
    "Block 32: 31 --> [177,178] --> 33\n"
    "Block 33: 32,31 --> [179,180] --> 37\n"
    "Block 34: 29,30 --> [181,186] --> 35,36\n"
    "Block 35: 34 --> [187,188] --> 36\n"
    "Block 36: 35,34 --> [189,189] --> 37\n"
    "Block 37: 36,33 --> [190,192] --> 38,39\n"
    "Block 38: 37 --> [193,199] --> 61\n"
    "Block 39: 37 --> [200,209] --> 61\n"
    "Block 40: 26 --> [210,215] --> 41,44\n"
    "Block 41: 40 --> [216,220] --> 42,43\n"
    "Block 42: 41 --> [221,222] --> 43\n"
    "Block 43: 42,41 --> [223,224] --> 47\n"
    "Block 44: 22,24,40 --> [225,230] --> 45,46\n"
    "Block 45: 44 --> [231,232] --> 46\n"
    "Block 46: 45,44 --> [233,233] --> 47\n"
    "Block 47: 46,43 --> [234,236] --> 48,49\n"
    "Block 48: 47 --> [237,243] --> 61\n"
    "Block 49: 20,47 --> [244,280] --> 50,53\n"
    "Block 50: 49 --> [281,286] --> 51,52\n"
    "Block 51: 50 --> [287,288] --> 52\n"
    "Block 52: 51,50 --> [289,289] --> 53\n"
    "Block 53: 52,49 --> [290,292] --> 54,57\n"
    "Block 54: 53 --> [293,298] --> 55,56\n"
    "Block 55: 54 --> [299,300] --> 56\n"
    "Block 56: 55,54 --> [301,301] --> 57\n"
    "Block 57: 56,53 --> [302,304] --> 58,61\n"
    "Block 58: 57 --> [305,310] --> 59,60\n"
    "Block 59: 58 --> [311,312] --> 60\n"
    "Block 60: 59,58 --> [313,313] --> 61\n"
    "Block 61: 60,38,39,48,57 --> [314,317] --> <none>\n"
    ;

#include "tests/rtl-disasm-test.i"
