/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/guest-ppc/insn/common.h"

static const uint8_t input[] = {
    0xEC,0x20,0x10,0x30,  // fres f1,f2
};

static const unsigned int guest_opt = BINREC_OPT_G_PPC_NATIVE_RECIPROCAL;
static const unsigned int common_opt = 0;

static const bool expected_success = true;

static const char expected[] =
    "[info] Scanning terminated at requested limit 0x3\n"
    "    0: LOAD_ARG   r1, 0\n"
    "    1: LOAD_ARG   r2, 1\n"
    "    2: GET_ALIAS  r3, a3\n"
    "    3: FGETSTATE  r4\n"
    "    4: FCMP       r5, r3, r3, UN\n"
    "    5: FCVT       r6, r3\n"
    "    6: SET_ALIAS  a5, r6\n"
    "    7: GOTO_IF_Z  r5, L1\n"
    "    8: BITCAST    r7, r3\n"
    "    9: BFEXT      r8, r7, 51, 1\n"
    "   10: GOTO_IF_NZ r8, L1\n"
    "   11: BITCAST    r9, r6\n"
    "   12: XORI       r10, r9, 4194304\n"
    "   13: BITCAST    r11, r10\n"
    "   14: SET_ALIAS  a5, r11\n"
    "   15: LABEL      L1\n"
    "   16: GET_ALIAS  r12, a5\n"
    "   17: FSETSTATE  r4\n"
    "   18: LOAD_IMM   r13, 1.0f\n"
    "   19: FDIV       r14, r13, r12\n"
    "   20: GET_ALIAS  r15, a4\n"
    "   21: FGETSTATE  r16\n"
    "   22: FCLEAREXC  r17, r16\n"
    "   23: FSETSTATE  r17\n"
    "   24: FTESTEXC   r18, r16, INVALID\n"
    "   25: GOTO_IF_Z  r18, L3\n"
    "   26: NOT        r19, r15\n"
    "   27: ORI        r20, r15, 16777216\n"
    "   28: ANDI       r21, r19, 16777216\n"
    "   29: SET_ALIAS  a4, r20\n"
    "   30: GOTO_IF_Z  r21, L4\n"
    "   31: ORI        r22, r20, -2147483648\n"
    "   32: SET_ALIAS  a4, r22\n"
    "   33: LABEL      L4\n"
    "   34: ANDI       r23, r15, 128\n"
    "   35: GOTO_IF_Z  r23, L3\n"
    "   36: LABEL      L5\n"
    "   37: GET_ALIAS  r24, a4\n"
    "   38: BFEXT      r25, r24, 12, 7\n"
    "   39: ANDI       r26, r25, 31\n"
    "   40: GET_ALIAS  r27, a4\n"
    "   41: BFINS      r28, r27, r26, 12, 7\n"
    "   42: SET_ALIAS  a4, r28\n"
    "   43: GOTO       L2\n"
    "   44: LABEL      L3\n"
    "   45: FTESTEXC   r29, r16, ZERO_DIVIDE\n"
    "   46: GOTO_IF_Z  r29, L6\n"
    "   47: NOT        r30, r15\n"
    "   48: ORI        r31, r15, 67108864\n"
    "   49: ANDI       r32, r30, 67108864\n"
    "   50: SET_ALIAS  a4, r31\n"
    "   51: GOTO_IF_Z  r32, L7\n"
    "   52: ORI        r33, r31, -2147483648\n"
    "   53: SET_ALIAS  a4, r33\n"
    "   54: LABEL      L7\n"
    "   55: ANDI       r34, r15, 16\n"
    "   56: GOTO_IF_NZ r34, L5\n"
    "   57: LABEL      L6\n"
    "   58: FCVT       r35, r14\n"
    "   59: STORE      408(r1), r35\n"
    "   60: SET_ALIAS  a2, r35\n"
    "   61: BITCAST    r36, r14\n"
    "   62: SGTUI      r37, r36, 0\n"
    "   63: SRLI       r38, r36, 31\n"
    "   64: BFEXT      r42, r36, 23, 8\n"
    "   65: SEQI       r39, r42, 0\n"
    "   66: SEQI       r40, r42, 255\n"
    "   67: SLLI       r43, r36, 9\n"
    "   68: SEQI       r41, r43, 0\n"
    "   69: AND        r44, r39, r41\n"
    "   70: XORI       r45, r41, 1\n"
    "   71: AND        r46, r40, r45\n"
    "   72: AND        r47, r39, r37\n"
    "   73: OR         r48, r47, r46\n"
    "   74: OR         r49, r44, r46\n"
    "   75: XORI       r50, r49, 1\n"
    "   76: XORI       r51, r38, 1\n"
    "   77: AND        r52, r38, r50\n"
    "   78: AND        r53, r51, r50\n"
    "   79: SLLI       r54, r48, 4\n"
    "   80: SLLI       r55, r52, 3\n"
    "   81: SLLI       r56, r53, 2\n"
    "   82: SLLI       r57, r44, 1\n"
    "   83: OR         r58, r54, r55\n"
    "   84: OR         r59, r56, r57\n"
    "   85: OR         r60, r58, r40\n"
    "   86: OR         r61, r60, r59\n"
    "   87: FTESTEXC   r62, r16, INEXACT\n"
    "   88: SLLI       r63, r62, 5\n"
    "   89: OR         r64, r61, r63\n"
    "   90: GET_ALIAS  r65, a4\n"
    "   91: BFINS      r66, r65, r64, 12, 7\n"
    "   92: SET_ALIAS  a4, r66\n"
    "   93: FTESTEXC   r67, r16, OVERFLOW\n"
    "   94: GOTO_IF_Z  r67, L8\n"
    "   95: GET_ALIAS  r68, a4\n"
    "   96: NOT        r69, r68\n"
    "   97: ORI        r70, r68, 268435456\n"
    "   98: ANDI       r71, r69, 268435456\n"
    "   99: SET_ALIAS  a4, r70\n"
    "  100: GOTO_IF_Z  r71, L9\n"
    "  101: ORI        r72, r70, -2147483648\n"
    "  102: SET_ALIAS  a4, r72\n"
    "  103: LABEL      L9\n"
    "  104: LABEL      L8\n"
    "  105: FTESTEXC   r73, r16, UNDERFLOW\n"
    "  106: GOTO_IF_Z  r73, L2\n"
    "  107: GET_ALIAS  r74, a4\n"
    "  108: NOT        r75, r74\n"
    "  109: ORI        r76, r74, 134217728\n"
    "  110: ANDI       r77, r75, 134217728\n"
    "  111: SET_ALIAS  a4, r76\n"
    "  112: GOTO_IF_Z  r77, L10\n"
    "  113: ORI        r78, r76, -2147483648\n"
    "  114: SET_ALIAS  a4, r78\n"
    "  115: LABEL      L10\n"
    "  116: LABEL      L2\n"
    "  117: LOAD_IMM   r79, 4\n"
    "  118: SET_ALIAS  a1, r79\n"
    "  119: RETURN     r1\n"
    "\n"
    "Alias 1: int32 @ 964(r1)\n"
    "Alias 2: float64 @ 400(r1)\n"
    "Alias 3: float64 @ 416(r1)\n"
    "Alias 4: int32 @ 944(r1)\n"
    "Alias 5: float32, no bound storage\n"
    "\n"
    "Block 0: <none> --> [0,7] --> 1,3\n"
    "Block 1: 0 --> [8,10] --> 2,3\n"
    "Block 2: 1 --> [11,14] --> 3\n"
    "Block 3: 2,0,1 --> [15,25] --> 4,8\n"
    "Block 4: 3 --> [26,30] --> 5,6\n"
    "Block 5: 4 --> [31,32] --> 6\n"
    "Block 6: 5,4 --> [33,35] --> 7,8\n"
    "Block 7: 6,11 --> [36,43] --> 20\n"
    "Block 8: 3,6 --> [44,46] --> 9,12\n"
    "Block 9: 8 --> [47,51] --> 10,11\n"
    "Block 10: 9 --> [52,53] --> 11\n"
    "Block 11: 10,9 --> [54,56] --> 12,7\n"
    "Block 12: 11,8 --> [57,94] --> 13,16\n"
    "Block 13: 12 --> [95,100] --> 14,15\n"
    "Block 14: 13 --> [101,102] --> 15\n"
    "Block 15: 14,13 --> [103,103] --> 16\n"
    "Block 16: 15,12 --> [104,106] --> 17,20\n"
    "Block 17: 16 --> [107,112] --> 18,19\n"
    "Block 18: 17 --> [113,114] --> 19\n"
    "Block 19: 18,17 --> [115,115] --> 20\n"
    "Block 20: 19,7,16 --> [116,119] --> <none>\n"
    ;

#include "tests/rtl-disasm-test.i"
