/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/guest-ppc/insn/common.h"

static const uint8_t input[] = {
    0xFC,0x20,0x10,0x18,  // frsp f1,f2
};

static const unsigned int guest_opt = 0;
static const unsigned int common_opt = 0;

static const bool expected_success = true;

static const char expected[] =
    "[info] Scanning terminated at requested limit 0x3\n"
    "    0: LOAD_ARG   r1, 0\n"
    "    1: LOAD_ARG   r2, 1\n"
    "    2: GET_ALIAS  r3, a3\n"
    "    3: FCVT       r4, r3\n"
    "    4: GET_ALIAS  r5, a4\n"
    "    5: FGETSTATE  r6\n"
    "    6: FCLEAREXC  r7, r6\n"
    "    7: FSETSTATE  r7\n"
    "    8: FTESTEXC   r8, r6, INVALID\n"
    "    9: GOTO_IF_Z  r8, L2\n"
    "   10: NOT        r9, r5\n"
    "   11: ORI        r10, r5, 16777216\n"
    "   12: ANDI       r11, r9, 16777216\n"
    "   13: SET_ALIAS  a4, r10\n"
    "   14: GOTO_IF_Z  r11, L3\n"
    "   15: ORI        r12, r10, -2147483648\n"
    "   16: SET_ALIAS  a4, r12\n"
    "   17: LABEL      L3\n"
    "   18: ANDI       r13, r5, 128\n"
    "   19: GOTO_IF_Z  r13, L2\n"
    "   20: GET_ALIAS  r14, a4\n"
    "   21: BFEXT      r15, r14, 12, 7\n"
    "   22: ANDI       r16, r15, 31\n"
    "   23: GET_ALIAS  r17, a4\n"
    "   24: BFINS      r18, r17, r16, 12, 7\n"
    "   25: SET_ALIAS  a4, r18\n"
    "   26: GOTO       L1\n"
    "   27: LABEL      L2\n"
    "   28: FCVT       r19, r4\n"
    "   29: STORE      408(r1), r19\n"
    "   30: SET_ALIAS  a2, r19\n"
    "   31: BITCAST    r20, r4\n"
    "   32: SGTUI      r21, r20, 0\n"
    "   33: SRLI       r22, r20, 31\n"
    "   34: BFEXT      r26, r20, 23, 8\n"
    "   35: SEQI       r23, r26, 0\n"
    "   36: SEQI       r24, r26, 255\n"
    "   37: SLLI       r27, r20, 9\n"
    "   38: SEQI       r25, r27, 0\n"
    "   39: AND        r28, r23, r25\n"
    "   40: XORI       r29, r25, 1\n"
    "   41: AND        r30, r24, r29\n"
    "   42: AND        r31, r23, r21\n"
    "   43: OR         r32, r31, r30\n"
    "   44: OR         r33, r28, r30\n"
    "   45: XORI       r34, r33, 1\n"
    "   46: XORI       r35, r22, 1\n"
    "   47: AND        r36, r22, r34\n"
    "   48: AND        r37, r35, r34\n"
    "   49: SLLI       r38, r32, 4\n"
    "   50: SLLI       r39, r36, 3\n"
    "   51: SLLI       r40, r37, 2\n"
    "   52: SLLI       r41, r28, 1\n"
    "   53: OR         r42, r38, r39\n"
    "   54: OR         r43, r40, r41\n"
    "   55: OR         r44, r42, r24\n"
    "   56: OR         r45, r44, r43\n"
    "   57: FTESTEXC   r46, r6, INEXACT\n"
    "   58: SLLI       r47, r46, 5\n"
    "   59: OR         r48, r45, r47\n"
    "   60: GET_ALIAS  r49, a4\n"
    "   61: BFINS      r50, r49, r48, 12, 7\n"
    "   62: SET_ALIAS  a4, r50\n"
    "   63: GOTO_IF_Z  r46, L4\n"
    "   64: GET_ALIAS  r51, a4\n"
    "   65: NOT        r52, r51\n"
    "   66: ORI        r53, r51, 33554432\n"
    "   67: ANDI       r54, r52, 33554432\n"
    "   68: SET_ALIAS  a4, r53\n"
    "   69: GOTO_IF_Z  r54, L5\n"
    "   70: ORI        r55, r53, -2147483648\n"
    "   71: SET_ALIAS  a4, r55\n"
    "   72: LABEL      L5\n"
    "   73: LABEL      L4\n"
    "   74: FTESTEXC   r56, r6, OVERFLOW\n"
    "   75: GOTO_IF_Z  r56, L6\n"
    "   76: GET_ALIAS  r57, a4\n"
    "   77: NOT        r58, r57\n"
    "   78: ORI        r59, r57, 268435456\n"
    "   79: ANDI       r60, r58, 268435456\n"
    "   80: SET_ALIAS  a4, r59\n"
    "   81: GOTO_IF_Z  r60, L7\n"
    "   82: ORI        r61, r59, -2147483648\n"
    "   83: SET_ALIAS  a4, r61\n"
    "   84: LABEL      L7\n"
    "   85: LABEL      L6\n"
    "   86: FTESTEXC   r62, r6, UNDERFLOW\n"
    "   87: GOTO_IF_Z  r62, L1\n"
    "   88: GET_ALIAS  r63, a4\n"
    "   89: NOT        r64, r63\n"
    "   90: ORI        r65, r63, 134217728\n"
    "   91: ANDI       r66, r64, 134217728\n"
    "   92: SET_ALIAS  a4, r65\n"
    "   93: GOTO_IF_Z  r66, L8\n"
    "   94: ORI        r67, r65, -2147483648\n"
    "   95: SET_ALIAS  a4, r67\n"
    "   96: LABEL      L8\n"
    "   97: LABEL      L1\n"
    "   98: LOAD_IMM   r68, 4\n"
    "   99: SET_ALIAS  a1, r68\n"
    "  100: RETURN     r1\n"
    "\n"
    "Alias 1: int32 @ 964(r1)\n"
    "Alias 2: float64 @ 400(r1)\n"
    "Alias 3: float64 @ 416(r1)\n"
    "Alias 4: int32 @ 944(r1)\n"
    "\n"
    "Block 0: <none> --> [0,9] --> 1,5\n"
    "Block 1: 0 --> [10,14] --> 2,3\n"
    "Block 2: 1 --> [15,16] --> 3\n"
    "Block 3: 2,1 --> [17,19] --> 4,5\n"
    "Block 4: 3 --> [20,26] --> 17\n"
    "Block 5: 0,3 --> [27,63] --> 6,9\n"
    "Block 6: 5 --> [64,69] --> 7,8\n"
    "Block 7: 6 --> [70,71] --> 8\n"
    "Block 8: 7,6 --> [72,72] --> 9\n"
    "Block 9: 8,5 --> [73,75] --> 10,13\n"
    "Block 10: 9 --> [76,81] --> 11,12\n"
    "Block 11: 10 --> [82,83] --> 12\n"
    "Block 12: 11,10 --> [84,84] --> 13\n"
    "Block 13: 12,9 --> [85,87] --> 14,17\n"
    "Block 14: 13 --> [88,93] --> 15,16\n"
    "Block 15: 14 --> [94,95] --> 16\n"
    "Block 16: 15,14 --> [96,96] --> 17\n"
    "Block 17: 16,4,13 --> [97,100] --> <none>\n"
    ;

#include "tests/rtl-disasm-test.i"
