/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/guest-ppc/insn/common.h"

static const uint8_t input[] = {
    0xFC,0x20,0x10,0x34,  // frsqrte f1,f2
};

static const unsigned int guest_opt = BINREC_OPT_G_PPC_NATIVE_RECIPROCAL;
static const unsigned int common_opt = 0;

static const bool expected_success = true;

static const char expected[] =
    "[info] Scanning terminated at requested limit 0x3\n"
    "    0: LOAD_ARG   r1, 0\n"
    "    1: LOAD_ARG   r2, 1\n"
    "    2: GET_ALIAS  r3, a3\n"
    "    3: FSQRT      r4, r3\n"
    "    4: LOAD_IMM   r5, 1.0\n"
    "    5: FDIV       r6, r5, r4\n"
    "    6: GET_ALIAS  r7, a4\n"
    "    7: FGETSTATE  r8\n"
    "    8: FCLEAREXC  r9, r8\n"
    "    9: FSETSTATE  r9\n"
    "   10: FTESTEXC   r10, r8, INVALID\n"
    "   11: GOTO_IF_Z  r10, L2\n"
    "   12: BITCAST    r11, r3\n"
    "   13: SLLI       r12, r11, 13\n"
    "   14: BFEXT      r13, r11, 51, 12\n"
    "   15: SEQI       r14, r13, 4094\n"
    "   16: GOTO_IF_Z  r14, L4\n"
    "   17: GOTO_IF_NZ r12, L3\n"
    "   18: LABEL      L4\n"
    "   19: NOT        r15, r7\n"
    "   20: ORI        r16, r7, 512\n"
    "   21: ANDI       r17, r15, 512\n"
    "   22: SET_ALIAS  a4, r16\n"
    "   23: GOTO_IF_Z  r17, L5\n"
    "   24: ORI        r18, r16, -2147483648\n"
    "   25: SET_ALIAS  a4, r18\n"
    "   26: LABEL      L5\n"
    "   27: ANDI       r19, r7, 128\n"
    "   28: GOTO_IF_Z  r19, L6\n"
    "   29: GET_ALIAS  r20, a4\n"
    "   30: BFEXT      r21, r20, 12, 7\n"
    "   31: ANDI       r22, r21, 31\n"
    "   32: GET_ALIAS  r23, a4\n"
    "   33: BFINS      r24, r23, r22, 12, 7\n"
    "   34: SET_ALIAS  a4, r24\n"
    "   35: GOTO       L1\n"
    "   36: LABEL      L6\n"
    "   37: LOAD_IMM   r25, nan(0x8000000000000)\n"
    "   38: SET_ALIAS  a2, r25\n"
    "   39: LOAD_IMM   r26, 17\n"
    "   40: GET_ALIAS  r27, a4\n"
    "   41: BFINS      r28, r27, r26, 12, 7\n"
    "   42: SET_ALIAS  a4, r28\n"
    "   43: GOTO       L1\n"
    "   44: LABEL      L3\n"
    "   45: NOT        r29, r7\n"
    "   46: ORI        r30, r7, 16777216\n"
    "   47: ANDI       r31, r29, 16777216\n"
    "   48: SET_ALIAS  a4, r30\n"
    "   49: GOTO_IF_Z  r31, L7\n"
    "   50: ORI        r32, r30, -2147483648\n"
    "   51: SET_ALIAS  a4, r32\n"
    "   52: LABEL      L7\n"
    "   53: ANDI       r33, r7, 128\n"
    "   54: GOTO_IF_Z  r33, L2\n"
    "   55: LABEL      L8\n"
    "   56: GET_ALIAS  r34, a4\n"
    "   57: BFEXT      r35, r34, 12, 7\n"
    "   58: ANDI       r36, r35, 31\n"
    "   59: GET_ALIAS  r37, a4\n"
    "   60: BFINS      r38, r37, r36, 12, 7\n"
    "   61: SET_ALIAS  a4, r38\n"
    "   62: GOTO       L1\n"
    "   63: LABEL      L2\n"
    "   64: FTESTEXC   r39, r8, ZERO_DIVIDE\n"
    "   65: GOTO_IF_Z  r39, L9\n"
    "   66: NOT        r40, r7\n"
    "   67: ORI        r41, r7, 67108864\n"
    "   68: ANDI       r42, r40, 67108864\n"
    "   69: SET_ALIAS  a4, r41\n"
    "   70: GOTO_IF_Z  r42, L10\n"
    "   71: ORI        r43, r41, -2147483648\n"
    "   72: SET_ALIAS  a4, r43\n"
    "   73: LABEL      L10\n"
    "   74: ANDI       r44, r7, 16\n"
    "   75: GOTO_IF_NZ r44, L8\n"
    "   76: LABEL      L9\n"
    "   77: SET_ALIAS  a2, r6\n"
    "   78: BITCAST    r45, r6\n"
    "   79: SGTUI      r46, r45, 0\n"
    "   80: SLTSI      r47, r45, 0\n"
    "   81: BFEXT      r51, r45, 52, 11\n"
    "   82: SEQI       r48, r51, 0\n"
    "   83: SEQI       r49, r51, 2047\n"
    "   84: SLLI       r52, r45, 12\n"
    "   85: SEQI       r50, r52, 0\n"
    "   86: AND        r53, r48, r50\n"
    "   87: XORI       r54, r50, 1\n"
    "   88: AND        r55, r49, r54\n"
    "   89: AND        r56, r48, r46\n"
    "   90: OR         r57, r56, r55\n"
    "   91: OR         r58, r53, r55\n"
    "   92: XORI       r59, r58, 1\n"
    "   93: XORI       r60, r47, 1\n"
    "   94: AND        r61, r47, r59\n"
    "   95: AND        r62, r60, r59\n"
    "   96: SLLI       r63, r57, 4\n"
    "   97: SLLI       r64, r61, 3\n"
    "   98: SLLI       r65, r62, 2\n"
    "   99: SLLI       r66, r53, 1\n"
    "  100: OR         r67, r63, r64\n"
    "  101: OR         r68, r65, r66\n"
    "  102: OR         r69, r67, r49\n"
    "  103: OR         r70, r69, r68\n"
    "  104: FTESTEXC   r71, r8, INEXACT\n"
    "  105: SLLI       r72, r71, 5\n"
    "  106: OR         r73, r70, r72\n"
    "  107: GET_ALIAS  r74, a4\n"
    "  108: BFINS      r75, r74, r73, 12, 7\n"
    "  109: SET_ALIAS  a4, r75\n"
    "  110: FTESTEXC   r76, r8, OVERFLOW\n"
    "  111: GOTO_IF_Z  r76, L11\n"
    "  112: GET_ALIAS  r77, a4\n"
    "  113: NOT        r78, r77\n"
    "  114: ORI        r79, r77, 268435456\n"
    "  115: ANDI       r80, r78, 268435456\n"
    "  116: SET_ALIAS  a4, r79\n"
    "  117: GOTO_IF_Z  r80, L12\n"
    "  118: ORI        r81, r79, -2147483648\n"
    "  119: SET_ALIAS  a4, r81\n"
    "  120: LABEL      L12\n"
    "  121: LABEL      L11\n"
    "  122: FTESTEXC   r82, r8, UNDERFLOW\n"
    "  123: GOTO_IF_Z  r82, L1\n"
    "  124: GET_ALIAS  r83, a4\n"
    "  125: NOT        r84, r83\n"
    "  126: ORI        r85, r83, 134217728\n"
    "  127: ANDI       r86, r84, 134217728\n"
    "  128: SET_ALIAS  a4, r85\n"
    "  129: GOTO_IF_Z  r86, L13\n"
    "  130: ORI        r87, r85, -2147483648\n"
    "  131: SET_ALIAS  a4, r87\n"
    "  132: LABEL      L13\n"
    "  133: LABEL      L1\n"
    "  134: LOAD_IMM   r88, 4\n"
    "  135: SET_ALIAS  a1, r88\n"
    "  136: RETURN     r1\n"
    "\n"
    "Alias 1: int32 @ 964(r1)\n"
    "Alias 2: float64 @ 400(r1)\n"
    "Alias 3: float64 @ 416(r1)\n"
    "Alias 4: int32 @ 944(r1)\n"
    "\n"
    "Block 0: <none> --> [0,11] --> 1,12\n"
    "Block 1: 0 --> [12,16] --> 2,3\n"
    "Block 2: 1 --> [17,17] --> 3,8\n"
    "Block 3: 2,1 --> [18,23] --> 4,5\n"
    "Block 4: 3 --> [24,25] --> 5\n"
    "Block 5: 4,3 --> [26,28] --> 6,7\n"
    "Block 6: 5 --> [29,35] --> 24\n"
    "Block 7: 5 --> [36,43] --> 24\n"
    "Block 8: 2 --> [44,49] --> 9,10\n"
    "Block 9: 8 --> [50,51] --> 10\n"
    "Block 10: 9,8 --> [52,54] --> 11,12\n"
    "Block 11: 10,15 --> [55,62] --> 24\n"
    "Block 12: 0,10 --> [63,65] --> 13,16\n"
    "Block 13: 12 --> [66,70] --> 14,15\n"
    "Block 14: 13 --> [71,72] --> 15\n"
    "Block 15: 14,13 --> [73,75] --> 16,11\n"
    "Block 16: 15,12 --> [76,111] --> 17,20\n"
    "Block 17: 16 --> [112,117] --> 18,19\n"
    "Block 18: 17 --> [118,119] --> 19\n"
    "Block 19: 18,17 --> [120,120] --> 20\n"
    "Block 20: 19,16 --> [121,123] --> 21,24\n"
    "Block 21: 20 --> [124,129] --> 22,23\n"
    "Block 22: 21 --> [130,131] --> 23\n"
    "Block 23: 22,21 --> [132,132] --> 24\n"
    "Block 24: 23,6,7,11,20 --> [133,136] --> <none>\n"
    ;

#include "tests/rtl-disasm-test.i"
