/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/guest-ppc/insn/common.h"

static const uint8_t input[] = {
    0x10,0x22,0x18,0x2A,  // ps_add f1,f2,f3
};

static const unsigned int guest_opt = BINREC_OPT_G_PPC_IGNORE_FPSCR_VXFOO;
static const unsigned int common_opt = 0;

static const bool expected_success = true;

static const char expected[] =
    "[info] Scanning terminated at requested limit 0x3\n"
    "    0: LOAD_ARG   r1, 0\n"
    "    1: LOAD_ARG   r2, 1\n"
    "    2: GET_ALIAS  r3, a3\n"
    "    3: GET_ALIAS  r4, a4\n"
    "    4: FADD       r5, r3, r4\n"
    "    5: VFCVT      r6, r5\n"
    "    6: LOAD_IMM   r7, 0x1000000\n"
    "    7: VEXTRACT   r8, r6, 1\n"
    "    8: BITCAST    r9, r8\n"
    "    9: SLLI       r10, r9, 1\n"
    "   10: SEQ        r11, r10, r7\n"
    "   11: GOTO_IF_NZ r11, L1\n"
    "   12: VEXTRACT   r12, r6, 0\n"
    "   13: BITCAST    r13, r12\n"
    "   14: SLLI       r14, r13, 1\n"
    "   15: SEQ        r15, r14, r7\n"
    "   16: GOTO_IF_Z  r15, L2\n"
    "   17: LABEL      L1\n"
    "   18: FGETSTATE  r16\n"
    "   19: FSETROUND  r17, r16, TRUNC\n"
    "   20: FSETSTATE  r17\n"
    "   21: FADD       r18, r3, r4\n"
    "   22: VFCVT      r19, r18\n"
    "   23: FGETSTATE  r20\n"
    "   24: FCOPYROUND r21, r20, r16\n"
    "   25: FSETSTATE  r21\n"
    "   26: LABEL      L2\n"
    "   27: FGETSTATE  r22\n"
    "   28: FTESTEXC   r23, r22, INVALID\n"
    "   29: GOTO_IF_Z  r23, L3\n"
    "   30: GET_ALIAS  r24, a2\n"
    "   31: FCLEAREXC  r25, r22\n"
    "   32: FSETSTATE  r25\n"
    "   33: VEXTRACT   r26, r3, 0\n"
    "   34: VEXTRACT   r27, r4, 0\n"
    "   35: FADD       r28, r26, r27\n"
    "   36: FCVT       r29, r28\n"
    "   37: GET_ALIAS  r30, a5\n"
    "   38: FGETSTATE  r31\n"
    "   39: FCLEAREXC  r32, r31\n"
    "   40: FSETSTATE  r32\n"
    "   41: FTESTEXC   r33, r31, INVALID\n"
    "   42: GOTO_IF_Z  r33, L5\n"
    "   43: NOT        r34, r30\n"
    "   44: ORI        r35, r30, 16777216\n"
    "   45: ANDI       r36, r34, 16777216\n"
    "   46: SET_ALIAS  a5, r35\n"
    "   47: GOTO_IF_Z  r36, L6\n"
    "   48: ORI        r37, r35, -2147483648\n"
    "   49: SET_ALIAS  a5, r37\n"
    "   50: LABEL      L6\n"
    "   51: ANDI       r38, r30, 128\n"
    "   52: GOTO_IF_Z  r38, L5\n"
    "   53: GET_ALIAS  r39, a5\n"
    "   54: BFEXT      r40, r39, 12, 7\n"
    "   55: ANDI       r41, r40, 31\n"
    "   56: GET_ALIAS  r42, a5\n"
    "   57: BFINS      r43, r42, r41, 12, 7\n"
    "   58: SET_ALIAS  a5, r43\n"
    "   59: GOTO       L4\n"
    "   60: LABEL      L5\n"
    "   61: FCVT       r44, r29\n"
    "   62: VBROADCAST r45, r44\n"
    "   63: SET_ALIAS  a2, r45\n"
    "   64: BITCAST    r46, r29\n"
    "   65: SGTUI      r47, r46, 0\n"
    "   66: SRLI       r48, r46, 31\n"
    "   67: BFEXT      r52, r46, 23, 8\n"
    "   68: SEQI       r49, r52, 0\n"
    "   69: SEQI       r50, r52, 255\n"
    "   70: SLLI       r53, r46, 9\n"
    "   71: SEQI       r51, r53, 0\n"
    "   72: AND        r54, r49, r51\n"
    "   73: XORI       r55, r51, 1\n"
    "   74: AND        r56, r50, r55\n"
    "   75: AND        r57, r49, r47\n"
    "   76: OR         r58, r57, r56\n"
    "   77: OR         r59, r54, r56\n"
    "   78: XORI       r60, r59, 1\n"
    "   79: XORI       r61, r48, 1\n"
    "   80: AND        r62, r48, r60\n"
    "   81: AND        r63, r61, r60\n"
    "   82: SLLI       r64, r58, 4\n"
    "   83: SLLI       r65, r62, 3\n"
    "   84: SLLI       r66, r63, 2\n"
    "   85: SLLI       r67, r54, 1\n"
    "   86: OR         r68, r64, r65\n"
    "   87: OR         r69, r66, r67\n"
    "   88: OR         r70, r68, r50\n"
    "   89: OR         r71, r70, r69\n"
    "   90: FTESTEXC   r72, r31, INEXACT\n"
    "   91: SLLI       r73, r72, 5\n"
    "   92: OR         r74, r71, r73\n"
    "   93: GET_ALIAS  r75, a5\n"
    "   94: BFINS      r76, r75, r74, 12, 7\n"
    "   95: SET_ALIAS  a5, r76\n"
    "   96: GOTO_IF_Z  r72, L7\n"
    "   97: GET_ALIAS  r77, a5\n"
    "   98: NOT        r78, r77\n"
    "   99: ORI        r79, r77, 33554432\n"
    "  100: ANDI       r80, r78, 33554432\n"
    "  101: SET_ALIAS  a5, r79\n"
    "  102: GOTO_IF_Z  r80, L8\n"
    "  103: ORI        r81, r79, -2147483648\n"
    "  104: SET_ALIAS  a5, r81\n"
    "  105: LABEL      L8\n"
    "  106: LABEL      L7\n"
    "  107: FTESTEXC   r82, r31, OVERFLOW\n"
    "  108: GOTO_IF_Z  r82, L9\n"
    "  109: GET_ALIAS  r83, a5\n"
    "  110: NOT        r84, r83\n"
    "  111: ORI        r85, r83, 268435456\n"
    "  112: ANDI       r86, r84, 268435456\n"
    "  113: SET_ALIAS  a5, r85\n"
    "  114: GOTO_IF_Z  r86, L10\n"
    "  115: ORI        r87, r85, -2147483648\n"
    "  116: SET_ALIAS  a5, r87\n"
    "  117: LABEL      L10\n"
    "  118: LABEL      L9\n"
    "  119: FTESTEXC   r88, r31, UNDERFLOW\n"
    "  120: GOTO_IF_Z  r88, L4\n"
    "  121: GET_ALIAS  r89, a5\n"
    "  122: NOT        r90, r89\n"
    "  123: ORI        r91, r89, 134217728\n"
    "  124: ANDI       r92, r90, 134217728\n"
    "  125: SET_ALIAS  a5, r91\n"
    "  126: GOTO_IF_Z  r92, L11\n"
    "  127: ORI        r93, r91, -2147483648\n"
    "  128: SET_ALIAS  a5, r93\n"
    "  129: LABEL      L11\n"
    "  130: LABEL      L4\n"
    "  131: GET_ALIAS  r94, a2\n"
    "  132: VEXTRACT   r95, r94, 0\n"
    "  133: GET_ALIAS  r96, a5\n"
    "  134: BFEXT      r97, r96, 12, 7\n"
    "  135: VEXTRACT   r98, r3, 1\n"
    "  136: VEXTRACT   r99, r4, 1\n"
    "  137: FADD       r100, r98, r99\n"
    "  138: FCVT       r101, r100\n"
    "  139: GET_ALIAS  r102, a5\n"
    "  140: FGETSTATE  r103\n"
    "  141: FCLEAREXC  r104, r103\n"
    "  142: FSETSTATE  r104\n"
    "  143: FTESTEXC   r105, r103, INVALID\n"
    "  144: GOTO_IF_Z  r105, L13\n"
    "  145: NOT        r106, r102\n"
    "  146: ORI        r107, r102, 16777216\n"
    "  147: ANDI       r108, r106, 16777216\n"
    "  148: SET_ALIAS  a5, r107\n"
    "  149: GOTO_IF_Z  r108, L14\n"
    "  150: ORI        r109, r107, -2147483648\n"
    "  151: SET_ALIAS  a5, r109\n"
    "  152: LABEL      L14\n"
    "  153: ANDI       r110, r102, 128\n"
    "  154: GOTO_IF_Z  r110, L13\n"
    "  155: GET_ALIAS  r111, a5\n"
    "  156: BFEXT      r112, r111, 12, 7\n"
    "  157: ANDI       r113, r112, 31\n"
    "  158: GET_ALIAS  r114, a5\n"
    "  159: BFINS      r115, r114, r113, 12, 7\n"
    "  160: SET_ALIAS  a5, r115\n"
    "  161: GOTO       L12\n"
    "  162: LABEL      L13\n"
    "  163: FCVT       r116, r101\n"
    "  164: VBROADCAST r117, r116\n"
    "  165: SET_ALIAS  a2, r117\n"
    "  166: BITCAST    r118, r101\n"
    "  167: SGTUI      r119, r118, 0\n"
    "  168: SRLI       r120, r118, 31\n"
    "  169: BFEXT      r124, r118, 23, 8\n"
    "  170: SEQI       r121, r124, 0\n"
    "  171: SEQI       r122, r124, 255\n"
    "  172: SLLI       r125, r118, 9\n"
    "  173: SEQI       r123, r125, 0\n"
    "  174: AND        r126, r121, r123\n"
    "  175: XORI       r127, r123, 1\n"
    "  176: AND        r128, r122, r127\n"
    "  177: AND        r129, r121, r119\n"
    "  178: OR         r130, r129, r128\n"
    "  179: OR         r131, r126, r128\n"
    "  180: XORI       r132, r131, 1\n"
    "  181: XORI       r133, r120, 1\n"
    "  182: AND        r134, r120, r132\n"
    "  183: AND        r135, r133, r132\n"
    "  184: SLLI       r136, r130, 4\n"
    "  185: SLLI       r137, r134, 3\n"
    "  186: SLLI       r138, r135, 2\n"
    "  187: SLLI       r139, r126, 1\n"
    "  188: OR         r140, r136, r137\n"
    "  189: OR         r141, r138, r139\n"
    "  190: OR         r142, r140, r122\n"
    "  191: OR         r143, r142, r141\n"
    "  192: FTESTEXC   r144, r103, INEXACT\n"
    "  193: SLLI       r145, r144, 5\n"
    "  194: OR         r146, r143, r145\n"
    "  195: GET_ALIAS  r147, a5\n"
    "  196: BFINS      r148, r147, r146, 12, 7\n"
    "  197: SET_ALIAS  a5, r148\n"
    "  198: GOTO_IF_Z  r144, L15\n"
    "  199: GET_ALIAS  r149, a5\n"
    "  200: NOT        r150, r149\n"
    "  201: ORI        r151, r149, 33554432\n"
    "  202: ANDI       r152, r150, 33554432\n"
    "  203: SET_ALIAS  a5, r151\n"
    "  204: GOTO_IF_Z  r152, L16\n"
    "  205: ORI        r153, r151, -2147483648\n"
    "  206: SET_ALIAS  a5, r153\n"
    "  207: LABEL      L16\n"
    "  208: LABEL      L15\n"
    "  209: FTESTEXC   r154, r103, OVERFLOW\n"
    "  210: GOTO_IF_Z  r154, L17\n"
    "  211: GET_ALIAS  r155, a5\n"
    "  212: NOT        r156, r155\n"
    "  213: ORI        r157, r155, 268435456\n"
    "  214: ANDI       r158, r156, 268435456\n"
    "  215: SET_ALIAS  a5, r157\n"
    "  216: GOTO_IF_Z  r158, L18\n"
    "  217: ORI        r159, r157, -2147483648\n"
    "  218: SET_ALIAS  a5, r159\n"
    "  219: LABEL      L18\n"
    "  220: LABEL      L17\n"
    "  221: FTESTEXC   r160, r103, UNDERFLOW\n"
    "  222: GOTO_IF_Z  r160, L12\n"
    "  223: GET_ALIAS  r161, a5\n"
    "  224: NOT        r162, r161\n"
    "  225: ORI        r163, r161, 134217728\n"
    "  226: ANDI       r164, r162, 134217728\n"
    "  227: SET_ALIAS  a5, r163\n"
    "  228: GOTO_IF_Z  r164, L19\n"
    "  229: ORI        r165, r163, -2147483648\n"
    "  230: SET_ALIAS  a5, r165\n"
    "  231: LABEL      L19\n"
    "  232: LABEL      L12\n"
    "  233: GET_ALIAS  r166, a2\n"
    "  234: VEXTRACT   r167, r166, 0\n"
    "  235: GET_ALIAS  r168, a5\n"
    "  236: BFEXT      r169, r168, 12, 7\n"
    "  237: ANDI       r170, r169, 32\n"
    "  238: OR         r171, r97, r170\n"
    "  239: GET_ALIAS  r172, a5\n"
    "  240: BFINS      r173, r172, r171, 12, 7\n"
    "  241: SET_ALIAS  a5, r173\n"
    "  242: GET_ALIAS  r174, a5\n"
    "  243: ANDI       r175, r174, 128\n"
    "  244: GOTO_IF_Z  r175, L20\n"
    "  245: SET_ALIAS  a2, r24\n"
    "  246: GOTO       L21\n"
    "  247: LABEL      L20\n"
    "  248: VBUILD2    r176, r95, r167\n"
    "  249: SET_ALIAS  a2, r176\n"
    "  250: GOTO       L21\n"
    "  251: LABEL      L3\n"
    "  252: GET_ALIAS  r177, a5\n"
    "  253: FGETSTATE  r178\n"
    "  254: FCLEAREXC  r179, r178\n"
    "  255: FSETSTATE  r179\n"
    "  256: VFCVT      r180, r6\n"
    "  257: SET_ALIAS  a2, r180\n"
    "  258: VEXTRACT   r181, r6, 0\n"
    "  259: BITCAST    r182, r181\n"
    "  260: SGTUI      r183, r182, 0\n"
    "  261: SRLI       r184, r182, 31\n"
    "  262: BFEXT      r188, r182, 23, 8\n"
    "  263: SEQI       r185, r188, 0\n"
    "  264: SEQI       r186, r188, 255\n"
    "  265: SLLI       r189, r182, 9\n"
    "  266: SEQI       r187, r189, 0\n"
    "  267: AND        r190, r185, r187\n"
    "  268: XORI       r191, r187, 1\n"
    "  269: AND        r192, r186, r191\n"
    "  270: AND        r193, r185, r183\n"
    "  271: OR         r194, r193, r192\n"
    "  272: OR         r195, r190, r192\n"
    "  273: XORI       r196, r195, 1\n"
    "  274: XORI       r197, r184, 1\n"
    "  275: AND        r198, r184, r196\n"
    "  276: AND        r199, r197, r196\n"
    "  277: SLLI       r200, r194, 4\n"
    "  278: SLLI       r201, r198, 3\n"
    "  279: SLLI       r202, r199, 2\n"
    "  280: SLLI       r203, r190, 1\n"
    "  281: OR         r204, r200, r201\n"
    "  282: OR         r205, r202, r203\n"
    "  283: OR         r206, r204, r186\n"
    "  284: OR         r207, r206, r205\n"
    "  285: FTESTEXC   r208, r178, INEXACT\n"
    "  286: SLLI       r209, r208, 5\n"
    "  287: OR         r210, r207, r209\n"
    "  288: GET_ALIAS  r211, a5\n"
    "  289: BFINS      r212, r211, r210, 12, 7\n"
    "  290: SET_ALIAS  a5, r212\n"
    "  291: GOTO_IF_Z  r208, L23\n"
    "  292: GET_ALIAS  r213, a5\n"
    "  293: NOT        r214, r213\n"
    "  294: ORI        r215, r213, 33554432\n"
    "  295: ANDI       r216, r214, 33554432\n"
    "  296: SET_ALIAS  a5, r215\n"
    "  297: GOTO_IF_Z  r216, L24\n"
    "  298: ORI        r217, r215, -2147483648\n"
    "  299: SET_ALIAS  a5, r217\n"
    "  300: LABEL      L24\n"
    "  301: LABEL      L23\n"
    "  302: FTESTEXC   r218, r178, OVERFLOW\n"
    "  303: GOTO_IF_Z  r218, L25\n"
    "  304: GET_ALIAS  r219, a5\n"
    "  305: NOT        r220, r219\n"
    "  306: ORI        r221, r219, 268435456\n"
    "  307: ANDI       r222, r220, 268435456\n"
    "  308: SET_ALIAS  a5, r221\n"
    "  309: GOTO_IF_Z  r222, L26\n"
    "  310: ORI        r223, r221, -2147483648\n"
    "  311: SET_ALIAS  a5, r223\n"
    "  312: LABEL      L26\n"
    "  313: LABEL      L25\n"
    "  314: FTESTEXC   r224, r178, UNDERFLOW\n"
    "  315: GOTO_IF_Z  r224, L22\n"
    "  316: GET_ALIAS  r225, a5\n"
    "  317: NOT        r226, r225\n"
    "  318: ORI        r227, r225, 134217728\n"
    "  319: ANDI       r228, r226, 134217728\n"
    "  320: SET_ALIAS  a5, r227\n"
    "  321: GOTO_IF_Z  r228, L27\n"
    "  322: ORI        r229, r227, -2147483648\n"
    "  323: SET_ALIAS  a5, r229\n"
    "  324: LABEL      L27\n"
    "  325: LABEL      L22\n"
    "  326: LABEL      L21\n"
    "  327: LOAD_IMM   r230, 4\n"
    "  328: SET_ALIAS  a1, r230\n"
    "  329: RETURN     r1\n"
    "\n"
    "Alias 1: int32 @ 964(r1)\n"
    "Alias 2: float64[2] @ 400(r1)\n"
    "Alias 3: float64[2] @ 416(r1)\n"
    "Alias 4: float64[2] @ 432(r1)\n"
    "Alias 5: int32 @ 944(r1)\n"
    "\n"
    "Block 0: <none> --> [0,11] --> 1,2\n"
    "Block 1: 0 --> [12,16] --> 2,3\n"
    "Block 2: 1,0 --> [17,25] --> 3\n"
    "Block 3: 2,1 --> [26,29] --> 4,41\n"
    "Block 4: 3 --> [30,42] --> 5,9\n"
    "Block 5: 4 --> [43,47] --> 6,7\n"
    "Block 6: 5 --> [48,49] --> 7\n"
    "Block 7: 6,5 --> [50,52] --> 8,9\n"
    "Block 8: 7 --> [53,59] --> 21\n"
    "Block 9: 4,7 --> [60,96] --> 10,13\n"
    "Block 10: 9 --> [97,102] --> 11,12\n"
    "Block 11: 10 --> [103,104] --> 12\n"
    "Block 12: 11,10 --> [105,105] --> 13\n"
    "Block 13: 12,9 --> [106,108] --> 14,17\n"
    "Block 14: 13 --> [109,114] --> 15,16\n"
    "Block 15: 14 --> [115,116] --> 16\n"
    "Block 16: 15,14 --> [117,117] --> 17\n"
    "Block 17: 16,13 --> [118,120] --> 18,21\n"
    "Block 18: 17 --> [121,126] --> 19,20\n"
    "Block 19: 18 --> [127,128] --> 20\n"
    "Block 20: 19,18 --> [129,129] --> 21\n"
    "Block 21: 20,8,17 --> [130,144] --> 22,26\n"
    "Block 22: 21 --> [145,149] --> 23,24\n"
    "Block 23: 22 --> [150,151] --> 24\n"
    "Block 24: 23,22 --> [152,154] --> 25,26\n"
    "Block 25: 24 --> [155,161] --> 38\n"
    "Block 26: 21,24 --> [162,198] --> 27,30\n"
    "Block 27: 26 --> [199,204] --> 28,29\n"
    "Block 28: 27 --> [205,206] --> 29\n"
    "Block 29: 28,27 --> [207,207] --> 30\n"
    "Block 30: 29,26 --> [208,210] --> 31,34\n"
    "Block 31: 30 --> [211,216] --> 32,33\n"
    "Block 32: 31 --> [217,218] --> 33\n"
    "Block 33: 32,31 --> [219,219] --> 34\n"
    "Block 34: 33,30 --> [220,222] --> 35,38\n"
    "Block 35: 34 --> [223,228] --> 36,37\n"
    "Block 36: 35 --> [229,230] --> 37\n"
    "Block 37: 36,35 --> [231,231] --> 38\n"
    "Block 38: 37,25,34 --> [232,244] --> 39,40\n"
    "Block 39: 38 --> [245,246] --> 54\n"
    "Block 40: 38 --> [247,250] --> 54\n"
    "Block 41: 3 --> [251,291] --> 42,45\n"
    "Block 42: 41 --> [292,297] --> 43,44\n"
    "Block 43: 42 --> [298,299] --> 44\n"
    "Block 44: 43,42 --> [300,300] --> 45\n"
    "Block 45: 44,41 --> [301,303] --> 46,49\n"
    "Block 46: 45 --> [304,309] --> 47,48\n"
    "Block 47: 46 --> [310,311] --> 48\n"
    "Block 48: 47,46 --> [312,312] --> 49\n"
    "Block 49: 48,45 --> [313,315] --> 50,53\n"
    "Block 50: 49 --> [316,321] --> 51,52\n"
    "Block 51: 50 --> [322,323] --> 52\n"
    "Block 52: 51,50 --> [324,324] --> 53\n"
    "Block 53: 52,49 --> [325,325] --> 54\n"
    "Block 54: 53,39,40 --> [326,329] --> <none>\n"
    ;

#include "tests/rtl-disasm-test.i"
