/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/guest-ppc/insn/common.h"

static const uint8_t input[] = {
    0x10,0x22,0x18,0x24,  // ps_div f1,f2,f3
};

static const unsigned int guest_opt = 0;
static const unsigned int common_opt = 0;

static const bool expected_success = true;

static const char expected[] =
    "[info] Scanning terminated at requested limit 0x3\n"
    "    0: LOAD_ARG   r1, 0\n"
    "    1: LOAD_ARG   r2, 1\n"
    "    2: GET_ALIAS  r3, a3\n"
    "    3: GET_ALIAS  r4, a4\n"
    "    4: FDIV       r5, r3, r4\n"
    "    5: VFCVT      r6, r5\n"
    "    6: LOAD_IMM   r7, 0x1000000\n"
    "    7: VEXTRACT   r8, r6, 1\n"
    "    8: BITCAST    r9, r8\n"
    "    9: SLLI       r10, r9, 1\n"
    "   10: SEQ        r11, r10, r7\n"
    "   11: GOTO_IF_NZ r11, L1\n"
    "   12: VEXTRACT   r12, r6, 0\n"
    "   13: BITCAST    r13, r12\n"
    "   14: SLLI       r14, r13, 1\n"
    "   15: SEQ        r15, r14, r7\n"
    "   16: GOTO_IF_Z  r15, L2\n"
    "   17: LABEL      L1\n"
    "   18: FGETSTATE  r16\n"
    "   19: FSETROUND  r17, r16, TRUNC\n"
    "   20: FSETSTATE  r17\n"
    "   21: FDIV       r18, r3, r4\n"
    "   22: VFCVT      r19, r18\n"
    "   23: FGETSTATE  r20\n"
    "   24: FCOPYROUND r21, r20, r16\n"
    "   25: FSETSTATE  r21\n"
    "   26: LABEL      L2\n"
    "   27: FGETSTATE  r22\n"
    "   28: FTESTEXC   r23, r22, INVALID\n"
    "   29: GOTO_IF_Z  r23, L3\n"
    "   30: GET_ALIAS  r24, a2\n"
    "   31: FCLEAREXC  r25, r22\n"
    "   32: FSETSTATE  r25\n"
    "   33: VEXTRACT   r26, r3, 0\n"
    "   34: VEXTRACT   r27, r4, 0\n"
    "   35: FDIV       r28, r26, r27\n"
    "   36: FCVT       r29, r28\n"
    "   37: GET_ALIAS  r30, a5\n"
    "   38: FGETSTATE  r31\n"
    "   39: FCLEAREXC  r32, r31\n"
    "   40: FSETSTATE  r32\n"
    "   41: FTESTEXC   r33, r31, INVALID\n"
    "   42: GOTO_IF_Z  r33, L5\n"
    "   43: BITCAST    r34, r26\n"
    "   44: SLLI       r35, r34, 13\n"
    "   45: BFEXT      r36, r34, 51, 12\n"
    "   46: SEQI       r37, r36, 4094\n"
    "   47: GOTO_IF_Z  r37, L7\n"
    "   48: GOTO_IF_NZ r35, L6\n"
    "   49: LABEL      L7\n"
    "   50: BITCAST    r38, r27\n"
    "   51: SLLI       r39, r38, 13\n"
    "   52: BFEXT      r40, r38, 51, 12\n"
    "   53: SEQI       r41, r40, 4094\n"
    "   54: GOTO_IF_Z  r41, L8\n"
    "   55: GOTO_IF_NZ r39, L6\n"
    "   56: LABEL      L8\n"
    "   57: BITCAST    r42, r26\n"
    "   58: BITCAST    r43, r27\n"
    "   59: OR         r44, r42, r43\n"
    "   60: SLLI       r45, r44, 1\n"
    "   61: GOTO_IF_NZ r45, L9\n"
    "   62: NOT        r46, r30\n"
    "   63: ORI        r47, r30, 2097152\n"
    "   64: ANDI       r48, r46, 2097152\n"
    "   65: SET_ALIAS  a5, r47\n"
    "   66: GOTO_IF_Z  r48, L10\n"
    "   67: ORI        r49, r47, -2147483648\n"
    "   68: SET_ALIAS  a5, r49\n"
    "   69: LABEL      L10\n"
    "   70: GOTO       L11\n"
    "   71: LABEL      L9\n"
    "   72: NOT        r50, r30\n"
    "   73: ORI        r51, r30, 4194304\n"
    "   74: ANDI       r52, r50, 4194304\n"
    "   75: SET_ALIAS  a5, r51\n"
    "   76: GOTO_IF_Z  r52, L12\n"
    "   77: ORI        r53, r51, -2147483648\n"
    "   78: SET_ALIAS  a5, r53\n"
    "   79: LABEL      L12\n"
    "   80: LABEL      L11\n"
    "   81: ANDI       r54, r30, 128\n"
    "   82: GOTO_IF_Z  r54, L13\n"
    "   83: GET_ALIAS  r55, a5\n"
    "   84: BFEXT      r56, r55, 12, 7\n"
    "   85: ANDI       r57, r56, 31\n"
    "   86: GET_ALIAS  r58, a5\n"
    "   87: BFINS      r59, r58, r57, 12, 7\n"
    "   88: SET_ALIAS  a5, r59\n"
    "   89: GOTO       L4\n"
    "   90: LABEL      L13\n"
    "   91: LOAD_IMM   r60, nan(0x400000)\n"
    "   92: FCVT       r61, r60\n"
    "   93: VBROADCAST r62, r61\n"
    "   94: SET_ALIAS  a2, r62\n"
    "   95: LOAD_IMM   r63, 17\n"
    "   96: GET_ALIAS  r64, a5\n"
    "   97: BFINS      r65, r64, r63, 12, 7\n"
    "   98: SET_ALIAS  a5, r65\n"
    "   99: GOTO       L4\n"
    "  100: LABEL      L6\n"
    "  101: NOT        r66, r30\n"
    "  102: ORI        r67, r30, 16777216\n"
    "  103: ANDI       r68, r66, 16777216\n"
    "  104: SET_ALIAS  a5, r67\n"
    "  105: GOTO_IF_Z  r68, L14\n"
    "  106: ORI        r69, r67, -2147483648\n"
    "  107: SET_ALIAS  a5, r69\n"
    "  108: LABEL      L14\n"
    "  109: ANDI       r70, r30, 128\n"
    "  110: GOTO_IF_Z  r70, L5\n"
    "  111: LABEL      L15\n"
    "  112: GET_ALIAS  r71, a5\n"
    "  113: BFEXT      r72, r71, 12, 7\n"
    "  114: ANDI       r73, r72, 31\n"
    "  115: GET_ALIAS  r74, a5\n"
    "  116: BFINS      r75, r74, r73, 12, 7\n"
    "  117: SET_ALIAS  a5, r75\n"
    "  118: GOTO       L4\n"
    "  119: LABEL      L5\n"
    "  120: FTESTEXC   r76, r31, ZERO_DIVIDE\n"
    "  121: GOTO_IF_Z  r76, L16\n"
    "  122: NOT        r77, r30\n"
    "  123: ORI        r78, r30, 67108864\n"
    "  124: ANDI       r79, r77, 67108864\n"
    "  125: SET_ALIAS  a5, r78\n"
    "  126: GOTO_IF_Z  r79, L17\n"
    "  127: ORI        r80, r78, -2147483648\n"
    "  128: SET_ALIAS  a5, r80\n"
    "  129: LABEL      L17\n"
    "  130: ANDI       r81, r30, 16\n"
    "  131: GOTO_IF_NZ r81, L15\n"
    "  132: LABEL      L16\n"
    "  133: FCVT       r82, r29\n"
    "  134: VBROADCAST r83, r82\n"
    "  135: SET_ALIAS  a2, r83\n"
    "  136: BITCAST    r84, r29\n"
    "  137: SGTUI      r85, r84, 0\n"
    "  138: SRLI       r86, r84, 31\n"
    "  139: BFEXT      r90, r84, 23, 8\n"
    "  140: SEQI       r87, r90, 0\n"
    "  141: SEQI       r88, r90, 255\n"
    "  142: SLLI       r91, r84, 9\n"
    "  143: SEQI       r89, r91, 0\n"
    "  144: AND        r92, r87, r89\n"
    "  145: XORI       r93, r89, 1\n"
    "  146: AND        r94, r88, r93\n"
    "  147: AND        r95, r87, r85\n"
    "  148: OR         r96, r95, r94\n"
    "  149: OR         r97, r92, r94\n"
    "  150: XORI       r98, r97, 1\n"
    "  151: XORI       r99, r86, 1\n"
    "  152: AND        r100, r86, r98\n"
    "  153: AND        r101, r99, r98\n"
    "  154: SLLI       r102, r96, 4\n"
    "  155: SLLI       r103, r100, 3\n"
    "  156: SLLI       r104, r101, 2\n"
    "  157: SLLI       r105, r92, 1\n"
    "  158: OR         r106, r102, r103\n"
    "  159: OR         r107, r104, r105\n"
    "  160: OR         r108, r106, r88\n"
    "  161: OR         r109, r108, r107\n"
    "  162: FTESTEXC   r110, r31, INEXACT\n"
    "  163: SLLI       r111, r110, 5\n"
    "  164: OR         r112, r109, r111\n"
    "  165: GET_ALIAS  r113, a5\n"
    "  166: BFINS      r114, r113, r112, 12, 7\n"
    "  167: SET_ALIAS  a5, r114\n"
    "  168: GOTO_IF_Z  r110, L18\n"
    "  169: GET_ALIAS  r115, a5\n"
    "  170: NOT        r116, r115\n"
    "  171: ORI        r117, r115, 33554432\n"
    "  172: ANDI       r118, r116, 33554432\n"
    "  173: SET_ALIAS  a5, r117\n"
    "  174: GOTO_IF_Z  r118, L19\n"
    "  175: ORI        r119, r117, -2147483648\n"
    "  176: SET_ALIAS  a5, r119\n"
    "  177: LABEL      L19\n"
    "  178: LABEL      L18\n"
    "  179: FTESTEXC   r120, r31, OVERFLOW\n"
    "  180: GOTO_IF_Z  r120, L20\n"
    "  181: GET_ALIAS  r121, a5\n"
    "  182: NOT        r122, r121\n"
    "  183: ORI        r123, r121, 268435456\n"
    "  184: ANDI       r124, r122, 268435456\n"
    "  185: SET_ALIAS  a5, r123\n"
    "  186: GOTO_IF_Z  r124, L21\n"
    "  187: ORI        r125, r123, -2147483648\n"
    "  188: SET_ALIAS  a5, r125\n"
    "  189: LABEL      L21\n"
    "  190: LABEL      L20\n"
    "  191: FTESTEXC   r126, r31, UNDERFLOW\n"
    "  192: GOTO_IF_Z  r126, L4\n"
    "  193: GET_ALIAS  r127, a5\n"
    "  194: NOT        r128, r127\n"
    "  195: ORI        r129, r127, 134217728\n"
    "  196: ANDI       r130, r128, 134217728\n"
    "  197: SET_ALIAS  a5, r129\n"
    "  198: GOTO_IF_Z  r130, L22\n"
    "  199: ORI        r131, r129, -2147483648\n"
    "  200: SET_ALIAS  a5, r131\n"
    "  201: LABEL      L22\n"
    "  202: LABEL      L4\n"
    "  203: GET_ALIAS  r132, a2\n"
    "  204: VEXTRACT   r133, r132, 0\n"
    "  205: GET_ALIAS  r134, a5\n"
    "  206: BFEXT      r135, r134, 12, 7\n"
    "  207: VEXTRACT   r136, r3, 1\n"
    "  208: VEXTRACT   r137, r4, 1\n"
    "  209: FDIV       r138, r136, r137\n"
    "  210: FCVT       r139, r138\n"
    "  211: GET_ALIAS  r140, a5\n"
    "  212: FGETSTATE  r141\n"
    "  213: FCLEAREXC  r142, r141\n"
    "  214: FSETSTATE  r142\n"
    "  215: FTESTEXC   r143, r141, INVALID\n"
    "  216: GOTO_IF_Z  r143, L24\n"
    "  217: BITCAST    r144, r136\n"
    "  218: SLLI       r145, r144, 13\n"
    "  219: BFEXT      r146, r144, 51, 12\n"
    "  220: SEQI       r147, r146, 4094\n"
    "  221: GOTO_IF_Z  r147, L26\n"
    "  222: GOTO_IF_NZ r145, L25\n"
    "  223: LABEL      L26\n"
    "  224: BITCAST    r148, r137\n"
    "  225: SLLI       r149, r148, 13\n"
    "  226: BFEXT      r150, r148, 51, 12\n"
    "  227: SEQI       r151, r150, 4094\n"
    "  228: GOTO_IF_Z  r151, L27\n"
    "  229: GOTO_IF_NZ r149, L25\n"
    "  230: LABEL      L27\n"
    "  231: BITCAST    r152, r136\n"
    "  232: BITCAST    r153, r137\n"
    "  233: OR         r154, r152, r153\n"
    "  234: SLLI       r155, r154, 1\n"
    "  235: GOTO_IF_NZ r155, L28\n"
    "  236: NOT        r156, r140\n"
    "  237: ORI        r157, r140, 2097152\n"
    "  238: ANDI       r158, r156, 2097152\n"
    "  239: SET_ALIAS  a5, r157\n"
    "  240: GOTO_IF_Z  r158, L29\n"
    "  241: ORI        r159, r157, -2147483648\n"
    "  242: SET_ALIAS  a5, r159\n"
    "  243: LABEL      L29\n"
    "  244: GOTO       L30\n"
    "  245: LABEL      L28\n"
    "  246: NOT        r160, r140\n"
    "  247: ORI        r161, r140, 4194304\n"
    "  248: ANDI       r162, r160, 4194304\n"
    "  249: SET_ALIAS  a5, r161\n"
    "  250: GOTO_IF_Z  r162, L31\n"
    "  251: ORI        r163, r161, -2147483648\n"
    "  252: SET_ALIAS  a5, r163\n"
    "  253: LABEL      L31\n"
    "  254: LABEL      L30\n"
    "  255: ANDI       r164, r140, 128\n"
    "  256: GOTO_IF_Z  r164, L32\n"
    "  257: GET_ALIAS  r165, a5\n"
    "  258: BFEXT      r166, r165, 12, 7\n"
    "  259: ANDI       r167, r166, 31\n"
    "  260: GET_ALIAS  r168, a5\n"
    "  261: BFINS      r169, r168, r167, 12, 7\n"
    "  262: SET_ALIAS  a5, r169\n"
    "  263: GOTO       L23\n"
    "  264: LABEL      L32\n"
    "  265: LOAD_IMM   r170, nan(0x400000)\n"
    "  266: FCVT       r171, r170\n"
    "  267: VBROADCAST r172, r171\n"
    "  268: SET_ALIAS  a2, r172\n"
    "  269: LOAD_IMM   r173, 17\n"
    "  270: GET_ALIAS  r174, a5\n"
    "  271: BFINS      r175, r174, r173, 12, 7\n"
    "  272: SET_ALIAS  a5, r175\n"
    "  273: GOTO       L23\n"
    "  274: LABEL      L25\n"
    "  275: NOT        r176, r140\n"
    "  276: ORI        r177, r140, 16777216\n"
    "  277: ANDI       r178, r176, 16777216\n"
    "  278: SET_ALIAS  a5, r177\n"
    "  279: GOTO_IF_Z  r178, L33\n"
    "  280: ORI        r179, r177, -2147483648\n"
    "  281: SET_ALIAS  a5, r179\n"
    "  282: LABEL      L33\n"
    "  283: ANDI       r180, r140, 128\n"
    "  284: GOTO_IF_Z  r180, L24\n"
    "  285: LABEL      L34\n"
    "  286: GET_ALIAS  r181, a5\n"
    "  287: BFEXT      r182, r181, 12, 7\n"
    "  288: ANDI       r183, r182, 31\n"
    "  289: GET_ALIAS  r184, a5\n"
    "  290: BFINS      r185, r184, r183, 12, 7\n"
    "  291: SET_ALIAS  a5, r185\n"
    "  292: GOTO       L23\n"
    "  293: LABEL      L24\n"
    "  294: FTESTEXC   r186, r141, ZERO_DIVIDE\n"
    "  295: GOTO_IF_Z  r186, L35\n"
    "  296: NOT        r187, r140\n"
    "  297: ORI        r188, r140, 67108864\n"
    "  298: ANDI       r189, r187, 67108864\n"
    "  299: SET_ALIAS  a5, r188\n"
    "  300: GOTO_IF_Z  r189, L36\n"
    "  301: ORI        r190, r188, -2147483648\n"
    "  302: SET_ALIAS  a5, r190\n"
    "  303: LABEL      L36\n"
    "  304: ANDI       r191, r140, 16\n"
    "  305: GOTO_IF_NZ r191, L34\n"
    "  306: LABEL      L35\n"
    "  307: FCVT       r192, r139\n"
    "  308: VBROADCAST r193, r192\n"
    "  309: SET_ALIAS  a2, r193\n"
    "  310: BITCAST    r194, r139\n"
    "  311: SGTUI      r195, r194, 0\n"
    "  312: SRLI       r196, r194, 31\n"
    "  313: BFEXT      r200, r194, 23, 8\n"
    "  314: SEQI       r197, r200, 0\n"
    "  315: SEQI       r198, r200, 255\n"
    "  316: SLLI       r201, r194, 9\n"
    "  317: SEQI       r199, r201, 0\n"
    "  318: AND        r202, r197, r199\n"
    "  319: XORI       r203, r199, 1\n"
    "  320: AND        r204, r198, r203\n"
    "  321: AND        r205, r197, r195\n"
    "  322: OR         r206, r205, r204\n"
    "  323: OR         r207, r202, r204\n"
    "  324: XORI       r208, r207, 1\n"
    "  325: XORI       r209, r196, 1\n"
    "  326: AND        r210, r196, r208\n"
    "  327: AND        r211, r209, r208\n"
    "  328: SLLI       r212, r206, 4\n"
    "  329: SLLI       r213, r210, 3\n"
    "  330: SLLI       r214, r211, 2\n"
    "  331: SLLI       r215, r202, 1\n"
    "  332: OR         r216, r212, r213\n"
    "  333: OR         r217, r214, r215\n"
    "  334: OR         r218, r216, r198\n"
    "  335: OR         r219, r218, r217\n"
    "  336: FTESTEXC   r220, r141, INEXACT\n"
    "  337: SLLI       r221, r220, 5\n"
    "  338: OR         r222, r219, r221\n"
    "  339: GET_ALIAS  r223, a5\n"
    "  340: BFINS      r224, r223, r222, 12, 7\n"
    "  341: SET_ALIAS  a5, r224\n"
    "  342: GOTO_IF_Z  r220, L37\n"
    "  343: GET_ALIAS  r225, a5\n"
    "  344: NOT        r226, r225\n"
    "  345: ORI        r227, r225, 33554432\n"
    "  346: ANDI       r228, r226, 33554432\n"
    "  347: SET_ALIAS  a5, r227\n"
    "  348: GOTO_IF_Z  r228, L38\n"
    "  349: ORI        r229, r227, -2147483648\n"
    "  350: SET_ALIAS  a5, r229\n"
    "  351: LABEL      L38\n"
    "  352: LABEL      L37\n"
    "  353: FTESTEXC   r230, r141, OVERFLOW\n"
    "  354: GOTO_IF_Z  r230, L39\n"
    "  355: GET_ALIAS  r231, a5\n"
    "  356: NOT        r232, r231\n"
    "  357: ORI        r233, r231, 268435456\n"
    "  358: ANDI       r234, r232, 268435456\n"
    "  359: SET_ALIAS  a5, r233\n"
    "  360: GOTO_IF_Z  r234, L40\n"
    "  361: ORI        r235, r233, -2147483648\n"
    "  362: SET_ALIAS  a5, r235\n"
    "  363: LABEL      L40\n"
    "  364: LABEL      L39\n"
    "  365: FTESTEXC   r236, r141, UNDERFLOW\n"
    "  366: GOTO_IF_Z  r236, L23\n"
    "  367: GET_ALIAS  r237, a5\n"
    "  368: NOT        r238, r237\n"
    "  369: ORI        r239, r237, 134217728\n"
    "  370: ANDI       r240, r238, 134217728\n"
    "  371: SET_ALIAS  a5, r239\n"
    "  372: GOTO_IF_Z  r240, L41\n"
    "  373: ORI        r241, r239, -2147483648\n"
    "  374: SET_ALIAS  a5, r241\n"
    "  375: LABEL      L41\n"
    "  376: LABEL      L23\n"
    "  377: GET_ALIAS  r242, a2\n"
    "  378: VEXTRACT   r243, r242, 0\n"
    "  379: GET_ALIAS  r244, a5\n"
    "  380: BFEXT      r245, r244, 12, 7\n"
    "  381: ANDI       r246, r245, 32\n"
    "  382: OR         r247, r135, r246\n"
    "  383: GET_ALIAS  r248, a5\n"
    "  384: BFINS      r249, r248, r247, 12, 7\n"
    "  385: SET_ALIAS  a5, r249\n"
    "  386: GET_ALIAS  r250, a5\n"
    "  387: ANDI       r251, r250, 128\n"
    "  388: GOTO_IF_Z  r251, L42\n"
    "  389: SET_ALIAS  a2, r24\n"
    "  390: GOTO       L43\n"
    "  391: LABEL      L42\n"
    "  392: VBUILD2    r252, r133, r243\n"
    "  393: SET_ALIAS  a2, r252\n"
    "  394: GOTO       L43\n"
    "  395: LABEL      L3\n"
    "  396: GET_ALIAS  r253, a5\n"
    "  397: FGETSTATE  r254\n"
    "  398: FCLEAREXC  r255, r254\n"
    "  399: FSETSTATE  r255\n"
    "  400: FTESTEXC   r256, r254, ZERO_DIVIDE\n"
    "  401: GOTO_IF_Z  r256, L45\n"
    "  402: NOT        r257, r253\n"
    "  403: ORI        r258, r253, 67108864\n"
    "  404: ANDI       r259, r257, 67108864\n"
    "  405: SET_ALIAS  a5, r258\n"
    "  406: GOTO_IF_Z  r259, L46\n"
    "  407: ORI        r260, r258, -2147483648\n"
    "  408: SET_ALIAS  a5, r260\n"
    "  409: LABEL      L46\n"
    "  410: ANDI       r261, r253, 16\n"
    "  411: GOTO_IF_Z  r261, L45\n"
    "  412: VEXTRACT   r262, r3, 0\n"
    "  413: VEXTRACT   r263, r3, 1\n"
    "  414: VEXTRACT   r264, r4, 0\n"
    "  415: VEXTRACT   r265, r4, 1\n"
    "  416: FDIV       r266, r262, r264\n"
    "  417: FCVT       r267, r266\n"
    "  418: FGETSTATE  r268\n"
    "  419: FDIV       r269, r263, r265\n"
    "  420: FTESTEXC   r270, r268, ZERO_DIVIDE\n"
    "  421: BITCAST    r271, r267\n"
    "  422: SGTUI      r272, r271, 0\n"
    "  423: SRLI       r273, r271, 31\n"
    "  424: BFEXT      r277, r271, 23, 8\n"
    "  425: SEQI       r274, r277, 0\n"
    "  426: SEQI       r275, r277, 255\n"
    "  427: SLLI       r278, r271, 9\n"
    "  428: SEQI       r276, r278, 0\n"
    "  429: AND        r279, r274, r276\n"
    "  430: XORI       r280, r276, 1\n"
    "  431: AND        r281, r275, r280\n"
    "  432: AND        r282, r274, r272\n"
    "  433: OR         r283, r282, r281\n"
    "  434: OR         r284, r279, r281\n"
    "  435: XORI       r285, r284, 1\n"
    "  436: XORI       r286, r273, 1\n"
    "  437: AND        r287, r273, r285\n"
    "  438: AND        r288, r286, r285\n"
    "  439: SLLI       r289, r283, 4\n"
    "  440: SLLI       r290, r287, 3\n"
    "  441: SLLI       r291, r288, 2\n"
    "  442: SLLI       r292, r279, 1\n"
    "  443: OR         r293, r289, r290\n"
    "  444: OR         r294, r291, r292\n"
    "  445: OR         r295, r293, r275\n"
    "  446: OR         r296, r295, r294\n"
    "  447: LOAD_IMM   r297, 0\n"
    "  448: SELECT     r298, r297, r296, r270\n"
    "  449: FCVT       r299, r269\n"
    "  450: FGETSTATE  r300\n"
    "  451: FSETSTATE  r255\n"
    "  452: FTESTEXC   r301, r300, INEXACT\n"
    "  453: GOTO_IF_Z  r301, L47\n"
    "  454: GET_ALIAS  r302, a5\n"
    "  455: NOT        r303, r302\n"
    "  456: ORI        r304, r302, 33554432\n"
    "  457: ANDI       r305, r303, 33554432\n"
    "  458: SET_ALIAS  a5, r304\n"
    "  459: GOTO_IF_Z  r305, L48\n"
    "  460: ORI        r306, r304, -2147483648\n"
    "  461: SET_ALIAS  a5, r306\n"
    "  462: LABEL      L48\n"
    "  463: LABEL      L47\n"
    "  464: FTESTEXC   r307, r300, OVERFLOW\n"
    "  465: GOTO_IF_Z  r307, L49\n"
    "  466: GET_ALIAS  r308, a5\n"
    "  467: NOT        r309, r308\n"
    "  468: ORI        r310, r308, 268435456\n"
    "  469: ANDI       r311, r309, 268435456\n"
    "  470: SET_ALIAS  a5, r310\n"
    "  471: GOTO_IF_Z  r311, L50\n"
    "  472: ORI        r312, r310, -2147483648\n"
    "  473: SET_ALIAS  a5, r312\n"
    "  474: LABEL      L50\n"
    "  475: LABEL      L49\n"
    "  476: FTESTEXC   r313, r300, UNDERFLOW\n"
    "  477: GOTO_IF_Z  r313, L51\n"
    "  478: GET_ALIAS  r314, a5\n"
    "  479: NOT        r315, r314\n"
    "  480: ORI        r316, r314, 134217728\n"
    "  481: ANDI       r317, r315, 134217728\n"
    "  482: SET_ALIAS  a5, r316\n"
    "  483: GOTO_IF_Z  r317, L52\n"
    "  484: ORI        r318, r316, -2147483648\n"
    "  485: SET_ALIAS  a5, r318\n"
    "  486: LABEL      L52\n"
    "  487: LABEL      L51\n"
    "  488: SLLI       r319, r301, 5\n"
    "  489: OR         r320, r319, r298\n"
    "  490: GET_ALIAS  r321, a5\n"
    "  491: BFINS      r322, r321, r320, 12, 7\n"
    "  492: SET_ALIAS  a5, r322\n"
    "  493: GOTO       L44\n"
    "  494: LABEL      L45\n"
    "  495: VFCVT      r323, r6\n"
    "  496: SET_ALIAS  a2, r323\n"
    "  497: VEXTRACT   r324, r6, 0\n"
    "  498: BITCAST    r325, r324\n"
    "  499: SGTUI      r326, r325, 0\n"
    "  500: SRLI       r327, r325, 31\n"
    "  501: BFEXT      r331, r325, 23, 8\n"
    "  502: SEQI       r328, r331, 0\n"
    "  503: SEQI       r329, r331, 255\n"
    "  504: SLLI       r332, r325, 9\n"
    "  505: SEQI       r330, r332, 0\n"
    "  506: AND        r333, r328, r330\n"
    "  507: XORI       r334, r330, 1\n"
    "  508: AND        r335, r329, r334\n"
    "  509: AND        r336, r328, r326\n"
    "  510: OR         r337, r336, r335\n"
    "  511: OR         r338, r333, r335\n"
    "  512: XORI       r339, r338, 1\n"
    "  513: XORI       r340, r327, 1\n"
    "  514: AND        r341, r327, r339\n"
    "  515: AND        r342, r340, r339\n"
    "  516: SLLI       r343, r337, 4\n"
    "  517: SLLI       r344, r341, 3\n"
    "  518: SLLI       r345, r342, 2\n"
    "  519: SLLI       r346, r333, 1\n"
    "  520: OR         r347, r343, r344\n"
    "  521: OR         r348, r345, r346\n"
    "  522: OR         r349, r347, r329\n"
    "  523: OR         r350, r349, r348\n"
    "  524: FTESTEXC   r351, r254, INEXACT\n"
    "  525: SLLI       r352, r351, 5\n"
    "  526: OR         r353, r350, r352\n"
    "  527: GET_ALIAS  r354, a5\n"
    "  528: BFINS      r355, r354, r353, 12, 7\n"
    "  529: SET_ALIAS  a5, r355\n"
    "  530: GOTO_IF_Z  r351, L53\n"
    "  531: GET_ALIAS  r356, a5\n"
    "  532: NOT        r357, r356\n"
    "  533: ORI        r358, r356, 33554432\n"
    "  534: ANDI       r359, r357, 33554432\n"
    "  535: SET_ALIAS  a5, r358\n"
    "  536: GOTO_IF_Z  r359, L54\n"
    "  537: ORI        r360, r358, -2147483648\n"
    "  538: SET_ALIAS  a5, r360\n"
    "  539: LABEL      L54\n"
    "  540: LABEL      L53\n"
    "  541: FTESTEXC   r361, r254, OVERFLOW\n"
    "  542: GOTO_IF_Z  r361, L55\n"
    "  543: GET_ALIAS  r362, a5\n"
    "  544: NOT        r363, r362\n"
    "  545: ORI        r364, r362, 268435456\n"
    "  546: ANDI       r365, r363, 268435456\n"
    "  547: SET_ALIAS  a5, r364\n"
    "  548: GOTO_IF_Z  r365, L56\n"
    "  549: ORI        r366, r364, -2147483648\n"
    "  550: SET_ALIAS  a5, r366\n"
    "  551: LABEL      L56\n"
    "  552: LABEL      L55\n"
    "  553: FTESTEXC   r367, r254, UNDERFLOW\n"
    "  554: GOTO_IF_Z  r367, L44\n"
    "  555: GET_ALIAS  r368, a5\n"
    "  556: NOT        r369, r368\n"
    "  557: ORI        r370, r368, 134217728\n"
    "  558: ANDI       r371, r369, 134217728\n"
    "  559: SET_ALIAS  a5, r370\n"
    "  560: GOTO_IF_Z  r371, L57\n"
    "  561: ORI        r372, r370, -2147483648\n"
    "  562: SET_ALIAS  a5, r372\n"
    "  563: LABEL      L57\n"
    "  564: LABEL      L44\n"
    "  565: LABEL      L43\n"
    "  566: LOAD_IMM   r373, 4\n"
    "  567: SET_ALIAS  a1, r373\n"
    "  568: RETURN     r1\n"
    "\n"
    "Alias 1: int32 @ 964(r1)\n"
    "Alias 2: float64[2] @ 400(r1)\n"
    "Alias 3: float64[2] @ 416(r1)\n"
    "Alias 4: float64[2] @ 432(r1)\n"
    "Alias 5: int32 @ 944(r1)\n"
    "\n"
    "Block 0: <none> --> [0,11] --> 1,2\n"
    "Block 1: 0 --> [12,16] --> 2,3\n"
    "Block 2: 1,0 --> [17,25] --> 3\n"
    "Block 3: 2,1 --> [26,29] --> 4,77\n"
    "Block 4: 3 --> [30,42] --> 5,23\n"
    "Block 5: 4 --> [43,47] --> 6,7\n"
    "Block 6: 5 --> [48,48] --> 7,19\n"
    "Block 7: 6,5 --> [49,54] --> 8,9\n"
    "Block 8: 7 --> [55,55] --> 9,19\n"
    "Block 9: 8,7 --> [56,61] --> 10,13\n"
    "Block 10: 9 --> [62,66] --> 11,12\n"
    "Block 11: 10 --> [67,68] --> 12\n"
    "Block 12: 11,10 --> [69,70] --> 16\n"
    "Block 13: 9 --> [71,76] --> 14,15\n"
    "Block 14: 13 --> [77,78] --> 15\n"
    "Block 15: 14,13 --> [79,79] --> 16\n"
    "Block 16: 15,12 --> [80,82] --> 17,18\n"
    "Block 17: 16 --> [83,89] --> 39\n"
    "Block 18: 16 --> [90,99] --> 39\n"
    "Block 19: 6,8 --> [100,105] --> 20,21\n"
    "Block 20: 19 --> [106,107] --> 21\n"
    "Block 21: 20,19 --> [108,110] --> 22,23\n"
    "Block 22: 21,26 --> [111,118] --> 39\n"
    "Block 23: 4,21 --> [119,121] --> 24,27\n"
    "Block 24: 23 --> [122,126] --> 25,26\n"
    "Block 25: 24 --> [127,128] --> 26\n"
    "Block 26: 25,24 --> [129,131] --> 27,22\n"
    "Block 27: 26,23 --> [132,168] --> 28,31\n"
    "Block 28: 27 --> [169,174] --> 29,30\n"
    "Block 29: 28 --> [175,176] --> 30\n"
    "Block 30: 29,28 --> [177,177] --> 31\n"
    "Block 31: 30,27 --> [178,180] --> 32,35\n"
    "Block 32: 31 --> [181,186] --> 33,34\n"
    "Block 33: 32 --> [187,188] --> 34\n"
    "Block 34: 33,32 --> [189,189] --> 35\n"
    "Block 35: 34,31 --> [190,192] --> 36,39\n"
    "Block 36: 35 --> [193,198] --> 37,38\n"
    "Block 37: 36 --> [199,200] --> 38\n"
    "Block 38: 37,36 --> [201,201] --> 39\n"
    "Block 39: 38,17,18,22,35 --> [202,216] --> 40,58\n"
    "Block 40: 39 --> [217,221] --> 41,42\n"
    "Block 41: 40 --> [222,222] --> 42,54\n"
    "Block 42: 41,40 --> [223,228] --> 43,44\n"
    "Block 43: 42 --> [229,229] --> 44,54\n"
    "Block 44: 43,42 --> [230,235] --> 45,48\n"
    "Block 45: 44 --> [236,240] --> 46,47\n"
    "Block 46: 45 --> [241,242] --> 47\n"
    "Block 47: 46,45 --> [243,244] --> 51\n"
    "Block 48: 44 --> [245,250] --> 49,50\n"
    "Block 49: 48 --> [251,252] --> 50\n"
    "Block 50: 49,48 --> [253,253] --> 51\n"
    "Block 51: 50,47 --> [254,256] --> 52,53\n"
    "Block 52: 51 --> [257,263] --> 74\n"
    "Block 53: 51 --> [264,273] --> 74\n"
    "Block 54: 41,43 --> [274,279] --> 55,56\n"
    "Block 55: 54 --> [280,281] --> 56\n"
    "Block 56: 55,54 --> [282,284] --> 57,58\n"
    "Block 57: 56,61 --> [285,292] --> 74\n"
    "Block 58: 39,56 --> [293,295] --> 59,62\n"
    "Block 59: 58 --> [296,300] --> 60,61\n"
    "Block 60: 59 --> [301,302] --> 61\n"
    "Block 61: 60,59 --> [303,305] --> 62,57\n"
    "Block 62: 61,58 --> [306,342] --> 63,66\n"
    "Block 63: 62 --> [343,348] --> 64,65\n"
    "Block 64: 63 --> [349,350] --> 65\n"
    "Block 65: 64,63 --> [351,351] --> 66\n"
    "Block 66: 65,62 --> [352,354] --> 67,70\n"
    "Block 67: 66 --> [355,360] --> 68,69\n"
    "Block 68: 67 --> [361,362] --> 69\n"
    "Block 69: 68,67 --> [363,363] --> 70\n"
    "Block 70: 69,66 --> [364,366] --> 71,74\n"
    "Block 71: 70 --> [367,372] --> 72,73\n"
    "Block 72: 71 --> [373,374] --> 73\n"
    "Block 73: 72,71 --> [375,375] --> 74\n"
    "Block 74: 73,52,53,57,70 --> [376,388] --> 75,76\n"
    "Block 75: 74 --> [389,390] --> 107\n"
    "Block 76: 74 --> [391,394] --> 107\n"
    "Block 77: 3 --> [395,401] --> 78,94\n"
    "Block 78: 77 --> [402,406] --> 79,80\n"
    "Block 79: 78 --> [407,408] --> 80\n"
    "Block 80: 79,78 --> [409,411] --> 81,94\n"
    "Block 81: 80 --> [412,453] --> 82,85\n"
    "Block 82: 81 --> [454,459] --> 83,84\n"
    "Block 83: 82 --> [460,461] --> 84\n"
    "Block 84: 83,82 --> [462,462] --> 85\n"
    "Block 85: 84,81 --> [463,465] --> 86,89\n"
    "Block 86: 85 --> [466,471] --> 87,88\n"
    "Block 87: 86 --> [472,473] --> 88\n"
    "Block 88: 87,86 --> [474,474] --> 89\n"
    "Block 89: 88,85 --> [475,477] --> 90,93\n"
    "Block 90: 89 --> [478,483] --> 91,92\n"
    "Block 91: 90 --> [484,485] --> 92\n"
    "Block 92: 91,90 --> [486,486] --> 93\n"
    "Block 93: 92,89 --> [487,493] --> 106\n"
    "Block 94: 77,80 --> [494,530] --> 95,98\n"
    "Block 95: 94 --> [531,536] --> 96,97\n"
    "Block 96: 95 --> [537,538] --> 97\n"
    "Block 97: 96,95 --> [539,539] --> 98\n"
    "Block 98: 97,94 --> [540,542] --> 99,102\n"
    "Block 99: 98 --> [543,548] --> 100,101\n"
    "Block 100: 99 --> [549,550] --> 101\n"
    "Block 101: 100,99 --> [551,551] --> 102\n"
    "Block 102: 101,98 --> [552,554] --> 103,106\n"
    "Block 103: 102 --> [555,560] --> 104,105\n"
    "Block 104: 103 --> [561,562] --> 105\n"
    "Block 105: 104,103 --> [563,563] --> 106\n"
    "Block 106: 105,93,102 --> [564,564] --> 107\n"
    "Block 107: 106,75,76 --> [565,568] --> <none>\n"
    ;

#include "tests/rtl-disasm-test.i"
