/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/guest-ppc/insn/common.h"

static const uint8_t input[] = {
    0xC0,0x43,0x00,0x00,  // lfs f2,0(r3)
    0xC0,0x63,0x00,0x04,  // lfs f3,4(r3)
    0xC0,0x83,0x00,0x08,  // lfs f4,8(r3)
    0x10,0x22,0x20,0xFA,  // ps_madd f1,f2,f3,f4
};

static const unsigned int guest_opt = 0;
static const unsigned int common_opt = 0;

static const bool expected_success = true;

static const char expected[] =
    "[info] Scanning terminated at requested limit 0xF\n"
    "    0: LOAD_ARG   r1, 0\n"
    "    1: LOAD_ARG   r2, 1\n"
    "    2: GET_ALIAS  r3, a2\n"
    "    3: ZCAST      r4, r3\n"
    "    4: ADD        r5, r2, r4\n"
    "    5: LOAD_BR    r6, 0(r5)\n"
    "    6: ZCAST      r7, r3\n"
    "    7: ADD        r8, r2, r7\n"
    "    8: LOAD_BR    r9, 4(r8)\n"
    "    9: ZCAST      r10, r3\n"
    "   10: ADD        r11, r2, r10\n"
    "   11: LOAD_BR    r12, 8(r11)\n"
    "   12: VBROADCAST r13, r6\n"
    "   13: VBROADCAST r14, r9\n"
    "   14: VBROADCAST r15, r12\n"
    "   15: VEXTRACT   r16, r13, 0\n"
    "   16: VEXTRACT   r17, r13, 1\n"
    "   17: VEXTRACT   r18, r14, 0\n"
    "   18: VEXTRACT   r19, r14, 1\n"
    "   19: VEXTRACT   r20, r15, 0\n"
    "   20: VEXTRACT   r21, r15, 1\n"
    "   21: GET_ALIAS  r22, a3\n"
    "   22: FMADD      r23, r16, r18, r20\n"
    "   23: FGETSTATE  r24\n"
    "   24: FTESTEXC   r25, r24, INVALID\n"
    "   25: LOAD_IMM   r26, 0.0f\n"
    "   26: FCMP       r27, r16, r26, NUN\n"
    "   27: FCMP       r28, r18, r26, UN\n"
    "   28: FCMP       r29, r20, r26, UN\n"
    "   29: AND        r30, r27, r28\n"
    "   30: AND        r31, r30, r29\n"
    "   31: BITCAST    r32, r20\n"
    "   32: LOAD_IMM   r33, 0x400000\n"
    "   33: OR         r34, r32, r33\n"
    "   34: BITCAST    r35, r34\n"
    "   35: SELECT     r36, r35, r23, r31\n"
    "   36: LOAD_IMM   r37, 0x1000000\n"
    "   37: BITCAST    r38, r36\n"
    "   38: SLLI       r39, r38, 1\n"
    "   39: SEQ        r40, r39, r37\n"
    "   40: GOTO_IF_Z  r40, L1\n"
    "   41: FGETSTATE  r41\n"
    "   42: FSETROUND  r42, r41, TRUNC\n"
    "   43: FSETSTATE  r42\n"
    "   44: FMADD      r43, r16, r18, r20\n"
    "   45: FGETSTATE  r44\n"
    "   46: FCOPYROUND r45, r44, r41\n"
    "   47: FSETSTATE  r45\n"
    "   48: LABEL      L1\n"
    "   49: GET_ALIAS  r46, a7\n"
    "   50: FGETSTATE  r47\n"
    "   51: FCLEAREXC  r48, r47\n"
    "   52: FSETSTATE  r48\n"
    "   53: FTESTEXC   r49, r47, INVALID\n"
    "   54: GOTO_IF_Z  r49, L3\n"
    "   55: BITCAST    r50, r16\n"
    "   56: SLLI       r51, r50, 10\n"
    "   57: BFEXT      r52, r50, 22, 9\n"
    "   58: SEQI       r53, r52, 510\n"
    "   59: GOTO_IF_Z  r53, L5\n"
    "   60: GOTO_IF_NZ r51, L4\n"
    "   61: LABEL      L5\n"
    "   62: BITCAST    r54, r18\n"
    "   63: SLLI       r55, r54, 10\n"
    "   64: BFEXT      r56, r54, 22, 9\n"
    "   65: SEQI       r57, r56, 510\n"
    "   66: GOTO_IF_Z  r57, L6\n"
    "   67: GOTO_IF_NZ r55, L4\n"
    "   68: LABEL      L6\n"
    "   69: BITCAST    r58, r20\n"
    "   70: SLLI       r59, r58, 10\n"
    "   71: BFEXT      r60, r58, 22, 9\n"
    "   72: SEQI       r61, r60, 510\n"
    "   73: GOTO_IF_Z  r61, L8\n"
    "   74: GOTO_IF_NZ r59, L7\n"
    "   75: LABEL      L8\n"
    "   76: BITCAST    r62, r16\n"
    "   77: BITCAST    r63, r18\n"
    "   78: SLLI       r64, r62, 1\n"
    "   79: SLLI       r65, r63, 1\n"
    "   80: SEQI       r66, r64, -16777216\n"
    "   81: GOTO_IF_Z  r66, L9\n"
    "   82: GOTO_IF_Z  r65, L10\n"
    "   83: GOTO       L11\n"
    "   84: LABEL      L9\n"
    "   85: GOTO_IF_NZ r64, L11\n"
    "   86: LABEL      L10\n"
    "   87: NOT        r67, r46\n"
    "   88: ORI        r68, r46, 1048576\n"
    "   89: ANDI       r69, r67, 1048576\n"
    "   90: SET_ALIAS  a7, r68\n"
    "   91: GOTO_IF_Z  r69, L12\n"
    "   92: ORI        r70, r68, -2147483648\n"
    "   93: SET_ALIAS  a7, r70\n"
    "   94: LABEL      L12\n"
    "   95: GOTO       L13\n"
    "   96: LABEL      L11\n"
    "   97: NOT        r71, r46\n"
    "   98: ORI        r72, r46, 8388608\n"
    "   99: ANDI       r73, r71, 8388608\n"
    "  100: SET_ALIAS  a7, r72\n"
    "  101: GOTO_IF_Z  r73, L14\n"
    "  102: ORI        r74, r72, -2147483648\n"
    "  103: SET_ALIAS  a7, r74\n"
    "  104: LABEL      L14\n"
    "  105: LABEL      L13\n"
    "  106: ANDI       r75, r46, 128\n"
    "  107: GOTO_IF_Z  r75, L15\n"
    "  108: GET_ALIAS  r76, a7\n"
    "  109: BFEXT      r77, r76, 12, 7\n"
    "  110: ANDI       r78, r77, 31\n"
    "  111: GET_ALIAS  r79, a7\n"
    "  112: BFINS      r80, r79, r78, 12, 7\n"
    "  113: SET_ALIAS  a7, r80\n"
    "  114: GOTO       L2\n"
    "  115: LABEL      L15\n"
    "  116: LOAD_IMM   r81, nan(0x400000)\n"
    "  117: FCVT       r82, r81\n"
    "  118: VBROADCAST r83, r82\n"
    "  119: SET_ALIAS  a3, r83\n"
    "  120: LOAD_IMM   r84, 17\n"
    "  121: GET_ALIAS  r85, a7\n"
    "  122: BFINS      r86, r85, r84, 12, 7\n"
    "  123: SET_ALIAS  a7, r86\n"
    "  124: GOTO       L2\n"
    "  125: LABEL      L7\n"
    "  126: FMUL       r87, r16, r18\n"
    "  127: FGETSTATE  r88\n"
    "  128: FSETSTATE  r48\n"
    "  129: FTESTEXC   r89, r88, INVALID\n"
    "  130: GOTO_IF_Z  r89, L4\n"
    "  131: NOT        r90, r46\n"
    "  132: ORI        r91, r46, 17825792\n"
    "  133: ANDI       r92, r90, 17825792\n"
    "  134: SET_ALIAS  a7, r91\n"
    "  135: GOTO_IF_Z  r92, L16\n"
    "  136: ORI        r93, r91, -2147483648\n"
    "  137: SET_ALIAS  a7, r93\n"
    "  138: LABEL      L16\n"
    "  139: GOTO       L17\n"
    "  140: LABEL      L4\n"
    "  141: NOT        r94, r46\n"
    "  142: ORI        r95, r46, 16777216\n"
    "  143: ANDI       r96, r94, 16777216\n"
    "  144: SET_ALIAS  a7, r95\n"
    "  145: GOTO_IF_Z  r96, L18\n"
    "  146: ORI        r97, r95, -2147483648\n"
    "  147: SET_ALIAS  a7, r97\n"
    "  148: LABEL      L18\n"
    "  149: LABEL      L17\n"
    "  150: ANDI       r98, r46, 128\n"
    "  151: GOTO_IF_Z  r98, L3\n"
    "  152: GET_ALIAS  r99, a7\n"
    "  153: BFEXT      r100, r99, 12, 7\n"
    "  154: ANDI       r101, r100, 31\n"
    "  155: GET_ALIAS  r102, a7\n"
    "  156: BFINS      r103, r102, r101, 12, 7\n"
    "  157: SET_ALIAS  a7, r103\n"
    "  158: GOTO       L2\n"
    "  159: LABEL      L3\n"
    "  160: FCVT       r104, r36\n"
    "  161: VBROADCAST r105, r104\n"
    "  162: SET_ALIAS  a3, r105\n"
    "  163: BITCAST    r106, r36\n"
    "  164: SGTUI      r107, r106, 0\n"
    "  165: SRLI       r108, r106, 31\n"
    "  166: BFEXT      r112, r106, 23, 8\n"
    "  167: SEQI       r109, r112, 0\n"
    "  168: SEQI       r110, r112, 255\n"
    "  169: SLLI       r113, r106, 9\n"
    "  170: SEQI       r111, r113, 0\n"
    "  171: AND        r114, r109, r111\n"
    "  172: XORI       r115, r111, 1\n"
    "  173: AND        r116, r110, r115\n"
    "  174: AND        r117, r109, r107\n"
    "  175: OR         r118, r117, r116\n"
    "  176: OR         r119, r114, r116\n"
    "  177: XORI       r120, r119, 1\n"
    "  178: XORI       r121, r108, 1\n"
    "  179: AND        r122, r108, r120\n"
    "  180: AND        r123, r121, r120\n"
    "  181: SLLI       r124, r118, 4\n"
    "  182: SLLI       r125, r122, 3\n"
    "  183: SLLI       r126, r123, 2\n"
    "  184: SLLI       r127, r114, 1\n"
    "  185: OR         r128, r124, r125\n"
    "  186: OR         r129, r126, r127\n"
    "  187: OR         r130, r128, r110\n"
    "  188: OR         r131, r130, r129\n"
    "  189: FTESTEXC   r132, r47, INEXACT\n"
    "  190: SLLI       r133, r132, 5\n"
    "  191: OR         r134, r131, r133\n"
    "  192: GET_ALIAS  r135, a7\n"
    "  193: BFINS      r136, r135, r134, 12, 7\n"
    "  194: SET_ALIAS  a7, r136\n"
    "  195: GOTO_IF_Z  r132, L19\n"
    "  196: GET_ALIAS  r137, a7\n"
    "  197: NOT        r138, r137\n"
    "  198: ORI        r139, r137, 33554432\n"
    "  199: ANDI       r140, r138, 33554432\n"
    "  200: SET_ALIAS  a7, r139\n"
    "  201: GOTO_IF_Z  r140, L20\n"
    "  202: ORI        r141, r139, -2147483648\n"
    "  203: SET_ALIAS  a7, r141\n"
    "  204: LABEL      L20\n"
    "  205: LABEL      L19\n"
    "  206: FTESTEXC   r142, r47, OVERFLOW\n"
    "  207: GOTO_IF_Z  r142, L21\n"
    "  208: GET_ALIAS  r143, a7\n"
    "  209: NOT        r144, r143\n"
    "  210: ORI        r145, r143, 268435456\n"
    "  211: ANDI       r146, r144, 268435456\n"
    "  212: SET_ALIAS  a7, r145\n"
    "  213: GOTO_IF_Z  r146, L22\n"
    "  214: ORI        r147, r145, -2147483648\n"
    "  215: SET_ALIAS  a7, r147\n"
    "  216: LABEL      L22\n"
    "  217: LABEL      L21\n"
    "  218: FTESTEXC   r148, r47, UNDERFLOW\n"
    "  219: GOTO_IF_Z  r148, L2\n"
    "  220: GET_ALIAS  r149, a7\n"
    "  221: NOT        r150, r149\n"
    "  222: ORI        r151, r149, 134217728\n"
    "  223: ANDI       r152, r150, 134217728\n"
    "  224: SET_ALIAS  a7, r151\n"
    "  225: GOTO_IF_Z  r152, L23\n"
    "  226: ORI        r153, r151, -2147483648\n"
    "  227: SET_ALIAS  a7, r153\n"
    "  228: LABEL      L23\n"
    "  229: LABEL      L2\n"
    "  230: GET_ALIAS  r154, a3\n"
    "  231: VEXTRACT   r155, r154, 0\n"
    "  232: GET_ALIAS  r156, a7\n"
    "  233: BFEXT      r157, r156, 12, 7\n"
    "  234: FMADD      r158, r17, r19, r21\n"
    "  235: FGETSTATE  r159\n"
    "  236: FTESTEXC   r160, r159, INVALID\n"
    "  237: LOAD_IMM   r161, 0.0f\n"
    "  238: FCMP       r162, r17, r161, NUN\n"
    "  239: FCMP       r163, r19, r161, UN\n"
    "  240: FCMP       r164, r21, r161, UN\n"
    "  241: AND        r165, r162, r163\n"
    "  242: AND        r166, r165, r164\n"
    "  243: BITCAST    r167, r21\n"
    "  244: LOAD_IMM   r168, 0x400000\n"
    "  245: OR         r169, r167, r168\n"
    "  246: BITCAST    r170, r169\n"
    "  247: SELECT     r171, r170, r158, r166\n"
    "  248: LOAD_IMM   r172, 0x1000000\n"
    "  249: BITCAST    r173, r171\n"
    "  250: SLLI       r174, r173, 1\n"
    "  251: SEQ        r175, r174, r172\n"
    "  252: GOTO_IF_Z  r175, L24\n"
    "  253: FGETSTATE  r176\n"
    "  254: FSETROUND  r177, r176, TRUNC\n"
    "  255: FSETSTATE  r177\n"
    "  256: FMADD      r178, r17, r19, r21\n"
    "  257: FGETSTATE  r179\n"
    "  258: FCOPYROUND r180, r179, r176\n"
    "  259: FSETSTATE  r180\n"
    "  260: LABEL      L24\n"
    "  261: GET_ALIAS  r181, a7\n"
    "  262: FGETSTATE  r182\n"
    "  263: FCLEAREXC  r183, r182\n"
    "  264: FSETSTATE  r183\n"
    "  265: FTESTEXC   r184, r182, INVALID\n"
    "  266: GOTO_IF_Z  r184, L26\n"
    "  267: BITCAST    r185, r17\n"
    "  268: SLLI       r186, r185, 10\n"
    "  269: BFEXT      r187, r185, 22, 9\n"
    "  270: SEQI       r188, r187, 510\n"
    "  271: GOTO_IF_Z  r188, L28\n"
    "  272: GOTO_IF_NZ r186, L27\n"
    "  273: LABEL      L28\n"
    "  274: BITCAST    r189, r19\n"
    "  275: SLLI       r190, r189, 10\n"
    "  276: BFEXT      r191, r189, 22, 9\n"
    "  277: SEQI       r192, r191, 510\n"
    "  278: GOTO_IF_Z  r192, L29\n"
    "  279: GOTO_IF_NZ r190, L27\n"
    "  280: LABEL      L29\n"
    "  281: BITCAST    r193, r21\n"
    "  282: SLLI       r194, r193, 10\n"
    "  283: BFEXT      r195, r193, 22, 9\n"
    "  284: SEQI       r196, r195, 510\n"
    "  285: GOTO_IF_Z  r196, L31\n"
    "  286: GOTO_IF_NZ r194, L30\n"
    "  287: LABEL      L31\n"
    "  288: BITCAST    r197, r17\n"
    "  289: BITCAST    r198, r19\n"
    "  290: SLLI       r199, r197, 1\n"
    "  291: SLLI       r200, r198, 1\n"
    "  292: SEQI       r201, r199, -16777216\n"
    "  293: GOTO_IF_Z  r201, L32\n"
    "  294: GOTO_IF_Z  r200, L33\n"
    "  295: GOTO       L34\n"
    "  296: LABEL      L32\n"
    "  297: GOTO_IF_NZ r199, L34\n"
    "  298: LABEL      L33\n"
    "  299: NOT        r202, r181\n"
    "  300: ORI        r203, r181, 1048576\n"
    "  301: ANDI       r204, r202, 1048576\n"
    "  302: SET_ALIAS  a7, r203\n"
    "  303: GOTO_IF_Z  r204, L35\n"
    "  304: ORI        r205, r203, -2147483648\n"
    "  305: SET_ALIAS  a7, r205\n"
    "  306: LABEL      L35\n"
    "  307: GOTO       L36\n"
    "  308: LABEL      L34\n"
    "  309: NOT        r206, r181\n"
    "  310: ORI        r207, r181, 8388608\n"
    "  311: ANDI       r208, r206, 8388608\n"
    "  312: SET_ALIAS  a7, r207\n"
    "  313: GOTO_IF_Z  r208, L37\n"
    "  314: ORI        r209, r207, -2147483648\n"
    "  315: SET_ALIAS  a7, r209\n"
    "  316: LABEL      L37\n"
    "  317: LABEL      L36\n"
    "  318: ANDI       r210, r181, 128\n"
    "  319: GOTO_IF_Z  r210, L38\n"
    "  320: GET_ALIAS  r211, a7\n"
    "  321: BFEXT      r212, r211, 12, 7\n"
    "  322: ANDI       r213, r212, 31\n"
    "  323: GET_ALIAS  r214, a7\n"
    "  324: BFINS      r215, r214, r213, 12, 7\n"
    "  325: SET_ALIAS  a7, r215\n"
    "  326: GOTO       L25\n"
    "  327: LABEL      L38\n"
    "  328: LOAD_IMM   r216, nan(0x400000)\n"
    "  329: FCVT       r217, r216\n"
    "  330: VBROADCAST r218, r217\n"
    "  331: SET_ALIAS  a3, r218\n"
    "  332: LOAD_IMM   r219, 17\n"
    "  333: GET_ALIAS  r220, a7\n"
    "  334: BFINS      r221, r220, r219, 12, 7\n"
    "  335: SET_ALIAS  a7, r221\n"
    "  336: GOTO       L25\n"
    "  337: LABEL      L30\n"
    "  338: FMUL       r222, r17, r19\n"
    "  339: FGETSTATE  r223\n"
    "  340: FSETSTATE  r183\n"
    "  341: FTESTEXC   r224, r223, INVALID\n"
    "  342: GOTO_IF_Z  r224, L27\n"
    "  343: NOT        r225, r181\n"
    "  344: ORI        r226, r181, 17825792\n"
    "  345: ANDI       r227, r225, 17825792\n"
    "  346: SET_ALIAS  a7, r226\n"
    "  347: GOTO_IF_Z  r227, L39\n"
    "  348: ORI        r228, r226, -2147483648\n"
    "  349: SET_ALIAS  a7, r228\n"
    "  350: LABEL      L39\n"
    "  351: GOTO       L40\n"
    "  352: LABEL      L27\n"
    "  353: NOT        r229, r181\n"
    "  354: ORI        r230, r181, 16777216\n"
    "  355: ANDI       r231, r229, 16777216\n"
    "  356: SET_ALIAS  a7, r230\n"
    "  357: GOTO_IF_Z  r231, L41\n"
    "  358: ORI        r232, r230, -2147483648\n"
    "  359: SET_ALIAS  a7, r232\n"
    "  360: LABEL      L41\n"
    "  361: LABEL      L40\n"
    "  362: ANDI       r233, r181, 128\n"
    "  363: GOTO_IF_Z  r233, L26\n"
    "  364: GET_ALIAS  r234, a7\n"
    "  365: BFEXT      r235, r234, 12, 7\n"
    "  366: ANDI       r236, r235, 31\n"
    "  367: GET_ALIAS  r237, a7\n"
    "  368: BFINS      r238, r237, r236, 12, 7\n"
    "  369: SET_ALIAS  a7, r238\n"
    "  370: GOTO       L25\n"
    "  371: LABEL      L26\n"
    "  372: FCVT       r239, r171\n"
    "  373: VBROADCAST r240, r239\n"
    "  374: SET_ALIAS  a3, r240\n"
    "  375: BITCAST    r241, r171\n"
    "  376: SGTUI      r242, r241, 0\n"
    "  377: SRLI       r243, r241, 31\n"
    "  378: BFEXT      r247, r241, 23, 8\n"
    "  379: SEQI       r244, r247, 0\n"
    "  380: SEQI       r245, r247, 255\n"
    "  381: SLLI       r248, r241, 9\n"
    "  382: SEQI       r246, r248, 0\n"
    "  383: AND        r249, r244, r246\n"
    "  384: XORI       r250, r246, 1\n"
    "  385: AND        r251, r245, r250\n"
    "  386: AND        r252, r244, r242\n"
    "  387: OR         r253, r252, r251\n"
    "  388: OR         r254, r249, r251\n"
    "  389: XORI       r255, r254, 1\n"
    "  390: XORI       r256, r243, 1\n"
    "  391: AND        r257, r243, r255\n"
    "  392: AND        r258, r256, r255\n"
    "  393: SLLI       r259, r253, 4\n"
    "  394: SLLI       r260, r257, 3\n"
    "  395: SLLI       r261, r258, 2\n"
    "  396: SLLI       r262, r249, 1\n"
    "  397: OR         r263, r259, r260\n"
    "  398: OR         r264, r261, r262\n"
    "  399: OR         r265, r263, r245\n"
    "  400: OR         r266, r265, r264\n"
    "  401: FTESTEXC   r267, r182, INEXACT\n"
    "  402: SLLI       r268, r267, 5\n"
    "  403: OR         r269, r266, r268\n"
    "  404: GET_ALIAS  r270, a7\n"
    "  405: BFINS      r271, r270, r269, 12, 7\n"
    "  406: SET_ALIAS  a7, r271\n"
    "  407: GOTO_IF_Z  r267, L42\n"
    "  408: GET_ALIAS  r272, a7\n"
    "  409: NOT        r273, r272\n"
    "  410: ORI        r274, r272, 33554432\n"
    "  411: ANDI       r275, r273, 33554432\n"
    "  412: SET_ALIAS  a7, r274\n"
    "  413: GOTO_IF_Z  r275, L43\n"
    "  414: ORI        r276, r274, -2147483648\n"
    "  415: SET_ALIAS  a7, r276\n"
    "  416: LABEL      L43\n"
    "  417: LABEL      L42\n"
    "  418: FTESTEXC   r277, r182, OVERFLOW\n"
    "  419: GOTO_IF_Z  r277, L44\n"
    "  420: GET_ALIAS  r278, a7\n"
    "  421: NOT        r279, r278\n"
    "  422: ORI        r280, r278, 268435456\n"
    "  423: ANDI       r281, r279, 268435456\n"
    "  424: SET_ALIAS  a7, r280\n"
    "  425: GOTO_IF_Z  r281, L45\n"
    "  426: ORI        r282, r280, -2147483648\n"
    "  427: SET_ALIAS  a7, r282\n"
    "  428: LABEL      L45\n"
    "  429: LABEL      L44\n"
    "  430: FTESTEXC   r283, r182, UNDERFLOW\n"
    "  431: GOTO_IF_Z  r283, L25\n"
    "  432: GET_ALIAS  r284, a7\n"
    "  433: NOT        r285, r284\n"
    "  434: ORI        r286, r284, 134217728\n"
    "  435: ANDI       r287, r285, 134217728\n"
    "  436: SET_ALIAS  a7, r286\n"
    "  437: GOTO_IF_Z  r287, L46\n"
    "  438: ORI        r288, r286, -2147483648\n"
    "  439: SET_ALIAS  a7, r288\n"
    "  440: LABEL      L46\n"
    "  441: LABEL      L25\n"
    "  442: GET_ALIAS  r289, a3\n"
    "  443: VEXTRACT   r290, r289, 0\n"
    "  444: GET_ALIAS  r291, a7\n"
    "  445: BFEXT      r292, r291, 12, 7\n"
    "  446: ANDI       r293, r292, 32\n"
    "  447: OR         r294, r157, r293\n"
    "  448: GET_ALIAS  r295, a7\n"
    "  449: BFINS      r296, r295, r294, 12, 7\n"
    "  450: SET_ALIAS  a7, r296\n"
    "  451: OR         r297, r25, r160\n"
    "  452: GOTO_IF_Z  r297, L47\n"
    "  453: GET_ALIAS  r298, a7\n"
    "  454: ANDI       r299, r298, 128\n"
    "  455: GOTO_IF_Z  r299, L47\n"
    "  456: SET_ALIAS  a3, r22\n"
    "  457: GOTO       L48\n"
    "  458: LABEL      L47\n"
    "  459: VBUILD2    r300, r155, r290\n"
    "  460: SET_ALIAS  a3, r300\n"
    "  461: LABEL      L48\n"
    "  462: FGETSTATE  r301\n"
    "  463: FCMP       r302, r6, r6, UN\n"
    "  464: FCVT       r303, r6\n"
    "  465: SET_ALIAS  a8, r303\n"
    "  466: GOTO_IF_Z  r302, L49\n"
    "  467: BITCAST    r304, r6\n"
    "  468: ANDI       r305, r304, 4194304\n"
    "  469: GOTO_IF_NZ r305, L49\n"
    "  470: BITCAST    r306, r303\n"
    "  471: LOAD_IMM   r307, 0x8000000000000\n"
    "  472: XOR        r308, r306, r307\n"
    "  473: BITCAST    r309, r308\n"
    "  474: SET_ALIAS  a8, r309\n"
    "  475: LABEL      L49\n"
    "  476: GET_ALIAS  r310, a8\n"
    "  477: VBROADCAST r311, r310\n"
    "  478: FSETSTATE  r301\n"
    "  479: SET_ALIAS  a4, r311\n"
    "  480: FGETSTATE  r312\n"
    "  481: FCMP       r313, r9, r9, UN\n"
    "  482: FCVT       r314, r9\n"
    "  483: SET_ALIAS  a9, r314\n"
    "  484: GOTO_IF_Z  r313, L50\n"
    "  485: BITCAST    r315, r9\n"
    "  486: ANDI       r316, r315, 4194304\n"
    "  487: GOTO_IF_NZ r316, L50\n"
    "  488: BITCAST    r317, r314\n"
    "  489: LOAD_IMM   r318, 0x8000000000000\n"
    "  490: XOR        r319, r317, r318\n"
    "  491: BITCAST    r320, r319\n"
    "  492: SET_ALIAS  a9, r320\n"
    "  493: LABEL      L50\n"
    "  494: GET_ALIAS  r321, a9\n"
    "  495: VBROADCAST r322, r321\n"
    "  496: FSETSTATE  r312\n"
    "  497: SET_ALIAS  a5, r322\n"
    "  498: FGETSTATE  r323\n"
    "  499: FCMP       r324, r12, r12, UN\n"
    "  500: FCVT       r325, r12\n"
    "  501: SET_ALIAS  a10, r325\n"
    "  502: GOTO_IF_Z  r324, L51\n"
    "  503: BITCAST    r326, r12\n"
    "  504: ANDI       r327, r326, 4194304\n"
    "  505: GOTO_IF_NZ r327, L51\n"
    "  506: BITCAST    r328, r325\n"
    "  507: LOAD_IMM   r329, 0x8000000000000\n"
    "  508: XOR        r330, r328, r329\n"
    "  509: BITCAST    r331, r330\n"
    "  510: SET_ALIAS  a10, r331\n"
    "  511: LABEL      L51\n"
    "  512: GET_ALIAS  r332, a10\n"
    "  513: VBROADCAST r333, r332\n"
    "  514: FSETSTATE  r323\n"
    "  515: SET_ALIAS  a6, r333\n"
    "  516: LOAD_IMM   r334, 16\n"
    "  517: SET_ALIAS  a1, r334\n"
    "  518: RETURN     r1\n"
    "\n"
    "Alias 1: int32 @ 964(r1)\n"
    "Alias 2: int32 @ 268(r1)\n"
    "Alias 3: float64[2] @ 400(r1)\n"
    "Alias 4: float64[2] @ 416(r1)\n"
    "Alias 5: float64[2] @ 432(r1)\n"
    "Alias 6: float64[2] @ 448(r1)\n"
    "Alias 7: int32 @ 944(r1)\n"
    "Alias 8: float64, no bound storage\n"
    "Alias 9: float64, no bound storage\n"
    "Alias 10: float64, no bound storage\n"
    "\n"
    "Block 0: <none> --> [0,40] --> 1,2\n"
    "Block 1: 0 --> [41,47] --> 2\n"
    "Block 2: 1,0 --> [48,54] --> 3,31\n"
    "Block 3: 2 --> [55,59] --> 4,5\n"
    "Block 4: 3 --> [60,60] --> 5,26\n"
    "Block 5: 4,3 --> [61,66] --> 6,7\n"
    "Block 6: 5 --> [67,67] --> 7,26\n"
    "Block 7: 6,5 --> [68,73] --> 8,9\n"
    "Block 8: 7 --> [74,74] --> 9,22\n"
    "Block 9: 8,7 --> [75,81] --> 10,12\n"
    "Block 10: 9 --> [82,82] --> 11,13\n"
    "Block 11: 10 --> [83,83] --> 16\n"
    "Block 12: 9 --> [84,85] --> 13,16\n"
    "Block 13: 12,10 --> [86,91] --> 14,15\n"
    "Block 14: 13 --> [92,93] --> 15\n"
    "Block 15: 14,13 --> [94,95] --> 19\n"
    "Block 16: 11,12 --> [96,101] --> 17,18\n"
    "Block 17: 16 --> [102,103] --> 18\n"
    "Block 18: 17,16 --> [104,104] --> 19\n"
    "Block 19: 18,15 --> [105,107] --> 20,21\n"
    "Block 20: 19 --> [108,114] --> 43\n"
    "Block 21: 19 --> [115,124] --> 43\n"
    "Block 22: 8 --> [125,130] --> 23,26\n"
    "Block 23: 22 --> [131,135] --> 24,25\n"
    "Block 24: 23 --> [136,137] --> 25\n"
    "Block 25: 24,23 --> [138,139] --> 29\n"
    "Block 26: 4,6,22 --> [140,145] --> 27,28\n"
    "Block 27: 26 --> [146,147] --> 28\n"
    "Block 28: 27,26 --> [148,148] --> 29\n"
    "Block 29: 28,25 --> [149,151] --> 30,31\n"
    "Block 30: 29 --> [152,158] --> 43\n"
    "Block 31: 2,29 --> [159,195] --> 32,35\n"
    "Block 32: 31 --> [196,201] --> 33,34\n"
    "Block 33: 32 --> [202,203] --> 34\n"
    "Block 34: 33,32 --> [204,204] --> 35\n"
    "Block 35: 34,31 --> [205,207] --> 36,39\n"
    "Block 36: 35 --> [208,213] --> 37,38\n"
    "Block 37: 36 --> [214,215] --> 38\n"
    "Block 38: 37,36 --> [216,216] --> 39\n"
    "Block 39: 38,35 --> [217,219] --> 40,43\n"
    "Block 40: 39 --> [220,225] --> 41,42\n"
    "Block 41: 40 --> [226,227] --> 42\n"
    "Block 42: 41,40 --> [228,228] --> 43\n"
    "Block 43: 42,20,21,30,39 --> [229,252] --> 44,45\n"
    "Block 44: 43 --> [253,259] --> 45\n"
    "Block 45: 44,43 --> [260,266] --> 46,74\n"
    "Block 46: 45 --> [267,271] --> 47,48\n"
    "Block 47: 46 --> [272,272] --> 48,69\n"
    "Block 48: 47,46 --> [273,278] --> 49,50\n"
    "Block 49: 48 --> [279,279] --> 50,69\n"
    "Block 50: 49,48 --> [280,285] --> 51,52\n"
    "Block 51: 50 --> [286,286] --> 52,65\n"
    "Block 52: 51,50 --> [287,293] --> 53,55\n"
    "Block 53: 52 --> [294,294] --> 54,56\n"
    "Block 54: 53 --> [295,295] --> 59\n"
    "Block 55: 52 --> [296,297] --> 56,59\n"
    "Block 56: 55,53 --> [298,303] --> 57,58\n"
    "Block 57: 56 --> [304,305] --> 58\n"
    "Block 58: 57,56 --> [306,307] --> 62\n"
    "Block 59: 54,55 --> [308,313] --> 60,61\n"
    "Block 60: 59 --> [314,315] --> 61\n"
    "Block 61: 60,59 --> [316,316] --> 62\n"
    "Block 62: 61,58 --> [317,319] --> 63,64\n"
    "Block 63: 62 --> [320,326] --> 86\n"
    "Block 64: 62 --> [327,336] --> 86\n"
    "Block 65: 51 --> [337,342] --> 66,69\n"
    "Block 66: 65 --> [343,347] --> 67,68\n"
    "Block 67: 66 --> [348,349] --> 68\n"
    "Block 68: 67,66 --> [350,351] --> 72\n"
    "Block 69: 47,49,65 --> [352,357] --> 70,71\n"
    "Block 70: 69 --> [358,359] --> 71\n"
    "Block 71: 70,69 --> [360,360] --> 72\n"
    "Block 72: 71,68 --> [361,363] --> 73,74\n"
    "Block 73: 72 --> [364,370] --> 86\n"
    "Block 74: 45,72 --> [371,407] --> 75,78\n"
    "Block 75: 74 --> [408,413] --> 76,77\n"
    "Block 76: 75 --> [414,415] --> 77\n"
    "Block 77: 76,75 --> [416,416] --> 78\n"
    "Block 78: 77,74 --> [417,419] --> 79,82\n"
    "Block 79: 78 --> [420,425] --> 80,81\n"
    "Block 80: 79 --> [426,427] --> 81\n"
    "Block 81: 80,79 --> [428,428] --> 82\n"
    "Block 82: 81,78 --> [429,431] --> 83,86\n"
    "Block 83: 82 --> [432,437] --> 84,85\n"
    "Block 84: 83 --> [438,439] --> 85\n"
    "Block 85: 84,83 --> [440,440] --> 86\n"
    "Block 86: 85,63,64,73,82 --> [441,452] --> 87,89\n"
    "Block 87: 86 --> [453,455] --> 88,89\n"
    "Block 88: 87 --> [456,457] --> 90\n"
    "Block 89: 86,87 --> [458,460] --> 90\n"
    "Block 90: 89,88 --> [461,466] --> 91,93\n"
    "Block 91: 90 --> [467,469] --> 92,93\n"
    "Block 92: 91 --> [470,474] --> 93\n"
    "Block 93: 92,90,91 --> [475,484] --> 94,96\n"
    "Block 94: 93 --> [485,487] --> 95,96\n"
    "Block 95: 94 --> [488,492] --> 96\n"
    "Block 96: 95,93,94 --> [493,502] --> 97,99\n"
    "Block 97: 96 --> [503,505] --> 98,99\n"
    "Block 98: 97 --> [506,510] --> 99\n"
    "Block 99: 98,96,97 --> [511,518] --> <none>\n"
    ;

#include "tests/rtl-disasm-test.i"
