/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/guest-ppc/insn/common.h"

static const uint8_t input[] = {
    0x10,0x22,0x20,0xFA,  // ps_madd f1,f2,f3,f4
};

static const unsigned int guest_opt = BINREC_OPT_G_PPC_NO_FPSCR_STATE
                                    | BINREC_OPT_G_PPC_FAST_FMADDS;
static const unsigned int common_opt = BINREC_OPT_NATIVE_IEEE_NAN;

static const bool expected_success = true;

static const char expected[] =
    "[info] Scanning terminated at requested limit 0x3\n"
    "    0: LOAD_ARG   r1, 0\n"
    "    1: LOAD_ARG   r2, 1\n"
    "    2: GET_ALIAS  r3, a3\n"
    "    3: GET_ALIAS  r4, a4\n"
    "    4: GET_ALIAS  r5, a5\n"
    "    5: VEXTRACT   r6, r3, 0\n"
    "    6: VEXTRACT   r7, r4, 0\n"
    "    7: SET_ALIAS  a6, r6\n"
    "    8: SET_ALIAS  a7, r7\n"
    "    9: BITCAST    r8, r7\n"
    "   10: BITCAST    r9, r6\n"
    "   11: BFEXT      r10, r8, 0, 52\n"
    "   12: GOTO_IF_Z  r10, L1\n"
    "   13: BFEXT      r11, r9, 52, 11\n"
    "   14: BFEXT      r12, r8, 52, 11\n"
    "   15: SEQI       r13, r11, 2047\n"
    "   16: GOTO_IF_NZ r13, L1\n"
    "   17: SEQI       r14, r12, 2047\n"
    "   18: GOTO_IF_NZ r14, L1\n"
    "   19: GOTO_IF_NZ r12, L2\n"
    "   20: CLZ        r15, r10\n"
    "   21: ADDI       r16, r15, -11\n"
    "   22: SUB        r17, r11, r16\n"
    "   23: SGTSI      r18, r17, 0\n"
    "   24: GOTO_IF_Z  r18, L1\n"
    "   25: LOAD_IMM   r19, 0x8000000000000000\n"
    "   26: AND        r20, r8, r19\n"
    "   27: SLL        r21, r10, r16\n"
    "   28: BFINS      r22, r9, r17, 52, 11\n"
    "   29: OR         r23, r20, r21\n"
    "   30: BITCAST    r24, r22\n"
    "   31: BITCAST    r25, r23\n"
    "   32: SET_ALIAS  a6, r24\n"
    "   33: SET_ALIAS  a7, r25\n"
    "   34: LABEL      L2\n"
    "   35: GET_ALIAS  r26, a7\n"
    "   36: BITCAST    r27, r26\n"
    "   37: ANDI       r28, r27, -134217728\n"
    "   38: ANDI       r29, r27, 134217728\n"
    "   39: ADD        r30, r28, r29\n"
    "   40: BITCAST    r31, r30\n"
    "   41: SET_ALIAS  a7, r31\n"
    "   42: BFEXT      r32, r30, 52, 11\n"
    "   43: SEQI       r33, r32, 2047\n"
    "   44: GOTO_IF_Z  r33, L1\n"
    "   45: LOAD_IMM   r34, 0x10000000000000\n"
    "   46: SUB        r35, r30, r34\n"
    "   47: BITCAST    r36, r35\n"
    "   48: SET_ALIAS  a7, r36\n"
    "   49: BFEXT      r37, r9, 52, 11\n"
    "   50: GOTO_IF_Z  r37, L3\n"
    "   51: SGTUI      r38, r37, 2045\n"
    "   52: GOTO_IF_NZ r38, L1\n"
    "   53: ADD        r39, r9, r34\n"
    "   54: BITCAST    r40, r39\n"
    "   55: SET_ALIAS  a6, r40\n"
    "   56: GOTO       L1\n"
    "   57: LABEL      L3\n"
    "   58: SLLI       r41, r9, 1\n"
    "   59: BFINS      r42, r9, r41, 0, 63\n"
    "   60: BITCAST    r43, r42\n"
    "   61: SET_ALIAS  a6, r43\n"
    "   62: LABEL      L1\n"
    "   63: GET_ALIAS  r44, a6\n"
    "   64: GET_ALIAS  r45, a7\n"
    "   65: VINSERT    r46, r3, r44, 0\n"
    "   66: VINSERT    r47, r4, r45, 0\n"
    "   67: FMADD      r48, r46, r47, r5\n"
    "   68: VFCVT      r49, r48\n"
    "   69: VFCVT      r50, r49\n"
    "   70: SET_ALIAS  a2, r50\n"
    "   71: LOAD_IMM   r51, 4\n"
    "   72: SET_ALIAS  a1, r51\n"
    "   73: RETURN     r1\n"
    "\n"
    "Alias 1: int32 @ 964(r1)\n"
    "Alias 2: float64[2] @ 400(r1)\n"
    "Alias 3: float64[2] @ 416(r1)\n"
    "Alias 4: float64[2] @ 432(r1)\n"
    "Alias 5: float64[2] @ 448(r1)\n"
    "Alias 6: float64, no bound storage\n"
    "Alias 7: float64, no bound storage\n"
    "\n"
    "Block 0: <none> --> [0,12] --> 1,11\n"
    "Block 1: 0 --> [13,16] --> 2,11\n"
    "Block 2: 1 --> [17,18] --> 3,11\n"
    "Block 3: 2 --> [19,19] --> 4,6\n"
    "Block 4: 3 --> [20,24] --> 5,11\n"
    "Block 5: 4 --> [25,33] --> 6\n"
    "Block 6: 5,3 --> [34,44] --> 7,11\n"
    "Block 7: 6 --> [45,50] --> 8,10\n"
    "Block 8: 7 --> [51,52] --> 9,11\n"
    "Block 9: 8 --> [53,56] --> 11\n"
    "Block 10: 7 --> [57,61] --> 11\n"
    "Block 11: 10,9,0,1,2,4,6,8 --> [62,73] --> <none>\n"
    ;

#include "tests/rtl-disasm-test.i"
