/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/guest-ppc/insn/common.h"

static const uint8_t input[] = {
    0xFC,0x43,0x20,0x2A,  // fadd f2,f3,f4
    0x10,0x20,0x10,0x90,  // ps_mr f1,f2
};

static const unsigned int guest_opt = BINREC_OPT_G_PPC_NO_FPSCR_STATE;
static const unsigned int common_opt = 0;

static const bool expected_success = true;

static const char expected[] =
    "[info] Scanning terminated at requested limit 0x7\n"
    "    0: LOAD_ARG   r1, 0\n"
    "    1: LOAD_ARG   r2, 1\n"
    "    2: GET_ALIAS  r3, a4\n"
    "    3: GET_ALIAS  r4, a5\n"
    "    4: FADD       r5, r3, r4\n"
    "    5: GET_ALIAS  r6, a3\n"
    "    6: VINSERT    r7, r6, r5, 0\n"
    "    7: VFCMP      r8, r7, r7, UN\n"
    "    8: VFCVT      r9, r7\n"
    "    9: SET_ALIAS  a6, r9\n"
    "   10: GOTO_IF_Z  r8, L1\n"
    "   11: VEXTRACT   r10, r7, 0\n"
    "   12: VEXTRACT   r11, r7, 1\n"
    "   13: BFEXT      r12, r8, 0, 32\n"
    "   14: BFEXT      r13, r8, 32, 32\n"
    "   15: ZCAST      r14, r12\n"
    "   16: ZCAST      r15, r13\n"
    "   17: BITCAST    r16, r10\n"
    "   18: BITCAST    r17, r11\n"
    "   19: NOT        r18, r16\n"
    "   20: NOT        r19, r17\n"
    "   21: LOAD_IMM   r20, 0x8000000000000\n"
    "   22: AND        r21, r18, r20\n"
    "   23: AND        r22, r19, r20\n"
    "   24: VEXTRACT   r23, r9, 0\n"
    "   25: VEXTRACT   r24, r9, 1\n"
    "   26: SRLI       r25, r21, 29\n"
    "   27: SRLI       r26, r22, 29\n"
    "   28: ZCAST      r27, r25\n"
    "   29: ZCAST      r28, r26\n"
    "   30: BITCAST    r29, r23\n"
    "   31: BITCAST    r30, r24\n"
    "   32: AND        r31, r27, r14\n"
    "   33: AND        r32, r28, r15\n"
    "   34: XOR        r33, r29, r31\n"
    "   35: XOR        r34, r30, r32\n"
    "   36: BITCAST    r35, r33\n"
    "   37: BITCAST    r36, r34\n"
    "   38: VBUILD2    r37, r35, r36\n"
    "   39: SET_ALIAS  a6, r37\n"
    "   40: LABEL      L1\n"
    "   41: GET_ALIAS  r38, a6\n"
    "   42: VFCMP      r39, r38, r38, UN\n"
    "   43: VFCVT      r40, r38\n"
    "   44: SET_ALIAS  a7, r40\n"
    "   45: GOTO_IF_Z  r39, L2\n"
    "   46: VEXTRACT   r41, r38, 0\n"
    "   47: VEXTRACT   r42, r38, 1\n"
    "   48: SLLI       r43, r39, 32\n"
    "   49: BITCAST    r44, r41\n"
    "   50: BITCAST    r45, r42\n"
    "   51: NOT        r46, r44\n"
    "   52: NOT        r47, r45\n"
    "   53: ANDI       r48, r46, 4194304\n"
    "   54: ANDI       r49, r47, 4194304\n"
    "   55: VEXTRACT   r50, r40, 0\n"
    "   56: VEXTRACT   r51, r40, 1\n"
    "   57: ZCAST      r52, r48\n"
    "   58: ZCAST      r53, r49\n"
    "   59: SLLI       r54, r52, 29\n"
    "   60: SLLI       r55, r53, 29\n"
    "   61: BITCAST    r56, r50\n"
    "   62: BITCAST    r57, r51\n"
    "   63: AND        r58, r54, r43\n"
    "   64: AND        r59, r55, r39\n"
    "   65: XOR        r60, r56, r58\n"
    "   66: XOR        r61, r57, r59\n"
    "   67: BITCAST    r62, r60\n"
    "   68: BITCAST    r63, r61\n"
    "   69: VBUILD2    r64, r62, r63\n"
    "   70: SET_ALIAS  a7, r64\n"
    "   71: LABEL      L2\n"
    "   72: GET_ALIAS  r65, a7\n"
    "   73: SET_ALIAS  a2, r65\n"
    "   74: GET_ALIAS  r66, a3\n"
    "   75: VINSERT    r67, r66, r5, 0\n"
    "   76: SET_ALIAS  a3, r67\n"
    "   77: LOAD_IMM   r68, 8\n"
    "   78: SET_ALIAS  a1, r68\n"
    "   79: RETURN     r1\n"
    "\n"
    "Alias 1: int32 @ 964(r1)\n"
    "Alias 2: float64[2] @ 400(r1)\n"
    "Alias 3: float64[2] @ 416(r1)\n"
    "Alias 4: float64 @ 432(r1)\n"
    "Alias 5: float64 @ 448(r1)\n"
    "Alias 6: float32[2], no bound storage\n"
    "Alias 7: float64[2], no bound storage\n"
    "\n"
    "Block 0: <none> --> [0,10] --> 1,2\n"
    "Block 1: 0 --> [11,39] --> 2\n"
    "Block 2: 1,0 --> [40,45] --> 3,4\n"
    "Block 3: 2 --> [46,70] --> 4\n"
    "Block 4: 3,2 --> [71,79] --> <none>\n"
    ;

#include "tests/rtl-disasm-test.i"
