/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/guest-ppc/insn/common.h"

static const uint8_t input[] = {
    0x10,0x20,0x10,0x30,  // ps_res f1,f2
};

static const unsigned int guest_opt = 0;
static const unsigned int common_opt = 0;

static const bool expected_success = true;

static const char expected[] =
    "[info] Scanning terminated at requested limit 0x3\n"
    "    0: LOAD_ARG   r1, 0\n"
    "    1: LOAD_ARG   r2, 1\n"
    "    2: GET_ALIAS  r3, a3\n"
    "    3: FGETSTATE  r4\n"
    "    4: VFCMP      r5, r3, r3, UN\n"
    "    5: VFCVT      r6, r3\n"
    "    6: SET_ALIAS  a5, r6\n"
    "    7: GOTO_IF_Z  r5, L1\n"
    "    8: VEXTRACT   r7, r3, 0\n"
    "    9: VEXTRACT   r8, r3, 1\n"
    "   10: BFEXT      r9, r5, 0, 32\n"
    "   11: BFEXT      r10, r5, 32, 32\n"
    "   12: ZCAST      r11, r9\n"
    "   13: ZCAST      r12, r10\n"
    "   14: BITCAST    r13, r7\n"
    "   15: BITCAST    r14, r8\n"
    "   16: NOT        r15, r13\n"
    "   17: NOT        r16, r14\n"
    "   18: LOAD_IMM   r17, 0x8000000000000\n"
    "   19: AND        r18, r15, r17\n"
    "   20: AND        r19, r16, r17\n"
    "   21: VEXTRACT   r20, r6, 0\n"
    "   22: VEXTRACT   r21, r6, 1\n"
    "   23: SRLI       r22, r18, 29\n"
    "   24: SRLI       r23, r19, 29\n"
    "   25: ZCAST      r24, r22\n"
    "   26: ZCAST      r25, r23\n"
    "   27: BITCAST    r26, r20\n"
    "   28: BITCAST    r27, r21\n"
    "   29: AND        r28, r24, r11\n"
    "   30: AND        r29, r25, r12\n"
    "   31: XOR        r30, r26, r28\n"
    "   32: XOR        r31, r27, r29\n"
    "   33: BITCAST    r32, r30\n"
    "   34: BITCAST    r33, r31\n"
    "   35: VBUILD2    r34, r32, r33\n"
    "   36: SET_ALIAS  a5, r34\n"
    "   37: LABEL      L1\n"
    "   38: GET_ALIAS  r35, a5\n"
    "   39: FSETSTATE  r4\n"
    "   40: LOAD_IMM   r36, 0\n"
    "   41: SET_ALIAS  a6, r36\n"
    "   42: VEXTRACT   r37, r35, 0\n"
    "   43: GET_ALIAS  r38, a4\n"
    "   44: BITCAST    r39, r37\n"
    "   45: ANDI       r40, r39, -2147483648\n"
    "   46: BFEXT      r41, r39, 0, 23\n"
    "   47: SET_ALIAS  a9, r41\n"
    "   48: BFEXT      r42, r39, 23, 8\n"
    "   49: SET_ALIAS  a8, r42\n"
    "   50: GOTO_IF_Z  r42, L5\n"
    "   51: SEQI       r43, r42, 255\n"
    "   52: GOTO_IF_NZ r43, L6\n"
    "   53: GOTO       L7\n"
    "   54: LABEL      L6\n"
    "   55: GOTO_IF_NZ r41, L8\n"
    "   56: BITCAST    r44, r40\n"
    "   57: SET_ALIAS  a7, r44\n"
    "   58: LOAD_IMM   r45, 2\n"
    "   59: LOAD_IMM   r46, 18\n"
    "   60: SELECT     r47, r46, r45, r40\n"
    "   61: GET_ALIAS  r48, a4\n"
    "   62: BFINS      r49, r48, r47, 12, 7\n"
    "   63: SET_ALIAS  a4, r49\n"
    "   64: GOTO       L3\n"
    "   65: LABEL      L8\n"
    "   66: ANDI       r50, r41, 4194304\n"
    "   67: GOTO_IF_Z  r50, L9\n"
    "   68: SET_ALIAS  a7, r37\n"
    "   69: LOAD_IMM   r51, 17\n"
    "   70: GET_ALIAS  r52, a4\n"
    "   71: BFINS      r53, r52, r51, 12, 7\n"
    "   72: SET_ALIAS  a4, r53\n"
    "   73: GOTO       L3\n"
    "   74: LABEL      L9\n"
    "   75: GET_ALIAS  r54, a4\n"
    "   76: NOT        r55, r54\n"
    "   77: ORI        r56, r54, 16777216\n"
    "   78: ANDI       r57, r55, 16777216\n"
    "   79: SET_ALIAS  a4, r56\n"
    "   80: GOTO_IF_Z  r57, L10\n"
    "   81: ORI        r58, r56, -2147483648\n"
    "   82: SET_ALIAS  a4, r58\n"
    "   83: LABEL      L10\n"
    "   84: ANDI       r59, r38, 128\n"
    "   85: GOTO_IF_NZ r59, L4\n"
    "   86: ORI        r60, r39, 4194304\n"
    "   87: BITCAST    r61, r60\n"
    "   88: SET_ALIAS  a7, r61\n"
    "   89: LOAD_IMM   r62, 17\n"
    "   90: GET_ALIAS  r63, a4\n"
    "   91: BFINS      r64, r63, r62, 12, 7\n"
    "   92: SET_ALIAS  a4, r64\n"
    "   93: GOTO       L3\n"
    "   94: LABEL      L5\n"
    "   95: GOTO_IF_NZ r41, L11\n"
    "   96: GET_ALIAS  r65, a4\n"
    "   97: NOT        r66, r65\n"
    "   98: ORI        r67, r65, 67108864\n"
    "   99: ANDI       r68, r66, 67108864\n"
    "  100: SET_ALIAS  a4, r67\n"
    "  101: GOTO_IF_Z  r68, L12\n"
    "  102: ORI        r69, r67, -2147483648\n"
    "  103: SET_ALIAS  a4, r69\n"
    "  104: LABEL      L12\n"
    "  105: ANDI       r70, r38, 16\n"
    "  106: GOTO_IF_NZ r70, L4\n"
    "  107: ORI        r71, r40, 2139095040\n"
    "  108: BITCAST    r72, r71\n"
    "  109: SET_ALIAS  a7, r72\n"
    "  110: LOAD_IMM   r73, 5\n"
    "  111: LOAD_IMM   r74, 9\n"
    "  112: SELECT     r75, r74, r73, r40\n"
    "  113: GET_ALIAS  r76, a4\n"
    "  114: BFINS      r77, r76, r75, 12, 7\n"
    "  115: SET_ALIAS  a4, r77\n"
    "  116: GOTO       L3\n"
    "  117: LABEL      L11\n"
    "  118: SLTUI      r78, r41, 2097152\n"
    "  119: GOTO_IF_Z  r78, L13\n"
    "  120: ORI        r79, r40, 2139095039\n"
    "  121: BITCAST    r80, r79\n"
    "  122: SET_ALIAS  a7, r80\n"
    "  123: GET_ALIAS  r81, a4\n"
    "  124: NOT        r82, r81\n"
    "  125: ORI        r83, r81, 268435456\n"
    "  126: ANDI       r84, r82, 268435456\n"
    "  127: SET_ALIAS  a4, r83\n"
    "  128: GOTO_IF_Z  r84, L14\n"
    "  129: ORI        r85, r83, -2147483648\n"
    "  130: SET_ALIAS  a4, r85\n"
    "  131: LABEL      L14\n"
    "  132: LOAD_IMM   r86, 36\n"
    "  133: LOAD_IMM   r87, 40\n"
    "  134: SELECT     r88, r87, r86, r40\n"
    "  135: GET_ALIAS  r89, a4\n"
    "  136: BFINS      r90, r89, r88, 12, 7\n"
    "  137: SET_ALIAS  a4, r90\n"
    "  138: GOTO       L3\n"
    "  139: LABEL      L13\n"
    "  140: SLLI       r91, r41, 1\n"
    "  141: SET_ALIAS  a9, r91\n"
    "  142: ANDI       r92, r91, 8388608\n"
    "  143: GOTO_IF_NZ r92, L15\n"
    "  144: LOAD_IMM   r93, -1\n"
    "  145: SET_ALIAS  a8, r93\n"
    "  146: SLLI       r94, r91, 1\n"
    "  147: SET_ALIAS  a9, r94\n"
    "  148: LABEL      L15\n"
    "  149: GET_ALIAS  r95, a9\n"
    "  150: ANDI       r96, r95, 8388607\n"
    "  151: SET_ALIAS  a9, r96\n"
    "  152: LABEL      L7\n"
    "  153: LOAD       r97, 1016(r1)\n"
    "  154: GET_ALIAS  r98, a9\n"
    "  155: SRLI       r99, r98, 18\n"
    "  156: SLLI       r100, r99, 2\n"
    "  157: LOAD_IMM   r101, 253\n"
    "  158: GET_ALIAS  r102, a8\n"
    "  159: SUB        r103, r101, r102\n"
    "  160: SET_ALIAS  a8, r103\n"
    "  161: ZCAST      r104, r100\n"
    "  162: ADD        r105, r97, r104\n"
    "  163: LOAD_U16   r106, 2(r105)\n"
    "  164: LOAD_U16   r107, 0(r105)\n"
    "  165: BFEXT      r108, r98, 8, 10\n"
    "  166: MUL        r109, r108, r106\n"
    "  167: SLLI       r110, r107, 10\n"
    "  168: SUB        r111, r110, r109\n"
    "  169: ANDI       r112, r111, 1\n"
    "  170: SET_ALIAS  a10, r112\n"
    "  171: SRLI       r113, r111, 1\n"
    "  172: SET_ALIAS  a9, r113\n"
    "  173: GOTO_IF_Z  r103, L16\n"
    "  174: SLTSI      r114, r103, 0\n"
    "  175: GOTO_IF_NZ r114, L17\n"
    "  176: LABEL      L18\n"
    "  177: GET_ALIAS  r115, a9\n"
    "  178: OR         r116, r115, r40\n"
    "  179: GET_ALIAS  r117, a8\n"
    "  180: SLLI       r118, r117, 23\n"
    "  181: OR         r119, r116, r118\n"
    "  182: BITCAST    r120, r119\n"
    "  183: SET_ALIAS  a7, r120\n"
    "  184: BITCAST    r121, r120\n"
    "  185: SGTUI      r122, r121, 0\n"
    "  186: SRLI       r123, r121, 31\n"
    "  187: BFEXT      r127, r121, 23, 8\n"
    "  188: SEQI       r124, r127, 0\n"
    "  189: SEQI       r125, r127, 255\n"
    "  190: SLLI       r128, r121, 9\n"
    "  191: SEQI       r126, r128, 0\n"
    "  192: AND        r129, r124, r126\n"
    "  193: XORI       r130, r126, 1\n"
    "  194: AND        r131, r125, r130\n"
    "  195: AND        r132, r124, r122\n"
    "  196: OR         r133, r132, r131\n"
    "  197: OR         r134, r129, r131\n"
    "  198: XORI       r135, r134, 1\n"
    "  199: XORI       r136, r123, 1\n"
    "  200: AND        r137, r123, r135\n"
    "  201: AND        r138, r136, r135\n"
    "  202: SLLI       r139, r133, 4\n"
    "  203: SLLI       r140, r137, 3\n"
    "  204: SLLI       r141, r138, 2\n"
    "  205: SLLI       r142, r129, 1\n"
    "  206: OR         r143, r139, r140\n"
    "  207: OR         r144, r141, r142\n"
    "  208: OR         r145, r143, r125\n"
    "  209: OR         r146, r145, r144\n"
    "  210: GET_ALIAS  r147, a10\n"
    "  211: SLLI       r148, r147, 5\n"
    "  212: GOTO_IF_Z  r148, L19\n"
    "  213: GOTO_IF_NZ r118, L19\n"
    "  214: GET_ALIAS  r149, a4\n"
    "  215: NOT        r150, r149\n"
    "  216: ORI        r151, r149, 134217728\n"
    "  217: ANDI       r152, r150, 134217728\n"
    "  218: SET_ALIAS  a4, r151\n"
    "  219: GOTO_IF_Z  r152, L20\n"
    "  220: ORI        r153, r151, -2147483648\n"
    "  221: SET_ALIAS  a4, r153\n"
    "  222: LABEL      L20\n"
    "  223: LABEL      L19\n"
    "  224: OR         r154, r148, r146\n"
    "  225: GET_ALIAS  r155, a4\n"
    "  226: BFINS      r156, r155, r154, 12, 7\n"
    "  227: SET_ALIAS  a4, r156\n"
    "  228: GOTO       L3\n"
    "  229: LABEL      L17\n"
    "  230: LOAD_IMM   r157, 0\n"
    "  231: SET_ALIAS  a8, r157\n"
    "  232: GET_ALIAS  r158, a9\n"
    "  233: GET_ALIAS  r159, a10\n"
    "  234: ANDI       r160, r158, 3\n"
    "  235: SGTUI      r161, r160, 0\n"
    "  236: OR         r162, r159, r161\n"
    "  237: SET_ALIAS  a10, r162\n"
    "  238: ORI        r163, r158, 8388608\n"
    "  239: SRLI       r164, r163, 2\n"
    "  240: SET_ALIAS  a9, r164\n"
    "  241: GOTO       L18\n"
    "  242: LABEL      L16\n"
    "  243: GET_ALIAS  r165, a9\n"
    "  244: GET_ALIAS  r166, a10\n"
    "  245: ANDI       r167, r165, 1\n"
    "  246: OR         r168, r166, r167\n"
    "  247: SET_ALIAS  a10, r168\n"
    "  248: ORI        r169, r165, 8388608\n"
    "  249: SRLI       r170, r169, 1\n"
    "  250: SET_ALIAS  a9, r170\n"
    "  251: GOTO       L18\n"
    "  252: LABEL      L4\n"
    "  253: GET_ALIAS  r171, a4\n"
    "  254: BFEXT      r172, r171, 12, 7\n"
    "  255: ANDI       r173, r172, 31\n"
    "  256: GET_ALIAS  r174, a4\n"
    "  257: BFINS      r175, r174, r172, 12, 7\n"
    "  258: SET_ALIAS  a4, r175\n"
    "  259: GOTO       L2\n"
    "  260: LABEL      L3\n"
    "  261: GOTO       L21\n"
    "  262: LABEL      L2\n"
    "  263: LOAD_IMM   r176, 1\n"
    "  264: SET_ALIAS  a6, r176\n"
    "  265: LABEL      L21\n"
    "  266: GET_ALIAS  r177, a4\n"
    "  267: BFEXT      r178, r177, 12, 7\n"
    "  268: GET_ALIAS  r179, a7\n"
    "  269: VEXTRACT   r180, r35, 1\n"
    "  270: GET_ALIAS  r181, a4\n"
    "  271: BITCAST    r182, r180\n"
    "  272: ANDI       r183, r182, -2147483648\n"
    "  273: BFEXT      r184, r182, 0, 23\n"
    "  274: SET_ALIAS  a13, r184\n"
    "  275: BFEXT      r185, r182, 23, 8\n"
    "  276: SET_ALIAS  a12, r185\n"
    "  277: GOTO_IF_Z  r185, L25\n"
    "  278: SEQI       r186, r185, 255\n"
    "  279: GOTO_IF_NZ r186, L26\n"
    "  280: GOTO       L27\n"
    "  281: LABEL      L26\n"
    "  282: GOTO_IF_NZ r184, L28\n"
    "  283: BITCAST    r187, r183\n"
    "  284: SET_ALIAS  a11, r187\n"
    "  285: LOAD_IMM   r188, 2\n"
    "  286: LOAD_IMM   r189, 18\n"
    "  287: SELECT     r190, r189, r188, r183\n"
    "  288: GET_ALIAS  r191, a4\n"
    "  289: BFINS      r192, r191, r190, 12, 7\n"
    "  290: SET_ALIAS  a4, r192\n"
    "  291: GOTO       L23\n"
    "  292: LABEL      L28\n"
    "  293: ANDI       r193, r184, 4194304\n"
    "  294: GOTO_IF_Z  r193, L29\n"
    "  295: SET_ALIAS  a11, r180\n"
    "  296: LOAD_IMM   r194, 17\n"
    "  297: GET_ALIAS  r195, a4\n"
    "  298: BFINS      r196, r195, r194, 12, 7\n"
    "  299: SET_ALIAS  a4, r196\n"
    "  300: GOTO       L23\n"
    "  301: LABEL      L29\n"
    "  302: GET_ALIAS  r197, a4\n"
    "  303: NOT        r198, r197\n"
    "  304: ORI        r199, r197, 16777216\n"
    "  305: ANDI       r200, r198, 16777216\n"
    "  306: SET_ALIAS  a4, r199\n"
    "  307: GOTO_IF_Z  r200, L30\n"
    "  308: ORI        r201, r199, -2147483648\n"
    "  309: SET_ALIAS  a4, r201\n"
    "  310: LABEL      L30\n"
    "  311: ANDI       r202, r181, 128\n"
    "  312: GOTO_IF_NZ r202, L24\n"
    "  313: ORI        r203, r182, 4194304\n"
    "  314: BITCAST    r204, r203\n"
    "  315: SET_ALIAS  a11, r204\n"
    "  316: LOAD_IMM   r205, 17\n"
    "  317: GET_ALIAS  r206, a4\n"
    "  318: BFINS      r207, r206, r205, 12, 7\n"
    "  319: SET_ALIAS  a4, r207\n"
    "  320: GOTO       L23\n"
    "  321: LABEL      L25\n"
    "  322: GOTO_IF_NZ r184, L31\n"
    "  323: GET_ALIAS  r208, a4\n"
    "  324: NOT        r209, r208\n"
    "  325: ORI        r210, r208, 67108864\n"
    "  326: ANDI       r211, r209, 67108864\n"
    "  327: SET_ALIAS  a4, r210\n"
    "  328: GOTO_IF_Z  r211, L32\n"
    "  329: ORI        r212, r210, -2147483648\n"
    "  330: SET_ALIAS  a4, r212\n"
    "  331: LABEL      L32\n"
    "  332: ANDI       r213, r181, 16\n"
    "  333: GOTO_IF_NZ r213, L24\n"
    "  334: ORI        r214, r183, 2139095040\n"
    "  335: BITCAST    r215, r214\n"
    "  336: SET_ALIAS  a11, r215\n"
    "  337: LOAD_IMM   r216, 5\n"
    "  338: LOAD_IMM   r217, 9\n"
    "  339: SELECT     r218, r217, r216, r183\n"
    "  340: GET_ALIAS  r219, a4\n"
    "  341: BFINS      r220, r219, r218, 12, 7\n"
    "  342: SET_ALIAS  a4, r220\n"
    "  343: GOTO       L23\n"
    "  344: LABEL      L31\n"
    "  345: SLTUI      r221, r184, 2097152\n"
    "  346: GOTO_IF_Z  r221, L33\n"
    "  347: ORI        r222, r183, 2139095039\n"
    "  348: BITCAST    r223, r222\n"
    "  349: SET_ALIAS  a11, r223\n"
    "  350: GET_ALIAS  r224, a4\n"
    "  351: NOT        r225, r224\n"
    "  352: ORI        r226, r224, 268435456\n"
    "  353: ANDI       r227, r225, 268435456\n"
    "  354: SET_ALIAS  a4, r226\n"
    "  355: GOTO_IF_Z  r227, L34\n"
    "  356: ORI        r228, r226, -2147483648\n"
    "  357: SET_ALIAS  a4, r228\n"
    "  358: LABEL      L34\n"
    "  359: LOAD_IMM   r229, 36\n"
    "  360: LOAD_IMM   r230, 40\n"
    "  361: SELECT     r231, r230, r229, r183\n"
    "  362: GET_ALIAS  r232, a4\n"
    "  363: BFINS      r233, r232, r231, 12, 7\n"
    "  364: SET_ALIAS  a4, r233\n"
    "  365: GOTO       L23\n"
    "  366: LABEL      L33\n"
    "  367: SLLI       r234, r184, 1\n"
    "  368: SET_ALIAS  a13, r234\n"
    "  369: ANDI       r235, r234, 8388608\n"
    "  370: GOTO_IF_NZ r235, L35\n"
    "  371: LOAD_IMM   r236, -1\n"
    "  372: SET_ALIAS  a12, r236\n"
    "  373: SLLI       r237, r234, 1\n"
    "  374: SET_ALIAS  a13, r237\n"
    "  375: LABEL      L35\n"
    "  376: GET_ALIAS  r238, a13\n"
    "  377: ANDI       r239, r238, 8388607\n"
    "  378: SET_ALIAS  a13, r239\n"
    "  379: LABEL      L27\n"
    "  380: LOAD       r240, 1016(r1)\n"
    "  381: GET_ALIAS  r241, a13\n"
    "  382: SRLI       r242, r241, 18\n"
    "  383: SLLI       r243, r242, 2\n"
    "  384: LOAD_IMM   r244, 253\n"
    "  385: GET_ALIAS  r245, a12\n"
    "  386: SUB        r246, r244, r245\n"
    "  387: SET_ALIAS  a12, r246\n"
    "  388: ZCAST      r247, r243\n"
    "  389: ADD        r248, r240, r247\n"
    "  390: LOAD_U16   r249, 2(r248)\n"
    "  391: LOAD_U16   r250, 0(r248)\n"
    "  392: BFEXT      r251, r241, 8, 10\n"
    "  393: MUL        r252, r251, r249\n"
    "  394: SLLI       r253, r250, 10\n"
    "  395: SUB        r254, r253, r252\n"
    "  396: ANDI       r255, r254, 1\n"
    "  397: SET_ALIAS  a14, r255\n"
    "  398: SRLI       r256, r254, 1\n"
    "  399: SET_ALIAS  a13, r256\n"
    "  400: GOTO_IF_Z  r246, L36\n"
    "  401: SLTSI      r257, r246, 0\n"
    "  402: GOTO_IF_NZ r257, L37\n"
    "  403: LABEL      L38\n"
    "  404: GET_ALIAS  r258, a13\n"
    "  405: OR         r259, r258, r183\n"
    "  406: GET_ALIAS  r260, a12\n"
    "  407: SLLI       r261, r260, 23\n"
    "  408: OR         r262, r259, r261\n"
    "  409: BITCAST    r263, r262\n"
    "  410: SET_ALIAS  a11, r263\n"
    "  411: BITCAST    r264, r263\n"
    "  412: SGTUI      r265, r264, 0\n"
    "  413: SRLI       r266, r264, 31\n"
    "  414: BFEXT      r270, r264, 23, 8\n"
    "  415: SEQI       r267, r270, 0\n"
    "  416: SEQI       r268, r270, 255\n"
    "  417: SLLI       r271, r264, 9\n"
    "  418: SEQI       r269, r271, 0\n"
    "  419: AND        r272, r267, r269\n"
    "  420: XORI       r273, r269, 1\n"
    "  421: AND        r274, r268, r273\n"
    "  422: AND        r275, r267, r265\n"
    "  423: OR         r276, r275, r274\n"
    "  424: OR         r277, r272, r274\n"
    "  425: XORI       r278, r277, 1\n"
    "  426: XORI       r279, r266, 1\n"
    "  427: AND        r280, r266, r278\n"
    "  428: AND        r281, r279, r278\n"
    "  429: SLLI       r282, r276, 4\n"
    "  430: SLLI       r283, r280, 3\n"
    "  431: SLLI       r284, r281, 2\n"
    "  432: SLLI       r285, r272, 1\n"
    "  433: OR         r286, r282, r283\n"
    "  434: OR         r287, r284, r285\n"
    "  435: OR         r288, r286, r268\n"
    "  436: OR         r289, r288, r287\n"
    "  437: GET_ALIAS  r290, a14\n"
    "  438: SLLI       r291, r290, 5\n"
    "  439: GOTO_IF_Z  r291, L39\n"
    "  440: GOTO_IF_NZ r261, L39\n"
    "  441: GET_ALIAS  r292, a4\n"
    "  442: NOT        r293, r292\n"
    "  443: ORI        r294, r292, 134217728\n"
    "  444: ANDI       r295, r293, 134217728\n"
    "  445: SET_ALIAS  a4, r294\n"
    "  446: GOTO_IF_Z  r295, L40\n"
    "  447: ORI        r296, r294, -2147483648\n"
    "  448: SET_ALIAS  a4, r296\n"
    "  449: LABEL      L40\n"
    "  450: LABEL      L39\n"
    "  451: OR         r297, r291, r289\n"
    "  452: GET_ALIAS  r298, a4\n"
    "  453: BFINS      r299, r298, r297, 12, 7\n"
    "  454: SET_ALIAS  a4, r299\n"
    "  455: GOTO       L23\n"
    "  456: LABEL      L37\n"
    "  457: LOAD_IMM   r300, 0\n"
    "  458: SET_ALIAS  a12, r300\n"
    "  459: GET_ALIAS  r301, a13\n"
    "  460: GET_ALIAS  r302, a14\n"
    "  461: ANDI       r303, r301, 3\n"
    "  462: SGTUI      r304, r303, 0\n"
    "  463: OR         r305, r302, r304\n"
    "  464: SET_ALIAS  a14, r305\n"
    "  465: ORI        r306, r301, 8388608\n"
    "  466: SRLI       r307, r306, 2\n"
    "  467: SET_ALIAS  a13, r307\n"
    "  468: GOTO       L38\n"
    "  469: LABEL      L36\n"
    "  470: GET_ALIAS  r308, a13\n"
    "  471: GET_ALIAS  r309, a14\n"
    "  472: ANDI       r310, r308, 1\n"
    "  473: OR         r311, r309, r310\n"
    "  474: SET_ALIAS  a14, r311\n"
    "  475: ORI        r312, r308, 8388608\n"
    "  476: SRLI       r313, r312, 1\n"
    "  477: SET_ALIAS  a13, r313\n"
    "  478: GOTO       L38\n"
    "  479: LABEL      L24\n"
    "  480: GET_ALIAS  r314, a4\n"
    "  481: BFEXT      r315, r314, 12, 7\n"
    "  482: ANDI       r316, r315, 31\n"
    "  483: GET_ALIAS  r317, a4\n"
    "  484: BFINS      r318, r317, r315, 12, 7\n"
    "  485: SET_ALIAS  a4, r318\n"
    "  486: GOTO       L22\n"
    "  487: LABEL      L23\n"
    "  488: GOTO       L41\n"
    "  489: LABEL      L22\n"
    "  490: LOAD_IMM   r319, 1\n"
    "  491: SET_ALIAS  a6, r319\n"
    "  492: LABEL      L41\n"
    "  493: GET_ALIAS  r320, a4\n"
    "  494: BFEXT      r321, r320, 12, 7\n"
    "  495: ANDI       r322, r321, 32\n"
    "  496: OR         r323, r178, r322\n"
    "  497: GET_ALIAS  r324, a4\n"
    "  498: BFINS      r325, r324, r323, 12, 7\n"
    "  499: SET_ALIAS  a4, r325\n"
    "  500: GET_ALIAS  r326, a11\n"
    "  501: GET_ALIAS  r327, a6\n"
    "  502: GOTO_IF_NZ r327, L42\n"
    "  503: VBUILD2    r328, r179, r326\n"
    "  504: VFCVT      r329, r328\n"
    "  505: SET_ALIAS  a2, r329\n"
    "  506: LABEL      L42\n"
    "  507: LOAD_IMM   r330, 4\n"
    "  508: SET_ALIAS  a1, r330\n"
    "  509: RETURN     r1\n"
    "\n"
    "Alias 1: int32 @ 964(r1)\n"
    "Alias 2: float64[2] @ 400(r1)\n"
    "Alias 3: float64[2] @ 416(r1)\n"
    "Alias 4: int32 @ 944(r1)\n"
    "Alias 5: float32[2], no bound storage\n"
    "Alias 6: int32, no bound storage\n"
    "Alias 7: float32, no bound storage\n"
    "Alias 8: int32, no bound storage\n"
    "Alias 9: int32, no bound storage\n"
    "Alias 10: int32, no bound storage\n"
    "Alias 11: float32, no bound storage\n"
    "Alias 12: int32, no bound storage\n"
    "Alias 13: int32, no bound storage\n"
    "Alias 14: int32, no bound storage\n"
    "\n"
    "Block 0: <none> --> [0,7] --> 1,2\n"
    "Block 1: 0 --> [8,36] --> 2\n"
    "Block 2: 1,0 --> [37,50] --> 3,13\n"
    "Block 3: 2 --> [51,52] --> 4,5\n"
    "Block 4: 3 --> [53,53] --> 25\n"
    "Block 5: 3 --> [54,55] --> 6,7\n"
    "Block 6: 5 --> [56,64] --> 36\n"
    "Block 7: 5 --> [65,67] --> 8,9\n"
    "Block 8: 7 --> [68,73] --> 36\n"
    "Block 9: 7 --> [74,80] --> 10,11\n"
    "Block 10: 9 --> [81,82] --> 11\n"
    "Block 11: 10,9 --> [83,85] --> 12,35\n"
    "Block 12: 11 --> [86,93] --> 36\n"
    "Block 13: 2 --> [94,95] --> 14,18\n"
    "Block 14: 13 --> [96,101] --> 15,16\n"
    "Block 15: 14 --> [102,103] --> 16\n"
    "Block 16: 15,14 --> [104,106] --> 17,35\n"
    "Block 17: 16 --> [107,116] --> 36\n"
    "Block 18: 13 --> [117,119] --> 19,22\n"
    "Block 19: 18 --> [120,128] --> 20,21\n"
    "Block 20: 19 --> [129,130] --> 21\n"
    "Block 21: 20,19 --> [131,138] --> 36\n"
    "Block 22: 18 --> [139,143] --> 23,24\n"
    "Block 23: 22 --> [144,147] --> 24\n"
    "Block 24: 23,22 --> [148,151] --> 25\n"
    "Block 25: 24,4 --> [152,173] --> 26,34\n"
    "Block 26: 25 --> [174,175] --> 27,33\n"
    "Block 27: 26,33,34 --> [176,212] --> 28,32\n"
    "Block 28: 27 --> [213,213] --> 29,32\n"
    "Block 29: 28 --> [214,219] --> 30,31\n"
    "Block 30: 29 --> [220,221] --> 31\n"
    "Block 31: 30,29 --> [222,222] --> 32\n"
    "Block 32: 31,27,28 --> [223,228] --> 36\n"
    "Block 33: 26 --> [229,241] --> 27\n"
    "Block 34: 25 --> [242,251] --> 27\n"
    "Block 35: 11,16 --> [252,259] --> 37\n"
    "Block 36: 6,8,12,17,21,32 --> [260,261] --> 38\n"
    "Block 37: 35 --> [262,264] --> 38\n"
    "Block 38: 37,36 --> [265,277] --> 39,49\n"
    "Block 39: 38 --> [278,279] --> 40,41\n"
    "Block 40: 39 --> [280,280] --> 61\n"
    "Block 41: 39 --> [281,282] --> 42,43\n"
    "Block 42: 41 --> [283,291] --> 72\n"
    "Block 43: 41 --> [292,294] --> 44,45\n"
    "Block 44: 43 --> [295,300] --> 72\n"
    "Block 45: 43 --> [301,307] --> 46,47\n"
    "Block 46: 45 --> [308,309] --> 47\n"
    "Block 47: 46,45 --> [310,312] --> 48,71\n"
    "Block 48: 47 --> [313,320] --> 72\n"
    "Block 49: 38 --> [321,322] --> 50,54\n"
    "Block 50: 49 --> [323,328] --> 51,52\n"
    "Block 51: 50 --> [329,330] --> 52\n"
    "Block 52: 51,50 --> [331,333] --> 53,71\n"
    "Block 53: 52 --> [334,343] --> 72\n"
    "Block 54: 49 --> [344,346] --> 55,58\n"
    "Block 55: 54 --> [347,355] --> 56,57\n"
    "Block 56: 55 --> [356,357] --> 57\n"
    "Block 57: 56,55 --> [358,365] --> 72\n"
    "Block 58: 54 --> [366,370] --> 59,60\n"
    "Block 59: 58 --> [371,374] --> 60\n"
    "Block 60: 59,58 --> [375,378] --> 61\n"
    "Block 61: 60,40 --> [379,400] --> 62,70\n"
    "Block 62: 61 --> [401,402] --> 63,69\n"
    "Block 63: 62,69,70 --> [403,439] --> 64,68\n"
    "Block 64: 63 --> [440,440] --> 65,68\n"
    "Block 65: 64 --> [441,446] --> 66,67\n"
    "Block 66: 65 --> [447,448] --> 67\n"
    "Block 67: 66,65 --> [449,449] --> 68\n"
    "Block 68: 67,63,64 --> [450,455] --> 72\n"
    "Block 69: 62 --> [456,468] --> 63\n"
    "Block 70: 61 --> [469,478] --> 63\n"
    "Block 71: 47,52 --> [479,486] --> 73\n"
    "Block 72: 42,44,48,53,57,68 --> [487,488] --> 74\n"
    "Block 73: 71 --> [489,491] --> 74\n"
    "Block 74: 73,72 --> [492,502] --> 75,76\n"
    "Block 75: 74 --> [503,505] --> 76\n"
    "Block 76: 75,74 --> [506,509] --> <none>\n"
    ;

#include "tests/rtl-disasm-test.i"
