/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/guest-ppc/insn/common.h"

static const uint8_t input[] = {
    0x10,0x22,0x19,0x14,  // ps_sum0 f1,f2,f4,f3
};

static const unsigned int guest_opt = 0;
static const unsigned int common_opt = BINREC_OPT_NATIVE_IEEE_NAN;

static const bool expected_success = true;

static const char expected[] =
    "[info] Scanning terminated at requested limit 0x3\n"
    "    0: LOAD_ARG   r1, 0\n"
    "    1: LOAD_ARG   r2, 1\n"
    "    2: GET_ALIAS  r3, a3\n"
    "    3: VEXTRACT   r4, r3, 0\n"
    "    4: GET_ALIAS  r5, a4\n"
    "    5: VEXTRACT   r6, r5, 1\n"
    "    6: GET_ALIAS  r7, a5\n"
    "    7: VEXTRACT   r8, r7, 1\n"
    "    8: FGETSTATE  r9\n"
    "    9: FCMP       r10, r8, r8, UN\n"
    "   10: FCVT       r11, r8\n"
    "   11: SET_ALIAS  a7, r11\n"
    "   12: GOTO_IF_Z  r10, L1\n"
    "   13: BITCAST    r12, r8\n"
    "   14: BFEXT      r13, r12, 51, 1\n"
    "   15: GOTO_IF_NZ r13, L1\n"
    "   16: BITCAST    r14, r11\n"
    "   17: XORI       r15, r14, 4194304\n"
    "   18: BITCAST    r16, r15\n"
    "   19: SET_ALIAS  a7, r16\n"
    "   20: LABEL      L1\n"
    "   21: GET_ALIAS  r17, a7\n"
    "   22: FSETSTATE  r9\n"
    "   23: FADD       r18, r4, r6\n"
    "   24: FCVT       r19, r18\n"
    "   25: LOAD_IMM   r20, 0x1000000\n"
    "   26: BITCAST    r21, r19\n"
    "   27: SLLI       r22, r21, 1\n"
    "   28: SEQ        r23, r22, r20\n"
    "   29: GOTO_IF_Z  r23, L2\n"
    "   30: FGETSTATE  r24\n"
    "   31: FSETROUND  r25, r24, TRUNC\n"
    "   32: FSETSTATE  r25\n"
    "   33: FADD       r26, r4, r6\n"
    "   34: FCVT       r27, r26\n"
    "   35: FGETSTATE  r28\n"
    "   36: FCOPYROUND r29, r28, r24\n"
    "   37: FSETSTATE  r29\n"
    "   38: LABEL      L2\n"
    "   39: VBUILD2    r30, r19, r17\n"
    "   40: GET_ALIAS  r31, a6\n"
    "   41: FGETSTATE  r32\n"
    "   42: FCLEAREXC  r33, r32\n"
    "   43: FSETSTATE  r33\n"
    "   44: FTESTEXC   r34, r32, INVALID\n"
    "   45: GOTO_IF_Z  r34, L4\n"
    "   46: BITCAST    r35, r4\n"
    "   47: SLLI       r36, r35, 13\n"
    "   48: BFEXT      r37, r35, 51, 12\n"
    "   49: SEQI       r38, r37, 4094\n"
    "   50: GOTO_IF_Z  r38, L6\n"
    "   51: GOTO_IF_NZ r36, L5\n"
    "   52: LABEL      L6\n"
    "   53: BITCAST    r39, r6\n"
    "   54: SLLI       r40, r39, 13\n"
    "   55: BFEXT      r41, r39, 51, 12\n"
    "   56: SEQI       r42, r41, 4094\n"
    "   57: GOTO_IF_Z  r42, L7\n"
    "   58: GOTO_IF_NZ r40, L5\n"
    "   59: LABEL      L7\n"
    "   60: NOT        r43, r31\n"
    "   61: ORI        r44, r31, 8388608\n"
    "   62: ANDI       r45, r43, 8388608\n"
    "   63: SET_ALIAS  a6, r44\n"
    "   64: GOTO_IF_Z  r45, L8\n"
    "   65: ORI        r46, r44, -2147483648\n"
    "   66: SET_ALIAS  a6, r46\n"
    "   67: LABEL      L8\n"
    "   68: GOTO       L9\n"
    "   69: LABEL      L5\n"
    "   70: NOT        r47, r31\n"
    "   71: ORI        r48, r31, 16777216\n"
    "   72: ANDI       r49, r47, 16777216\n"
    "   73: SET_ALIAS  a6, r48\n"
    "   74: GOTO_IF_Z  r49, L10\n"
    "   75: ORI        r50, r48, -2147483648\n"
    "   76: SET_ALIAS  a6, r50\n"
    "   77: LABEL      L10\n"
    "   78: LABEL      L9\n"
    "   79: ANDI       r51, r31, 128\n"
    "   80: GOTO_IF_Z  r51, L4\n"
    "   81: GET_ALIAS  r52, a6\n"
    "   82: BFEXT      r53, r52, 12, 7\n"
    "   83: ANDI       r54, r53, 31\n"
    "   84: GET_ALIAS  r55, a6\n"
    "   85: BFINS      r56, r55, r54, 12, 7\n"
    "   86: SET_ALIAS  a6, r56\n"
    "   87: GOTO       L3\n"
    "   88: LABEL      L4\n"
    "   89: FGETSTATE  r57\n"
    "   90: VFCMP      r58, r30, r30, UN\n"
    "   91: VFCVT      r59, r30\n"
    "   92: SET_ALIAS  a8, r59\n"
    "   93: GOTO_IF_Z  r58, L11\n"
    "   94: VEXTRACT   r60, r30, 0\n"
    "   95: VEXTRACT   r61, r30, 1\n"
    "   96: SLLI       r62, r58, 32\n"
    "   97: BITCAST    r63, r60\n"
    "   98: BITCAST    r64, r61\n"
    "   99: NOT        r65, r63\n"
    "  100: NOT        r66, r64\n"
    "  101: ANDI       r67, r65, 4194304\n"
    "  102: ANDI       r68, r66, 4194304\n"
    "  103: VEXTRACT   r69, r59, 0\n"
    "  104: VEXTRACT   r70, r59, 1\n"
    "  105: ZCAST      r71, r67\n"
    "  106: ZCAST      r72, r68\n"
    "  107: SLLI       r73, r71, 29\n"
    "  108: SLLI       r74, r72, 29\n"
    "  109: BITCAST    r75, r69\n"
    "  110: BITCAST    r76, r70\n"
    "  111: AND        r77, r73, r62\n"
    "  112: AND        r78, r74, r58\n"
    "  113: XOR        r79, r75, r77\n"
    "  114: XOR        r80, r76, r78\n"
    "  115: BITCAST    r81, r79\n"
    "  116: BITCAST    r82, r80\n"
    "  117: VBUILD2    r83, r81, r82\n"
    "  118: SET_ALIAS  a8, r83\n"
    "  119: LABEL      L11\n"
    "  120: GET_ALIAS  r84, a8\n"
    "  121: FSETSTATE  r57\n"
    "  122: SET_ALIAS  a2, r84\n"
    "  123: VEXTRACT   r85, r30, 0\n"
    "  124: BITCAST    r86, r85\n"
    "  125: SGTUI      r87, r86, 0\n"
    "  126: SRLI       r88, r86, 31\n"
    "  127: BFEXT      r92, r86, 23, 8\n"
    "  128: SEQI       r89, r92, 0\n"
    "  129: SEQI       r90, r92, 255\n"
    "  130: SLLI       r93, r86, 9\n"
    "  131: SEQI       r91, r93, 0\n"
    "  132: AND        r94, r89, r91\n"
    "  133: XORI       r95, r91, 1\n"
    "  134: AND        r96, r90, r95\n"
    "  135: AND        r97, r89, r87\n"
    "  136: OR         r98, r97, r96\n"
    "  137: OR         r99, r94, r96\n"
    "  138: XORI       r100, r99, 1\n"
    "  139: XORI       r101, r88, 1\n"
    "  140: AND        r102, r88, r100\n"
    "  141: AND        r103, r101, r100\n"
    "  142: SLLI       r104, r98, 4\n"
    "  143: SLLI       r105, r102, 3\n"
    "  144: SLLI       r106, r103, 2\n"
    "  145: SLLI       r107, r94, 1\n"
    "  146: OR         r108, r104, r105\n"
    "  147: OR         r109, r106, r107\n"
    "  148: OR         r110, r108, r90\n"
    "  149: OR         r111, r110, r109\n"
    "  150: FTESTEXC   r112, r32, INEXACT\n"
    "  151: SLLI       r113, r112, 5\n"
    "  152: OR         r114, r111, r113\n"
    "  153: GET_ALIAS  r115, a6\n"
    "  154: BFINS      r116, r115, r114, 12, 7\n"
    "  155: SET_ALIAS  a6, r116\n"
    "  156: GOTO_IF_Z  r112, L12\n"
    "  157: GET_ALIAS  r117, a6\n"
    "  158: NOT        r118, r117\n"
    "  159: ORI        r119, r117, 33554432\n"
    "  160: ANDI       r120, r118, 33554432\n"
    "  161: SET_ALIAS  a6, r119\n"
    "  162: GOTO_IF_Z  r120, L13\n"
    "  163: ORI        r121, r119, -2147483648\n"
    "  164: SET_ALIAS  a6, r121\n"
    "  165: LABEL      L13\n"
    "  166: LABEL      L12\n"
    "  167: FTESTEXC   r122, r32, OVERFLOW\n"
    "  168: GOTO_IF_Z  r122, L14\n"
    "  169: GET_ALIAS  r123, a6\n"
    "  170: NOT        r124, r123\n"
    "  171: ORI        r125, r123, 268435456\n"
    "  172: ANDI       r126, r124, 268435456\n"
    "  173: SET_ALIAS  a6, r125\n"
    "  174: GOTO_IF_Z  r126, L15\n"
    "  175: ORI        r127, r125, -2147483648\n"
    "  176: SET_ALIAS  a6, r127\n"
    "  177: LABEL      L15\n"
    "  178: LABEL      L14\n"
    "  179: FTESTEXC   r128, r32, UNDERFLOW\n"
    "  180: GOTO_IF_Z  r128, L3\n"
    "  181: GET_ALIAS  r129, a6\n"
    "  182: NOT        r130, r129\n"
    "  183: ORI        r131, r129, 134217728\n"
    "  184: ANDI       r132, r130, 134217728\n"
    "  185: SET_ALIAS  a6, r131\n"
    "  186: GOTO_IF_Z  r132, L16\n"
    "  187: ORI        r133, r131, -2147483648\n"
    "  188: SET_ALIAS  a6, r133\n"
    "  189: LABEL      L16\n"
    "  190: LABEL      L3\n"
    "  191: LOAD_IMM   r134, 4\n"
    "  192: SET_ALIAS  a1, r134\n"
    "  193: RETURN     r1\n"
    "\n"
    "Alias 1: int32 @ 964(r1)\n"
    "Alias 2: float64[2] @ 400(r1)\n"
    "Alias 3: float64[2] @ 416(r1)\n"
    "Alias 4: float64[2] @ 432(r1)\n"
    "Alias 5: float64[2] @ 448(r1)\n"
    "Alias 6: int32 @ 944(r1)\n"
    "Alias 7: float32, no bound storage\n"
    "Alias 8: float64[2], no bound storage\n"
    "\n"
    "Block 0: <none> --> [0,12] --> 1,3\n"
    "Block 1: 0 --> [13,15] --> 2,3\n"
    "Block 2: 1 --> [16,19] --> 3\n"
    "Block 3: 2,0,1 --> [20,29] --> 4,5\n"
    "Block 4: 3 --> [30,37] --> 5\n"
    "Block 5: 4,3 --> [38,45] --> 6,18\n"
    "Block 6: 5 --> [46,50] --> 7,8\n"
    "Block 7: 6 --> [51,51] --> 8,13\n"
    "Block 8: 7,6 --> [52,57] --> 9,10\n"
    "Block 9: 8 --> [58,58] --> 10,13\n"
    "Block 10: 9,8 --> [59,64] --> 11,12\n"
    "Block 11: 10 --> [65,66] --> 12\n"
    "Block 12: 11,10 --> [67,68] --> 16\n"
    "Block 13: 7,9 --> [69,74] --> 14,15\n"
    "Block 14: 13 --> [75,76] --> 15\n"
    "Block 15: 14,13 --> [77,77] --> 16\n"
    "Block 16: 15,12 --> [78,80] --> 17,18\n"
    "Block 17: 16 --> [81,87] --> 32\n"
    "Block 18: 5,16 --> [88,93] --> 19,20\n"
    "Block 19: 18 --> [94,118] --> 20\n"
    "Block 20: 19,18 --> [119,156] --> 21,24\n"
    "Block 21: 20 --> [157,162] --> 22,23\n"
    "Block 22: 21 --> [163,164] --> 23\n"
    "Block 23: 22,21 --> [165,165] --> 24\n"
    "Block 24: 23,20 --> [166,168] --> 25,28\n"
    "Block 25: 24 --> [169,174] --> 26,27\n"
    "Block 26: 25 --> [175,176] --> 27\n"
    "Block 27: 26,25 --> [177,177] --> 28\n"
    "Block 28: 27,24 --> [178,180] --> 29,32\n"
    "Block 29: 28 --> [181,186] --> 30,31\n"
    "Block 30: 29 --> [187,188] --> 31\n"
    "Block 31: 30,29 --> [189,189] --> 32\n"
    "Block 32: 31,17,28 --> [190,193] --> <none>\n"
    ;

#include "tests/rtl-disasm-test.i"
