/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/common.h"
#include "tests/host-x86/common.h"


static const binrec_setup_t setup = {
    .host = BINREC_ARCH_X86_64_SYSV,
};
static const unsigned int host_opt = 0;

static int add_rtl(RTLUnit *unit)
{
    int reg1, reg2, reg3, reg4;
    EXPECT(reg1 = rtl_alloc_register(unit, RTLTYPE_FLOAT32));
    EXPECT(rtl_add_insn(unit, RTLOP_LOAD_IMM, reg1, 0, 0, 0x3F800000));
    EXPECT(reg2 = rtl_alloc_register(unit, RTLTYPE_FLOAT64));
    EXPECT(rtl_add_insn(unit, RTLOP_LOAD_IMM,
                        reg2, 0, 0, UINT64_C(0x3FF0000000000000)));
    EXPECT(reg3 = rtl_alloc_register(unit, RTLTYPE_V2_FLOAT32));
    EXPECT(rtl_add_insn(unit, RTLOP_VBROADCAST, reg3, reg1, 0, 0));
    EXPECT(reg4 = rtl_alloc_register(unit, RTLTYPE_V2_FLOAT64));
    EXPECT(rtl_add_insn(unit, RTLOP_VBROADCAST, reg4, reg2, 0, 0));

    int reg5, reg6, reg7, reg8;
    EXPECT(reg5 = rtl_alloc_register(unit, RTLTYPE_FLOAT32));
    EXPECT(rtl_add_insn(unit, RTLOP_FNEG, reg5, reg1, 0, 0));
    EXPECT(reg6 = rtl_alloc_register(unit, RTLTYPE_FLOAT64));
    EXPECT(rtl_add_insn(unit, RTLOP_FNEG, reg6, reg2, 0, 0));
    EXPECT(reg7 = rtl_alloc_register(unit, RTLTYPE_V2_FLOAT32));
    EXPECT(rtl_add_insn(unit, RTLOP_FNEG, reg7, reg3, 0, 0));
    EXPECT(reg8 = rtl_alloc_register(unit, RTLTYPE_V2_FLOAT64));
    EXPECT(rtl_add_insn(unit, RTLOP_FNEG, reg8, reg4, 0, 0));

    int reg9, reg10, reg11, reg12;
    EXPECT(reg9 = rtl_alloc_register(unit, RTLTYPE_FLOAT32));
    EXPECT(rtl_add_insn(unit, RTLOP_FABS, reg9, reg5, 0, 0));
    EXPECT(reg10 = rtl_alloc_register(unit, RTLTYPE_FLOAT64));
    EXPECT(rtl_add_insn(unit, RTLOP_FABS, reg10, reg6, 0, 0));
    EXPECT(reg11 = rtl_alloc_register(unit, RTLTYPE_V2_FLOAT32));
    EXPECT(rtl_add_insn(unit, RTLOP_FABS, reg11, reg7, 0, 0));
    EXPECT(reg12 = rtl_alloc_register(unit, RTLTYPE_V2_FLOAT64));
    EXPECT(rtl_add_insn(unit, RTLOP_FABS, reg12, reg8, 0, 0));

    return EXIT_SUCCESS;
}

static const uint8_t expected_code[] = {
    0x48,0x83,0xEC,0x08,                // sub $8,%rsp
    0xE9,0x87,0x00,0x00,0x00,           // jmp 0x90
    0x00,0x00,0x00,0x00,0x00,0x00,0x00, // (padding)

    0x00,0x00,0x00,0x80,0x00,0x00,0x00,0x00, // (data)
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // (data)
    0xFF,0xFF,0xFF,0x7F,0x00,0x00,0x00,0x00, // (data)
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // (data)
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80, // (data)
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // (data)
    0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x7F, // (data)
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // (data)
    0x00,0x00,0x00,0x80,0x00,0x00,0x00,0x80, // (data)
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // (data)
    0xFF,0xFF,0xFF,0x7F,0xFF,0xFF,0xFF,0x7F, // (data)
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // (data)
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80, // (data)
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80, // (data)
    0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x7F, // (data)
    0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x7F, // (data)

    0xB8,0x00,0x00,0x80,0x3F,           // mov $0x3F800000,%eax
    0x66,0x0F,0x6E,0xC0,                // movd %eax,%xmm0
    0x48,0xB8,0x00,0x00,0x00,0x00,0x00, // mov $0x3FF0000000000000,%rax
      0x00,0xF0,0x3F,
    0x66,0x48,0x0F,0x6E,0xC8,           // movq %rax,%xmm1
    0x0F,0x28,0xD0,                     // movaps %xmm0,%xmm2
    0x0F,0x14,0xD2,                     // unpcklps %xmm2,%xmm2
    0xF3,0x0F,0x7E,0xD2,                // movq %xmm2,%xmm2
    0xF2,0x0F,0x12,0xD9,                // movddup %xmm1,%xmm3
    0x0F,0x57,0x05,0x53,0xFF,0xFF,0xFF, // xorps -173(%rip),%xmm0
    0x0F,0x57,0x0D,0x6C,0xFF,0xFF,0xFF, // xorps -148(%rip),%xmm1
    0x0F,0x57,0x15,0x85,0xFF,0xFF,0xFF, // xorps -123(%rip),%xmm2
    0x0F,0x57,0x1D,0x9E,0xFF,0xFF,0xFF, // xorps -98(%rip),%xmm3
    0x0F,0x54,0x05,0x47,0xFF,0xFF,0xFF, // andps -185(%rip),%xmm0
    0x0F,0x54,0x0D,0x60,0xFF,0xFF,0xFF, // andps -160(%rip),%xmm1
    0x0F,0x54,0x15,0x79,0xFF,0xFF,0xFF, // andps -135(%rip),%xmm2
    0x0F,0x54,0x1D,0x92,0xFF,0xFF,0xFF, // andps -110(%rip),%xmm3
    0x48,0x83,0xC4,0x08,                // add $8,%rsp
    0xC3,                               // ret
};

static const char expected_log[] = "";

#include "tests/rtl-translate-test.i"
