/*
 * libbinrec: a recompiling translator for machine code
 * Copyright (c) 2016 Andrew Church <achurch@achurch.org>
 *
 * This software may be copied and redistributed under certain conditions;
 * see the file "COPYING" in the source code distribution for details.
 * NO WARRANTY is provided with this software.
 */

#include "tests/common.h"
#include "tests/host-x86/common.h"


static const binrec_setup_t setup = {
    .host = BINREC_ARCH_X86_64_SYSV,
    .host_features = BINREC_FEATURE_X86_MOVBE,
};
static const unsigned int host_opt = BINREC_OPT_H_X86_MERGE_REGS;

static int add_rtl(RTLUnit *unit)
{
    int alias[15], merge_src[15];
    for (int i = 0; i < lenof(alias); i++) {
        EXPECT(alias[i] = rtl_alloc_alias_register(unit, RTLTYPE_FLOAT32));
        EXPECT(merge_src[i] = rtl_alloc_register(unit, RTLTYPE_FLOAT32));
        EXPECT(rtl_add_insn(unit, RTLOP_LOAD_IMM, merge_src[i], 0, 0, i+1));
        EXPECT(rtl_add_insn(unit, RTLOP_SET_ALIAS,
                            0, merge_src[i], 0, alias[i]));
    }

    int reg1, label;
    EXPECT(label = rtl_alloc_label(unit));
    EXPECT(rtl_add_insn(unit, RTLOP_LABEL, 0, 0, 0, label));
    /* The register allocator should cancel one of the merges to make
     * room for this register. */
    EXPECT(reg1 = rtl_alloc_register(unit, RTLTYPE_FLOAT32));
    EXPECT(rtl_add_insn(unit, RTLOP_LOAD_IMM, reg1, 0, 0, 101));
    for (int i = 0; i < lenof(alias); i++) {
        int merge_dest;
        EXPECT(merge_dest = rtl_alloc_register(unit, RTLTYPE_FLOAT32));
        EXPECT(rtl_add_insn(unit, RTLOP_GET_ALIAS,
                            merge_dest, 0, 0, alias[i]));
        EXPECT(rtl_add_insn(unit, RTLOP_SET_ALIAS,
                            0, merge_dest, 0, alias[i]));
    }

    return EXIT_SUCCESS;
}

static const uint8_t expected_code[] = {
    0x48,0x83,0xEC,0x48,                // sub $72,%rsp
    0xB8,0x01,0x00,0x00,0x00,           // mov $1,%eax
    0x66,0x0F,0x6E,0xC0,                // movd %eax,%xmm0
    0xF3,0x0F,0x11,0x04,0x24,           // movss %xmm0,(%rsp)
    0xB8,0x02,0x00,0x00,0x00,           // mov $2,%eax
    0x66,0x0F,0x6E,0xC8,                // movd %eax,%xmm1
    0xB8,0x03,0x00,0x00,0x00,           // mov $3,%eax
    0x66,0x0F,0x6E,0xD0,                // movd %eax,%xmm2
    0xB8,0x04,0x00,0x00,0x00,           // mov $4,%eax
    0x66,0x0F,0x6E,0xD8,                // movd %eax,%xmm3
    0xB8,0x05,0x00,0x00,0x00,           // mov $5,%eax
    0x66,0x0F,0x6E,0xE0,                // movd %eax,%xmm4
    0xB8,0x06,0x00,0x00,0x00,           // mov $6,%eax
    0x66,0x0F,0x6E,0xE8,                // movd %eax,%xmm5
    0xB8,0x07,0x00,0x00,0x00,           // mov $7,%eax
    0x66,0x0F,0x6E,0xF0,                // movd %eax,%xmm6
    0xB8,0x08,0x00,0x00,0x00,           // mov $8,%eax
    0x66,0x0F,0x6E,0xF8,                // movd %eax,%xmm7
    0xB8,0x09,0x00,0x00,0x00,           // mov $9,%eax
    0x66,0x44,0x0F,0x6E,0xC0,           // movd %eax,%xmm8
    0xB8,0x0A,0x00,0x00,0x00,           // mov $10,%eax
    0x66,0x44,0x0F,0x6E,0xC8,           // movd %eax,%xmm9
    0xB8,0x0B,0x00,0x00,0x00,           // mov $11,%eax
    0x66,0x44,0x0F,0x6E,0xD0,           // movd %eax,%xmm10
    0xB8,0x0C,0x00,0x00,0x00,           // mov $12,%eax
    0x66,0x44,0x0F,0x6E,0xD8,           // movd %eax,%xmm11
    0xB8,0x0D,0x00,0x00,0x00,           // mov $13,%eax
    0x66,0x44,0x0F,0x6E,0xE0,           // movd %eax,%xmm12
    0xB8,0x0E,0x00,0x00,0x00,           // mov $14,%eax
    0x66,0x44,0x0F,0x6E,0xE8,           // movd %eax,%xmm13
    0xB8,0x0F,0x00,0x00,0x00,           // mov $15,%eax
    0x66,0x44,0x0F,0x6E,0xF0,           // movd %eax,%xmm14
    0xB8,0x65,0x00,0x00,0x00,           // mov $101,%eax
    0x66,0x0F,0x6E,0xC0,                // movd %eax,%xmm0
    0xF3,0x0F,0x10,0x04,0x24,           // movss (%rsp),%xmm0
    0xF3,0x0F,0x11,0x04,0x24,           // movss %xmm0,(%rsp)
    0xF3,0x0F,0x11,0x4C,0x24,0x04,      // movss %xmm1,4(%rsp)
    0xF3,0x0F,0x11,0x54,0x24,0x08,      // movss %xmm2,8(%rsp)
    0xF3,0x0F,0x11,0x5C,0x24,0x0C,      // movss %xmm3,12(%rsp)
    0xF3,0x0F,0x11,0x64,0x24,0x10,      // movss %xmm4,16(%rsp)
    0xF3,0x0F,0x11,0x6C,0x24,0x14,      // movss %xmm5,20(%rsp)
    0xF3,0x0F,0x11,0x74,0x24,0x18,      // movss %xmm6,24(%rsp)
    0xF3,0x0F,0x11,0x7C,0x24,0x1C,      // movss %xmm7,28(%rsp)
    0xF3,0x44,0x0F,0x11,0x44,0x24,0x20, // movss %xmm8,32(%rsp)
    0xF3,0x44,0x0F,0x11,0x4C,0x24,0x24, // movss %xmm9,36(%rsp)
    0xF3,0x44,0x0F,0x11,0x54,0x24,0x28, // movss %xmm10,40(%rsp)
    0xF3,0x44,0x0F,0x11,0x5C,0x24,0x2C, // movss %xmm11,44(%rsp)
    0xF3,0x44,0x0F,0x11,0x64,0x24,0x30, // movss %xmm12,48(%rsp)
    0xF3,0x44,0x0F,0x11,0x6C,0x24,0x34, // movss %xmm13,52(%rsp)
    0xF3,0x44,0x0F,0x11,0x74,0x24,0x38, // movss %xmm14,56(%rsp)
    0x48,0x83,0xC4,0x48,                // add $72,%rsp
    0xC3,                               // ret
};

static const char expected_log[] = "";

#include "tests/rtl-translate-test.i"
